/*******************************************************************************

Description: An animation class that holds 1 animation

Authors: Clay Smith

Liscense: See <a href="http://www.opensource.org/licenses/zlib-license.php">zlib/libpng license</a>

*******************************************************************************/

module arc.gfx.animation;

private import std.math; 

private import 	arc.gfx.entity,
				arc.gfx.prim,
				arc.gfx.texture,
				arc.gfx.frame.frame,
				arc.gfx.frame.boxframe,
				arc.gfx.frame.radframe,
				arc.io.sound,
				arc.phy.staticcollision,
				arc.phy.point;

private import derelict.opengl.gl,
               derelict.sdl.sdl,
               derelict.sdl.image;

debug import arc.phy.phyutil;

import std.string,
       std.stdio;

/// provides animations and encapulates frame collision
class Animation 
{
  public:

	this() 
	{
		frame.length = 0;
	}

	// cleanup
	~this()  
	{
		delete frame;
	}

	/// add a frame to the animation
	Point addFrame(	char[] img, int time, Sound snd, 
					COLLISION_TYPE col, float argGap)
	{
		// grow frame length
		frame.length = frame.length+1;
		int i = frame.length-1;

		// extract size from frames after they load
		Point size;

		if (col == COLLISION_TYPE.BOX)
		{
			frame[i] = new BoxFrame(img, time, snd, size);
		}
		else if (col == COLLISION_TYPE.RADIUS)
		{
			frame[i] = new RadFrame(img, time, snd, size);
		}
		else
		{
			arc.io.log.write("animation", "addFrame", "error", "invalid collision type");
		}

		return size;
	}


	/// add a frame to the animation
	Point addFrame(	Texture tex, int time, Sound snd, 
					COLLISION_TYPE col, float argGap)
	{
		// grow frame length
		frame.length = frame.length+1;
		int i = frame.length-1;

		// extract size from frames after they load
		Point size;

		if (col == COLLISION_TYPE.BOX)
		{
			frame[i] = new BoxFrame(tex, time, snd, size);
		}
		else if (col == COLLISION_TYPE.RADIUS)
		{
			frame[i] = new RadFrame(tex, time, snd, size);
		}
		else
		{
			arc.io.log.write("animation", "addFrame", "error", "invalid collision type");
		}

		return size;
	}

	// reload per pixel frames with new width and height
	void reload(double zoomX, double zoomY, float argG)
	{
		foreach (Frame f; frame)
			f.reload(zoomX, zoomY, argG);
	}

	/// draw frame 
	void draw(float x, float y, float width, float height, 
				short r, short g, short b, short a, 
				float pivX, float pivY, float angle, float argGap,
				float nx, float ny,
				COLLISION_TYPE col)
	{	
		drawImage(x, y, width, height, r, g, b, a, pivX, pivY, angle, frame[curr].getID);
	}

	/// process pivots, call every frame
	void process()
	{
		frame[curr].process();

		// time of less than 0 will be frozen
		if (frame[curr].time > 0)
		{
			// calculate time elapsed between calls
			prevTime = currTime;
			currTime = SDL_GetTicks();

			// add it to the amount of time elapsed so far
			elapsed += currTime-prevTime;

			// check to see if enough time elapsed to move to next frame
			if (elapsed >= frame[curr].time)
			{
				// play snd if there is one
				if (frame[curr].snd !is null)
				{
					frame[curr].snd.stop(); 
					frame[curr].snd.play();
				}
				
				// we reached the end of the frame
				if (curr == frame.length-1)
				{
					// loop back to beginning if specified
					if (loop)
						curr = 0;
					// otherwise just stay here
				}
				else
				{
					curr++;
				}
				
				// reset
				elapsed = 0;
			}
		}

	}

	Frame getCurrFrame() 
	{
		return frame[curr]; 
	}

	int getCurr() { return curr; }

	void setLoop(bool v)
	{
		loop = v;
	}

	Sound getSound(int argNum)
	in
	{
		assert (argNum <= frame.length);
	}
	body
	{
		return frame[argNum].snd;
	}

	void setSound(Sound snd, int argNum)
	in
	{
		assert (argNum <= frame.length);
	}
	body
	{
		// remove old sound if its there
		if (frame[argNum].snd !is null)
			delete frame[argNum].snd;

		frame[argNum].snd = snd;
	}

	int getTime(int aNum)
	{
		return frame[aNum].time;
	}

	void setTime(int aT, int aNum)
	{
		frame[aNum].time = aT; 
	}

	void reset() 
	{ 
		curr = 0; 
		currTime=SDL_GetTicks();
		prevTime = currTime; 
		//elapsed=0;
	}

	bool isLoop() { return loop; }

  private:  
	// animation information
	Frame[] frame;
	int curr = 0;
	uint prevTime=0, currTime=0, elapsed = 0;
	bool loop = true;
}


