/*******************************************************************************

   Frame with radius collision detection
     
   Date: (2005): initial version
   
   Authors: Clay Smith
   
   Liscense: 
      See <a href="http://www.opensource.org/licenses/zlib-license.php">zlib/libpng license</a>

Examples:
-------------------------------------------------------------------------------
import arc.gfx.frame.radframe,
       arc.io.input;

int main() {

   // radius based collision frame with zoom of 3.0
   Frame radframe = new RadFrame("img.png",3.0);    

   return 0;
}
-------------------------------------------------------------------------------
*******************************************************************************/
module arc.gfx.frame.radframe; 

public import arc.gfx.frame.frame;

private import std.math; 

private import arc.gfx.gfxutil, 
               arc.gfx.texture,
			   arc.gfx.prim,
			   arc.io.sound,
               arc.io.window,
               arc.phy.point,
               arc.phy.staticcollision; 

private import derelict.opengl.gl,
               derelict.sdl.sdl,
               derelict.sdl.image;

debug import std.stdio;

/// Frame with radius based collision detection
class RadFrame : Frame
{
  public:

	this() {}

	// radius is assumed to be (width+height)/2
	this(char[] argFullFileName, int argTime, Sound argSnd, inout Point s)
	{
		s = init(argFullFileName, argTime, argSnd);
	}

	// radius is assumed to be (width+height)/2
	this(Texture tex, int argTime, Sound argSnd, inout Point s)
	{
		s = init(tex, argTime, argSnd);
	}

	Point init(char[] argFullFileName, int argTime, Sound argSnd)
	{
		// will crash if file doesn't exist
		assert (std.file.exists(argFullFileName));

		// turn image into an OpenGL texture and extract width and height 
		Texture tex = arc.gfx.texture.texture(argFullFileName);

		ID = tex.getID; 

		time = argTime;
		snd = argSnd;

		Point size = Point(tex.getWidth, tex.getHeight);
		return size;
	}

	Point init(Texture tex, int argTime, Sound argSnd)
	{
		ID = tex.getID; 

		time = argTime;
		snd = argSnd;

		Point size = Point(tex.getWidth, tex.getHeight);
		return size;
	}

   ~this()  
   {
   }

	int getID() { return ID; }


  private:
   int ID;
}
