/*******************************************************************************

Description: An animation class that holds 1 animation

Authors: Clay Smith

Liscense: See <a href="http://www.opensource.org/licenses/zlib-license.php">zlib/libpng license</a>

*******************************************************************************/

module arc.gui.guisprite.animation;

import std.math; 

import 	arc.io.log, 
		arc.gfx.entity,
		arc.gfx.prim,
		arc.gfx.texture,
		arc.gui.guisprite.frame,
		arc.io.sound,
		arc.phy.point,
		arc.phy.time;

import derelict.opengl.gl,
       derelict.sdl.sdl;

import 	std.string;

/// provides animations and encapulates frame collision
class Animation 
{
  public:

	this() 
	{
		frame.length = 0;
	}

	// cleanup
	~this()  
	{
		delete frame;
	}

	/// Add frame to animation 
	void addFrame(Texture tex, int time, Sound snd)
	{
		// grow frame length
		frame.length = frame.length+1;
		// set current frame 
		frame[frame.length-1] = new Frame(tex, time, snd);
	}

	/// draw frame from top left corner, good for drawing tiles 
	void draw(Point pos, Point size, ushort r, ushort g, ushort b, ushort a)
	{
		drawImageTopLeft(pos.x, pos.y, size.x, size.y, r,g,b,a, frame[curr].getID);
	}

	/// process pivots, call every frame
	void process(Point pos, Point zoom)
	{		
		// time of less than 0 will be frozen
		if (frame[curr].getTime > 0)
		{
			// calculate time elapsed between calls
			prevTime = currTime;
			currTime = arc.phy.time.getTime();

			// add it to the amount of time elapsed so far
			elapsed += (currTime-prevTime);

			// check to see if enough time elapsed to move to next frame
			if (elapsed >= frame[curr].getTime)
			{
				// play snd if there is one
				if (frame[curr].getSound !is null)
				{
					frame[curr].getSound.stop();
					frame[curr].getSound.play();
				}
				
				// we reached the end of the frame
				if (curr == frame.length-1)
				{
					// loop back to beginning if specified
					if (loop)
					{
						curr = 0;
					}
					// otherwise just stay here
				}
				else
				{
					curr++;
				}
				
				// reset
				elapsed = 0;
			}
		}
	}

	/// Get current frame from animation 
	Frame getCurrFrame() 
	{
		return frame[curr]; 
	}

	/// get current animation 
	int getCurr() { return curr; }

	/// set loop 
	void setLoop(bool v)
	{
		loop = v;
	}

	/// get sound from animation 
	Sound getSound(int argNum)
	in
	{
		assert (argNum <= frame.length);
	}
	body
	{
		return frame[argNum].getSound;
	}

	/// set sound in animation 
	void setSound(Sound snd, int argNum)
	in
	{
		assert (argNum <= frame.length);
	}
	body
	{
		frame[argNum].removeSound(); 
		frame[argNum].setSound(snd); 
	}

	/// get time 
	int getTime(int argNum)
	{
		return frame[argNum].getTime;
	}

	/// set time 
	void setTime(int aT, int aNum)
	{
		frame[aNum].setTime(aT); 
	}

	/// reset animation 
	void reset() 
	{ 
		curr = 0; 
		currTime=arc.phy.time.getTime();
		prevTime = currTime; 
	}

	/// return true if animation is looping
	bool isLoop() { return loop; }

  private:  
	// animation information
	Frame[] frame;
	int curr = 0;
	uint prevTime=0, currTime=0, elapsed = 0;
	bool loop = true;
}


