/*******************************************************************************

   Frame with polygon collision detection
     
   Date: (2006): initial version
   
   Authors: Clay Smith
   
   Liscense: 
      See <a href="http://www.opensource.org/licenses/zlib-license.php">zlib/libpng license</a>

Examples:
-------------------------------------------------------------------------------
import arc.gfx.frame.radframe,
       arc.io.input;

int main() {

	Frame f = new Frame(tex, time, sound, polygon); 

	return 0;
}
-------------------------------------------------------------------------------
*******************************************************************************/
module arc.gui.guisprite.frame; 

private import std.math; 

private import 	arc.gfx.texture,
				arc.io.sound, 
				arc.phy.point;
				
import derelict.opengl.gl; 

debug private import std.stdio;

/// Frame with radius based collision detection
class Frame
{
  public:
	/// Load frame based on texture, time, sound, and polygon 
	this(Texture tex, int argTime, Sound argSnd)
	{
		init(tex, argTime, argSnd);
	}

	/// Initialize frame based on texture, time, sound, and polygon 
	void init(Texture tex, int argTime, Sound argSnd)
	{
		ID = tex.getID; 

		time = argTime;
		sound = argSnd;
	}

	/// get time 
	int getTime() { return time; }

	/// set time 
	void setTime(int argT) { time = argT; } 

	/// get ID amount 
	uint getID() { return ID; }

	/// set ID 
	void setID(uint argID) { ID = argID; }

	/// get sound amount 
	Sound getSound() { return sound; } 

	/// set sound 
	void setSound(Sound argS) { sound = argS; }

	/// remove sound if it exists 
	void removeSound() { if (sound !is null) delete sound; }

  private:
	int time; 
	uint ID;
	Sound sound; 
}
