﻿/*******************************************************************************

Description: Polygon collision, made for GUI code only  

Authors: Clay Smith

Liscense: See <a href="http://www.opensource.org/licenses/zlib-license.php">zlib/libpng license</a>

*******************************************************************************/

module arc.gui.guisprite.polygon; 

import 	arc.phy.point, 
		arc.phy.triangle,
		arc.phy.phyutil, 
		arc.phy.staticcollision,
		arc.gfx.prim, 
		std.stdarg,
		std.math; 

import derelict.opengl.gl;

import std.stdio;

/// A single polygon created from a list of points 
struct Polygon 
{
	/// transform points to the given pivot, angle, zoom, and position 
	void transformPoints(Point position, Point zoom)
	{
		pos = origPos * zoom;
		//pos = pos.getRotation(pivot, angle); 
		pos += position; 
		
		for(int i = 0; i < points.length; i++)
		{
			points[i] = origPoints[i] * zoom;
			//points[i] = points[i].getRotation(pivot, angle); 
			points[i] += position; 

		} 
	}

	/// create a circle with radius and specified detail 
	void createCircle(float circRadius, int detail)
	{
		origPoints.length=0;

		Point p; 
		
		for (int i = 0; i < 360; i+=360.0/detail)
		{
			p = Point(circRadius, i); 
			p.polarToRect(); 
			
			origPoints.length = origPoints.length+1;
			origPoints[origPoints.length-1] = p + circRadius; 
		}

		points.length = origPoints.length;
	}

	/// create box based upon width and height  
	void createBox(float width, float height)
	{
		origPoints.length=4;
		points.length=4;
		origPoints[0] = Point(0,0); 
		origPoints[1] = Point(width, 0); 
		origPoints[2] = Point(width,height);
		origPoints[3] = Point(0,height); 
	}

	/// draw polygon shape 
	void draw(ushort r, ushort g, ushort b, ushort a)
	{
		glDisable(GL_TEXTURE_2D); 
		glColor4ub(r,g,b,a);
	
		glBegin(GL_LINE_LOOP);

		foreach(Point v; points)
		{
			glVertex2f(v.getX, v.getY); 
		}

		glEnd();
		glEnable(GL_TEXTURE_2D); 
	}

	/// print all points in polygon, for debug purposes 
	void print(char[] str="")
	{		
		foreach(Point v; points)
		{
			writef(str); 
			v.print(); 
		}
	}

	/// test if polygon collides with point x, y
	bool collide(float mouseX, float mouseY)
	{
		return polygonPointCollision(Point(mouseX, mouseY), points);
	}

	/// test if polygon collides with another polygon
	bool collide(inout Polygon o)
	{
		// return true when lines overlap 
		if (polygonLineCollisionTest(points, o.getPoints()))
			return true;

		// 2) If line test doesn't work, check to see if one of the others 
		//    points is inside of it
		foreach(Point p; points)
		{
			if (polygonPointCollision(p, o.getPoints))
				return true;
		}

		foreach(Point p; o.getPoints)
		{
			if (polygonPointCollision(p, points))
				return true;
		}

		return false; 
	}

	// Test on pair of lines against the others 
	bool polygonLineCollisionTest(Point[] poly1, Point[] poly2)
	{
		// indexing vars to keep track of which line it is 
		int i, j;
		int k, l; 
		Point overlap;
		
		// i will loop from 0 to poly.length-1 
		for (i = 0, j = 1; i < poly1.length; i++, j++)
		{
			if (j == poly1.length)
				j = 0;
			
			for (k = 0, l = 1; k < poly2.length; k++, l++)
			{
				if (l == poly2.length)
					l = 0;

				// return true when two lines collide with each other 
				if (lineLineCollision(poly1[i],
										poly1[j], 
										poly2[k], 
										poly2[l], overlap))
					return true; 
			}
	
		}

		return false; 
	}

	/// remove all data from polygon 
	void destroy()
	{
		origPoints.length=0;
		points.length=0;
		origPos = pos = Point(0,0); 
	}

	/// get list of all points in polygon 
	Point[] getPoints() { return points; }

	Point[] origPoints; 
	Point[] points;

	// position of the polygon 
	Point origPos, pos; 
}


