module arc.gui.layout; 

import arc.gui.readxml;

import 	std.stream;

import 	arc.gui.widgets.widget,
		arc.gui.widgets.image,
		arc.gui.widgets.label,
		arc.gui.widgets.button,
		arc.gui.widgets.textfield,
		arc.gui.guisprite.guisprite,
		arc.gfx.prim,
		arc.io.sound; 

import std.stdio;

class Layout : Widget
{
  public:
	this(char[] fileName)
	{
		sprite = new GUISprite(AUTOPOLY.BOX);
		
		LayoutData layout = readLayoutXMLData(fileName); 
	
		sprite.setPosition(layout.x, layout.y);
		sprite.setSize(layout.width, layout.height);
		
		name = layout.name; 

		// read skin data for the layout 
		skin = readSkinXMLData(layout.skin); 

		// add all images to the layout 
		foreach(ImageData image; layout.images)
		{
			widgets.length = widgets.length+1;
			int curr = widgets.length-1; 
						
			widgets[curr] = new Image(sprite, image.x, image.y, image.width, image.height, image.color); 

			// Check Bounds, Assert if not in current layouts bounds /////
			float currXBound = widgets[curr].getX + widgets[curr].getWidth;
			float currYBound = widgets[curr].getY + widgets[curr].getHeight;
			char[] currName = image.name; 

			if (currXBound > (layout.x + layout.width))
			{
				writefln("layout: image widget ", currName, " out of X bounds");
				if (widgets[curr].getX < layout.x)
				{
					writefln(widgets[curr].getX, " is less than ", layout.x); 
				}
				else if (layout.x || currXBound > (layout.x + layout.width))
				{
					writefln("x is too big"); 
				}
				assert(0);
			}

			if (currYBound > (layout.y + layout.height))
			{	
				writefln("layout: image widget ", currName, " out of Y bounds");
				assert(0);
			}


			foreach(ImageFrameData frame; image.frames)
			{
				Sound snd;// = new Sound(new SoundFile(frame.soundfx)); 
				if (frame.soundfx == "null")
				{
					snd = null; 
				}
				else
				{
					snd = new Sound(new SoundFile(frame.soundfx)); 
				}
				
				widgets[curr].addFrame(frame.image, frame.animation,  frame.speed, snd); 
			}
		}

		// add all labels to the layout 
		foreach(LabelData label; layout.labels)
		{
			widgets.length = widgets.length+1;
			int curr = widgets.length-1;
			widgets[curr] = new Label(sprite, skin, label.x, label.y, label.fontheight, label.padding, label.name, label.text, label.color, label.fontcolor); 

			// Check Bounds, Assert if not in current layouts bounds /////
			float currXBound = widgets[curr].getX + widgets[curr].getWidth;
			float currYBound = widgets[curr].getY + widgets[curr].getHeight;
			char[] currName = label.name; 

			if (widgets[curr].getX < layout.x || currXBound > (layout.x + layout.width))
			{
				writefln("layout: label widget ", currName, " out of X bounds");
				assert(0);
			}

			if (widgets[curr].getY < layout.y || currYBound > (layout.y + layout.height))
			{	
				writefln("layout: label widget ", currName, " out of Y bounds");
				assert(0);
			}

		}

		// add all buttons to the layout 
		foreach(ButtonData button; layout.buttons)
		{
			widgets.length = widgets.length+1;
			int curr = widgets.length-1;
			widgets[curr] = new Button(sprite, skin, button.x, button.y, button.fontheight, button.padding, button.name, button.text, button.color, button.fontcolor); 

			// Check Bounds, Assert if not in current layouts bounds /////
			float currXBound = widgets[curr].getX + widgets[curr].getWidth;
			float currYBound = widgets[curr].getY + widgets[curr].getHeight;
			char[] currName = button.name; 

			if (widgets[curr].getX < layout.x || currXBound > (layout.x + layout.width))
			{
				writefln("layout: button widget ", currName, " out of X bounds");
				if (widgets[curr].getX < layout.x)
					writefln("too small"); 
				if (currXBound > (layout.x + layout.width))
					writefln("Too big ", currXBound);
				writefln(widgets[curr].getX, " - ", widgets[curr].getWidth);
				
				assert(0);
			}

			if (widgets[curr].getY < layout.y || currYBound > (layout.y + layout.height))
			{	
				writefln("layout: button widget ", currName, " out of Y bounds");
				assert(0);
			}

		}


		// add all textfields to the layout 
		foreach(TextfieldData textfield; layout.textfields)
		{
			// Add New Widget 
			widgets.length = widgets.length+1;
			int curr = widgets.length-1;

			widgets[curr] = new TextField(sprite, skin, textfield.x, textfield.y,textfield.maxwidth,textfield.fontheight, textfield.padding,textfield.maxlines, textfield.name, textfield.text, textfield.color, textfield.fontcolor); 

			// Check Bounds, Assert if not in current layouts bounds /////
			float currXBound = widgets[curr].getX + widgets[curr].getWidth;
			float currYBound = widgets[curr].getY + widgets[curr].getHeight;
			char[] currName = textfield.name; 

			if (widgets[curr].getX < layout.x || currXBound > (layout.x + layout.width))
			{
				writefln("layout: textfield widget ", currName, " out of X bounds");
				assert(0);
			}

			if (widgets[curr].getY < layout.y || currYBound > (layout.y + layout.height))
			{	
				writefln("layout: textfield widget ", currName, " out of Y bounds");
				assert(0);
			}

		}

	}

	void drawBounds(ushort r, ushort g, ushort b, ushort a)
	{
		drawRect(sprite.getX, sprite.getY, sprite.getWidth, sprite.getHeight, r,g,b,a,false);
	}

	void draw()
	{
		foreach(Widget w; widgets)
			w.draw(sprite); 
	}

	void process()
	{
		foreach(Widget w; widgets)
		{
			w.process(); 
		}
	}

	~this()
	{
		delete skin; 
	}

  private:
    	Widget[] widgets;

		// layout skin being used and the name of the layout 
		char[] name;
		// skin for the current layout 
		Skin skin; 
		
}