module arc.gui.widgets.button;

import std.stream, std.conv; 

import 	arc.gui.widgets.widget,
		arc.gui.guisprite.polygon,
		arc.gfx.font,
		arc.gui.guisprite.guisprite,
		arc.io.sound,
		arc.io.input,
		arc.io.log; 

// skin data structure 
import arc.gui.readxml; 
import arc.gfx.texture; 




debug import std.stdio;

class Button : Widget 
{
  public: 

	this(AUTOPOLY pType)
	{
		init(pType); 
	}

	this(Polygon p)
	{
		init(p);
	}
  
	this(GUISprite s, Skin skin, float argX, float argY, uint fontheight, uint argP, char[] argName, char[] argText, char[] argColor, char[] argFontColor)
	{
		init(AUTOPOLY.BOX); 

		setPadding(argP);

	
		setParentPosition(x,y); 

		// create font with name and height 
		font = new Font(skin.getFont, fontheight); 

		//text = argText; 
		setText(argText); 


		// draw text to calculate width of font 
		font.draw(text,x,y,0,0,0,0); 

		setSize(font.getWidth+argP, font.getHeight+argP); 
		setChildPosition(x+sprite.getX, y+sprite.getY);

		// extract color from color string that looks like "r;g;b;a"
		char[][] splitColor = std.string.split(argColor, ";");
		setColor( 	toUshort(splitColor[0]),
					toUshort(splitColor[1]),
					toUshort(splitColor[2]),
					toUshort(splitColor[3]));

		// extract color from color string that looks like "r;g;b;a"
		char[][] splitFontColor = std.string.split(argFontColor, ";");
		
		setFontColor( 	toUshort(splitFontColor[0]),
					toUshort(splitFontColor[1]),
					toUshort(splitFontColor[2]),
					toUshort(splitFontColor[3]));

		// get frame data for the label widget from the skin 
		FrameData[][char[]] multiframes = skin.getWidgetFrames("button");

		// the key is the animations 
		foreach(char[] key; multiframes.keys)
		{
			// each animation holds a number of frames 
			foreach(FrameData frame; multiframes[key])
			{
				// create a sound effect if it exists 
				SoundFile f = null; 
				Sound snd = null; 
				if (std.file.exists(frame.soundfx))
				{
					f = new SoundFile(frame.soundfx); 
					snd = new Sound(f);
				}

//				writefln("add frame ", frame.image);

				sprite.addFrame(frame.image, key, frame.time, snd);
			}
		}

		sprite.setAnim("default"); 

	}

	void draw(GUISprite s)
	{
		sprite.setPosition(x+s.getX, y+s.getY);
		
		sprite.draw(); 
		font.draw(text, sprite.getX+halfPadding, sprite.getY+halfPadding, fr,fg,fb,fa); 
	}

	void draw()
	{
		sprite.draw(); 
		sprite.drawBounds(255,255,255,255);
		font.draw(text, sprite.getX+halfPadding, sprite.getY+halfPadding, fr,fg,fb,fa); 
	}

	/// set font color 
	void setFontColor(ushort r, ushort g, ushort b, ushort a )
	{
		fr = r; fg = g; fb = b; fa = a; 
	}

	/// set text of widget 
	void setText(char[] argText)
	{
		text = argText; 

		if (font !is null)
		{
			setSize(font.getTextWidth(text)+(halfPadding*2), font.getHeight+(halfPadding*2)); 
		}
	}

	/// set padding amount on label 
	void setPadding(float argP)
	{
		halfPadding = cast(uint)argP/2;
		setSize(font.getTextWidth(text)+argP, font.getHeight+argP); 
	}

	/// set font and set widget size correctly
	void setFont(Font argFont)
	{
		font = argFont; 
		setSize(font.getTextWidth(text)+(halfPadding*2), font.getHeight+(halfPadding*2)); 
	}

  private: 
	Font font; 
	char[] name; 
	char[] text; 
	ushort fr=255, fg=255, fb=255, fa=255; 
	uint halfPadding; 
	
}