module arc.gui.widgets.label;

import std.stream, std.conv; 

import 	arc.gui.widgets.widget,
		arc.gfx.font,
		arc.gfx.texture,
		arc.gui.guisprite.polygon,
		arc.gui.guisprite.guisprite,
		arc.io.sound,
		arc.io.input; 

// skin data structure 
import arc.gui.readxml; 

import std.stdio;

class Label : Widget 
{
  public: 
  
	this(AUTOPOLY pType)
	{
		init(pType); 
	}

	this(Polygon p)
	{
		init(p);
	}
  
	this(GUISprite s, Skin skin, float argX, float argY, uint fontheight, uint argP, char[] argName, char[] argText, char[] argColor, char[] argFontColor)
	{
		//sprite = new GUISprite(AUTOPOLY.BOX);
		init(AUTOPOLY.BOX); 

		//halfPadding = argP/2;
		setPadding(argP); 

		//x=argX;
		//y=argY;
		setParentPosition(x,y); 

		// create font with name and height 

		//font = new Font(skin.getFont, fontheight); 
		setFont(new Font(skin.getFont, fontheight)); 

		//text = argText; 
		setText(argText); 

		// draw text to calculate width of font 
		font.draw(text,x,y,0,0,0,0); 

		// set sprite to calculated font width and height 
		sprite.setSize(font.getWidth+argP, font.getHeight+argP); 
		sprite.setPosition(x+s.getX, y+s.getY);

		// extract color from color string that looks like "r;g;b;a"
		char[][] splitColor = std.string.split(argColor, ";");
		setColor( 	toUshort(splitColor[0]),
					toUshort(splitColor[1]),
					toUshort(splitColor[2]),
					toUshort(splitColor[3]));

		// extract color from color string that looks like "r;g;b;a"
		char[][] splitFontColor = std.string.split(argFontColor, ";");		
		setFontColor( 	toUshort(splitFontColor[0]),
						toUshort(splitFontColor[1]),
						toUshort(splitFontColor[2]),
						toUshort(splitFontColor[3]));		

		// get frame data for the label widget from the skin 
		FrameData[][char[]] multiframes = skin.getWidgetFrames("label");

		// the key is the animations 
		foreach(char[] key; multiframes.keys)
		{
			// each animation holds a number of frames 
			foreach(FrameData frame; multiframes[key])
			{
				// create a sound effect if it exists 
				SoundFile f = null; 
				Sound snd = null; 
				if (std.file.exists(frame.soundfx))
				{
					f = new SoundFile(frame.soundfx); 
					snd = new Sound(f);
				}

				sprite.addFrame(frame.image, key, frame.time, snd);
			}
		}

		sprite.setAnim("default"); 

	}

	/// draw label based on sprite position 
	void draw(GUISprite s)
	{
		sprite.setPosition(x+s.getX, y+s.getY); 
		sprite.draw(); 
		font.draw(text, sprite.getX+halfPadding, sprite.getY+halfPadding, fr,fg,fb,fa); 
	}

	/// draw label 
	void draw()
	in 
	{
		// catch ugly assert errors 
		assert(font !is null); 
	}
	body 
	{
		sprite.draw(); 
		font.draw(text, sprite.getX+halfPadding, sprite.getY+halfPadding, fr,fg,fb,fa);	
	}

	/// set font color 
	void setFontColor(ushort r, ushort g, ushort b, ushort a )
	{
		fr = r; fg = g; fb = b; fa = a; 
	}

	/// set text of widget 
	void setText(char[] argText)
	{
		text = argText; 

		if (font !is null)
		{
			setSize(font.getTextWidth(text)+(halfPadding*2), font.getHeight+(halfPadding*2)); 
		}
	}

	/// set padding amount on label 
	void setPadding(float argP)
	{
		halfPadding = cast(uint)argP/2;
		setSize(font.getTextWidth(text)+argP, font.getHeight+argP); 
	}

	/// set font and set widget size correctly
	void setFont(Font argFont)
	{
		font = argFont; 
		setSize(font.getTextWidth(text)+(halfPadding*2), font.getHeight+(halfPadding*2)); 
	}

  private: 
	Font font; 
	char[] text; 
	ushort fr=255, fg=255, fb=255, fa=255; 
	uint halfPadding; 
}