module arc.gui.widgets.textfield;

import std.stream, std.conv, std.string; 

import 	arc.gui.widgets.widget,
		arc.gfx.font,
		arc.gfx.texture,
		arc.gui.guisprite.guisprite,
		arc.gfx.prim, 
		arc.io.sound,
		arc.gui.guisprite.polygon,
		arc.phy.time,
		arc.phy.point,
		arc.io.input,
		arc.io.log; 

// skin data structure 
import arc.gui.readxml; 

debug import std.stdio;

class TextField : Widget 
{
  public: 
	this(AUTOPOLY pType)
	{
		init(pType); 
		blink = new Blinker; 
	}

	this(Polygon p)
	{
		init(p);
		blink = new Blinker; 
	}
  
	this(GUISprite s, Skin skin, float argX, float argY, uint argMaxWidth, uint fontheight, uint argP, uint argMaxLines, char[] argName, char[] argText, char[] argColor, char[] argFontColor)
	{
		maxWidth = argMaxWidth;
		maxLines = argMaxLines; 
		
		//sprite = new GUISprite(AUTOPOLY.BOX);
		init(AUTOPOLY.BOX); 

		//halfPadding = argP/2;
		setPadding(argP);

		//x = argX;
		//y = argY;
		setParentPosition(x, y); 

		// create font with name and height 
		font = new Font(skin.getFont, fontheight); 
		blink = new Blinker; 

		text = argText; 
		index = text.length; 
		
		// draw text to calculate width of font 
		font.draw(text,x,y,0,0,0,0); 

		// set sprite to calculated font width and height 
		sprite.setSize(maxWidth+argP, (font.getHeight*maxLines)+argP); 
		//sprite.setPosition(x+s.getX, y+s.getY);
		setChildPosition(x,y); 

		// extract color from color string that looks like "r;g;b;a"
		char[][] splitColor = std.string.split(argColor, ";");
		setColor( 	toUshort(splitColor[0]),
							toUshort(splitColor[1]),
							toUshort(splitColor[2]),
							toUshort(splitColor[3]));

		// extract color from color string that looks like "r;g;b;a"
		char[][] splitFontColor = std.string.split(argFontColor, ";");
		
		setColor( 	toUshort(splitFontColor[0]),
					toUshort(splitFontColor[1]),
					toUshort(splitFontColor[2]),
					toUshort(splitFontColor[3]));

		// get frame data for the label widget from the skin 
		FrameData[][char[]] multiframes = skin.getWidgetFrames("textfield");

		// the key is the animations 
		foreach(char[] key; multiframes.keys)
		{
			// each animation holds a number of frames 
			foreach(FrameData frame; multiframes[key])
			{
				// create a sound effect if it exists 
				SoundFile f = null; 
				Sound snd = null; 
				if (std.file.exists(frame.soundfx))
				{
					f = new SoundFile(frame.soundfx); 
					snd = new Sound(f);
				}

				sprite.addFrame(frame.image, key, frame.time, snd);
			}
		}

		sprite.setAnim("default"); 
	}

	void draw(GUISprite s)
	{
		sprite.setPosition(x+s.getX, y+s.getY);
		sprite.draw(); 
	}

	void draw()
	{
		sprite.draw(); 
	}

	void process()
	in
	{
		assert(blink !is null); 
	}
	body 
	{
		super.process();

		// INPUT BOX PROCESSING ////////////
		// blink every 1 second
		blink.process(.5); 

		if (focus)
		{
			// delete keys 
			if (arc.io.input.keyHit(ARC_DELETE) || arc.io.input.keyHit(ARC_BACKSPACE))
			{
				if (text.length > 0)
				{
					// in middle of string 
					if (index < text.length && index >= 0)
					{
						char[] newstr = text.dup; 

						if (arc.io.input.keyHit(ARC_DELETE))
						{
							text = newstr[0 .. index];
							text ~= newstr[index+1 .. newstr.length];	
	//							writefln("\ndel ", index);
						}
						else if (arc.io.input.keyDown(ARC_BACKSPACE))
						{
							if (index >= 1)
							{
								text = newstr[0 .. index-1];
								text ~= newstr[index .. newstr.length];			
								index--; 
		//						writefln("\nmid back ", index); 
							}
						}
					}
					// at the end of the string 
					else if (!arc.io.input.keyHit(ARC_DELETE))
					{

						text = text[0 .. text.length-1];
						index--;
						
						
			//			writefln("\nback ", index); 
					}

				//	writefln(index); 
				//	writefln(text); 
				}
				//writefln("Index is ", index, " and str length is ", text.length); 
				//writefln(text); 
			}
			// add to string
			else if (arc.io.input.charHit && font.getTextWidth(text ~ "|") < maxWidth-halfPadding*2)
			{
				// if user hits 'return' 
				if (std.string.find(arc.io.input.lastChars, '\r') != -1)
				{
					int numlines=0;

					foreach(char c; text)
						if (c == '\r' || c == '\n')
							numlines++; 

					//writefln("Num lines are ", numlines, " for text ", text);

					// it is okay to add this line
					if (numlines < maxLines)
					{
						//writefln("Index is ", index, " and str length is ", text.length); 
						if (index == text.length)
						{
							text ~= arc.io.input.lastChars; 
							index += arc.io.input.lastChars.length; 
							//writefln(index); 
						}
						else
						{
							text = insert(text, index, arc.io.input.lastChars); 
							index += arc.io.input.lastChars.length; 
							//writefln(index); 
						}

					}
				}
				else
				{
					//writefln("Index is ", index, " and str length is ", text.length); 
					if (index == text.length)
					{
						text ~= arc.io.input.lastChars; 
						index += arc.io.input.lastChars.length; 
						//writefln(index); 
					}
					else
					{
						text = insert(text, index, arc.io.input.lastChars); 
						index += arc.io.input.lastChars.length; 
						//writefln(index); 
					}
				}
				
			//	writefln(text); 
			}

			if (arc.io.input.keyHit(ARC_LEFT))
			{
				if (index > 0)
				{
					index--;
					writefln(index); 
				}
			}

			
			if (arc.io.input.keyHit(ARC_RIGHT))
			{
				if (index < text.length)
				{
					index++;
					writefln(index);
				}
			}
		}

		if (blink.on)
		{
			showcursor = !showcursor; 
		}

		//writefln("Showcursor is ", showcursor); 
		//writefln("Focus is ", focus); 

		if (showcursor && focus)
		{
			char[] cursortext = insert(text, index, "|");
			font.draw(cursortext, sprite.getX + halfPadding, sprite.getY + halfPadding,fr,fg,fb,fa);
		}
		else
		{
			font.draw(text, sprite.getX + halfPadding, sprite.getY + halfPadding,fr,fg,fb,fa); 
		}

		if (mouseOver && arc.io.input.mouseHit(LEFT))
		{
			focus = true; 
		}
		else if (arc.io.input.mouseHit(LEFT))
		{
			focus = false; 
		}

		// mouse click
		if (arc.io.input.mouseHit(LEFT))
		{
			if (showcursor)
			{
				char[] cursortext = insert(text, index, "|");
				index = font.calculateIndex(cursortext, sprite.getX + halfPadding, sprite.getY + halfPadding, arc.io.input.mouseX, arc.io.input.mouseY);
//				writefln("Calculate index is ", index); 
			}
			else
				index = font.calculateIndex(text, sprite.getX + halfPadding, sprite.getY + halfPadding, arc.io.input.mouseX, arc.io.input.mouseY);

			if (index > text.length)
				index = text.length;
			if (index == 0) index = text.length;
		}

	}

	/// set font color 
	void setFontColor(ushort r, ushort g, ushort b, ushort a )
	{
		fr = r; fg = g; fb = b; fa = a; 
	}

	/// set text of widget 
	void setText(char[] argText)
	{
		text = argText; 

		if (font !is null)
		{
			setSize(font.getTextWidth(text)+(halfPadding*2), font.getHeight+(halfPadding*2)); 
		}
	}

	/// set padding amount on label 
	void setPadding(float argP)
	{
		halfPadding = cast(uint)argP/2;
		setSize(font.getTextWidth(text)+argP, font.getHeight+argP); 
	}

	/// set font and set widget size correctly
	void setFont(Font argFont)
	{
		font = argFont; 
		setSize(font.getTextWidth(text)+(halfPadding*2), font.getHeight+(halfPadding*2)); 
	}


  private: 
	Font font; 
	char[] text; 
	ushort fr=255, fg=255, fb=255, fa=255; 
	uint halfPadding=10, maxWidth, numLines=1, maxLines=1; 	

	Blinker blink; 
	bool showcursor = false; 
	bool focus = false; 
	int index; 
}