module arc.gui.widgets.widget; 

// import this to fix import conflicts 
import std.stream;

import arc.gui.readxml; 

import 	arc.gui.guisprite.guisprite,
		arc.gui.guisprite.polygon,
		arc.gfx.texture,
		arc.gfx.font,
		arc.io.sound,
		arc.io.input; 

import std.signals; 

class Widget
{
  public:
	this()
	{
	}

	/// initialize with polygon type 
	void init(AUTOPOLY pType)
	{
		sprite = new GUISprite(pType); 
	}

	/// initialize with the exact polygon 
	void init(Polygon p)
	{
		sprite = new GUISprite(p); 
	}

	/// set name of the widget 
	/// set size of the widget 
	void setSize(float argWidth, float argHeight)
	{
		sprite.setSize(argWidth, argHeight); 
	}

	/// set parent position of sprite 
	void setParentPosition(float argX, float argY)
	{
		x = argX;
		y = argY; 
	}

	/// set child position of sprite 
	void setChildPosition(float argX, float argY)
	{
		sprite.setPosition(argX, argY);
	}

	/// set color of the widget 
	void setColor(short r, short g, short b, short a)
	{
		sprite.setColor(r,g,b,a); 
	}

	/// add frame for image
	void addFrame(char[] argImage, char[] argAnim, int argSpeed, Sound snd)
	in 
	{
		assert(sprite !is null);
	}
	body
	{
		sprite.addFrame(argImage, argAnim, argSpeed, snd); 
	}

	/// draw using the location of another sprite 
	void draw(GUISprite s)
	{
		// draw sprite from top left location
		sprite.setPosition(x+s.getX, y+s.getY);
		sprite.draw(); 
	}

	/// draw using the location of another sprite 
	void draw()
	{
		// draw sprite
		sprite.draw(); 
	}

	

	/// process widget 
	void process()
	{
		sprite.setAnim("default");
	
		if (sprite.mouseOver && arc.io.input.mouseDown(LEFT))
		{
			sprite.setAnim("click");
			focus = true; 
		}
		else if (arc.io.input.mouseDown(LEFT))
		{
			sprite.setAnim("unclick");
			focus = false; 
		}
		else if (sprite.mouseOver)
		{
			sprite.setAnim("hover"); 
		}
	
		if (focus)
			sprite.setAnim("focus");

		sprite.process(); 
	}

	/// get width 
	float getWidth() {  return sprite.getWidth; }

	/// get height 
	float getHeight() { return sprite.getHeight; } 

	/// get x position 
	float getX() { return sprite.getX; }

	/// get Y position 
	float getY() { return sprite.getY; }

	bool mouseOver() { return sprite.mouseOver; } 

	/// make widget a signal, which will emit signals which the GUI 
	/// observer class will take care of 
    mixin Signal!(char[]);


/// These are not implemented but are virtual functions that other classes will use
	void setFontColor(ushort r, ushort g, ushort b, ushort a )
	{
		assert(0);
	}

	void setText(char[] argText)
	{
		assert(0);
	}

	void setPadding(float argP)
	{
		assert(0);
	}

	void setFont(Font argFont)
	{
		assert(0);
	}


  protected:
	GUISprite sprite; 

	// will hold the position relative to the layout
	float x=0, y=0; 
	bool focus=false;
}