﻿module arc.io.camera; 

/**

		

*/

import arc.io.window; 
import arc.phy.point;
import derelict.opengl.gl; 

class Camera
{
  public: 
	/// update camera-space mouse coordinates
	void process()
	{
		// transform mouse pos
		Point normalized_mouse = Point(arc.io.input.mouseX / arc.io.window.getWidth, arc.io.input.mouseY / arc.io.window.getHeight);
		mousePos = normalized_to_camera(normalized_mouse);
		
		// get view area
		viewPos = normalized_to_camera(Point(0,0));
		viewSize = normalized_to_camera(Point(1,1)) - viewPos;
	}
		
	/// move
	void move(Point v)
	{
		pos += v;
	}
	
	void setPosition(Point p)
	{
		pos = p;
	}

	Point getPosition() { return pos; }
	Point getMousePos() { return mousePos; }

	/// set zoom amount 
	void setZoom(float argZoom)
	{
		zoom = argZoom; 
	}

	/// get zoom amount 
	float getZoom() { return zoom; } 

	/// set angle amount
	void setAngle(float argAngle)
	{
		angle = argAngle; 
	}

	/// get angle amount 
	float getAngle() { return angle; } 

	/// Set GL projection settings with glOrtho 
	void setProjection(float argLeft, float argRight, float argBottom, float argTop)
	{
		left = argLeft;
		right = argRight;
		bottom = argBottom;
		top = argTop; 
	}

	/// start viewing transformations 
	void open() 
	{
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();

		float width = right - left;
		float height = bottom - top;
		float mid_x = (left + right) / 2.;
		float mid_y = (top + bottom) / 2.;
		glOrtho(mid_x - zoom * width / 2., mid_x + zoom * width / 2., mid_y + zoom * height / 2., mid_y - zoom * height / 2., -1.0f, 1.0f); 

		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();

		glDisable(GL_DEPTH_TEST);     
		glEnable(GL_TEXTURE_2D);
		
		glPushMatrix();

		// rotate 
		glRotatef(angle, 0,0,1);

		// translate to position (can be center of screen) 
		glTranslatef(-pos.getX, -pos.getY, 0);
	}
	
	// transforms a point
	Point camera_to_normalized(Point from)
	{
		Point extends = Point(right - left, bottom - top) * zoom;
		Point mid = Point((left + right) / 2., (top + bottom) / 2.);
		
		from -= pos;
		from.rotate(angle);
		from -= mid;
		from = from / extends;
		from += Point(0.5, 0.5);
		
		return from;
	}
	
	// un-transforms a point
	Point normalized_to_camera(Point from)
	{
		Point extends = Point(right - left, bottom - top) * zoom;
		Point mid = Point((left + right) / 2., (top + bottom) / 2.);
		
		from -= Point(0.5, 0.5);		
		from = from * extends;
		from += mid;
		from.rotate(-angle);
		from += pos;
		
		return from;
	}

	/// end viewing transformations 
	void close() 
	{
		glPopMatrix(); 
	}
	
	Point getViewPos()
	{
		return viewPos;
	}
	
	Point getViewSize()
	{
		return viewSize;
	}
	
  private:
	Point pos;
	float zoom = 1;
	float angle = 0;
	float left, right, bottom, top; 
	
  // the mouse position in this camera's coordinate system, updated through process()
  Point mousePos;
	
	//BUG: Does not work with rotation
	// the lower left point of the view area
	Point viewPos;
	// the size of the view area
	Point viewSize;
}