module arc.io.freelist; 

/** A class to allow fast allocations / de-allocations, using a free list.
 *  This can be on the order of 40-80x faster than new / delete.
 *  Inherit from this class and instantiate it as the same type as the class it's used in.
 *  Ex:  class Vector : FreeList!(Vector) { }*/
class FreeList(T)
{
	// Used for fast allocations / frees.
    private static T freelist;		// points to the end of the free list
    private T next;

	/** Used to quickly allocate a type T. (only fast if there's one waiting in the free list).
	 *  Just remember to call free() on it when finished (not delete!).
	 *  Also keep in mind that the new type T will contain memory garbage and needs to be set.*/
    static T allocate()
    {   T hello;
		if (freelist)
		{   hello = freelist;
			freelist = hello.next;
		}else hello = new T();
		return hello;
    }

    /// Place this type T back on a free list for the next allocate.
    static void free(T goodbye)
    {	goodbye.next = freelist;
		freelist = goodbye;
    }

	static void free(T goodbye1, T goodbye2)
    {	free(goodbye1);
		free(goodbye2);
    }

	static void free(T goodbye1, T goodbye2, T goodbye3)
    {	free(goodbye1);
		free(goodbye2);
		free(goodbye3);
    }
}