/*******************************************************************************

   Helpful mathmatical / physics utilities
     
   Date: (2005): initial version
   
   Authors: Clay Smith
   
   Liscense: See <a href="http://www.opensource.org/licenses/zlib-license.php">zlib/libpng license</a>

Examples:
-------------------------------------------------------------------------------
import arc.phy.collisionmap;

int main() {
   
   CollisionMap map = new CollisionMap("img.png"); 
   CollisionMap map2 = new CollisionMap("img2.png");

   if (map.perPixelCollision(map2))
   {
      // we have per pixel collision between the two images
   }

   map2.setPosition(50,50);
   map.draw();

   return 0;
}
-------------------------------------------------------------------------------
*******************************************************************************/

module arc.phy.phyutil; 

import 	std.random,
		std.stdio,
		std.math;  

import arc.phy.point; 

/// precalculated PI / 180 Degrees value
const real DEGREE_TO_RADIAN = PI / 180; 
/// precalculated 180 / PI value
const real RADIAN_TO_DEGREE = 180 / PI; 
/// Epsilon 
const float EPSILON=0.0000000001f;

/*******************************************************************************

   See if number is in range of target, rangle is variable

*******************************************************************************/

// use w/ caution
bool withinRange(real argNum, real argTarget, real argRange)
{
	if (argNum <= (argTarget+argRange) && argNum >= (argTarget-argRange))
		return true;

	return false;
}

/// wrap angle to stay within range of 0-360
void wrapAngle(inout float argAngle)
{
	while (argAngle > 360)
		argAngle = argAngle-360;

	while (argAngle < 0)
		argAngle = 360+argAngle;
}

/*******************************************************************************

   Calculate the distance between 2 points

*******************************************************************************/
real distance(real x1, real y1, real x2, real y2)
   out(result)
   {
      if (result < 0)
      {
         writefln("arc.phy.phyutil: error - result less than zero");
         assert(result < 0); 
      }
   }
   body
   {
      return sqrt(pow(x2-x1, 2) + pow(y2-y1, 2));
   }

/*******************************************************************************

   Returns the next power of 2

*******************************************************************************/
int nextPowerOf2( int a )
{
	int rval=1;
	while(rval<a) rval<<=1;
	return rval;
}

/*******************************************************************************

   Returns a random integer between a and b

*******************************************************************************/
int random_range(real a, real b)
   in
   {
      assert( a <= b );
   }
   out (result)
   {
      assert (result >= a && result <= b+1); 
   }
   body
   {
      return cast(int)(a + (std.random.rand() % (b+1-a) ));
   }



/// Finds the roots
bool findRoots(float a, float b, float c, inout float t0, inout float t1)
{
	float d = b*b - (4.0f * a * c);

	if (d < 0.0f)
		return false;

	d = cast(float) sqrt(d);

	float one_over_two_a = 1.0f / (2.0f * a);

	t0 = (-b - d) * one_over_two_a;
	t1 = (-b + d) * one_over_two_a;

	if (t1 < t0)
	{
		float t = t0;
		t0 = t1;
		t1 = t;
	}
	return true;
}


/// area of a polygon 
float area(Point[] contour)
{
  int n = contour.length;

  float A=0.0f;

  for(int p=n-1,q=0; q<n; p=q++)
  {
	A+= contour[p].getX*contour[q].getY - contour[q].getX*contour[p].getY;
  }

  return A*0.5f;
}

/// Max distance of a given point from a given set of points 
float maxDistance(Point given, Point[] set)
{
	float max = 0; 
	float tmp = 0; 

	foreach(Point p; set)
	{
		// measure distance 
		tmp = given.distance(p); 

		// if greater than current max point, then make this distance the maximum 
		if (tmp > max)
			max = tmp; 
	}

	return max; 
}

/// returns the max of two uints 
uint max(uint a, uint b)
{
	if (a > b)
		return a;
	return b; 
}


unittest
{
   writefln("util: unit test begin");

   for (int i = 0; i < 10; i++)
   {
      random_range(random_range(1,10), random_range(11, 20));
   }

   writefln("util: unit test end");
}
