﻿module classes.dockinfo;

private import init.init;
private import classes.fu_object;
private import classes.item;
private import classes.ship;
private import classes.job;

static const int MAX_SCALE = 35; // 35%
static const ulong TIME_CHUNK = 60 * 30; // 30 seconds

class FU_DOCKINFO {
private:
	char[] statusText;
	ulong lastDock;
	FU_OBJECT dockHost;
	FU_SHIP dockClient;
	int backgroundTexID;
	int underAttackCount = 0;
public:
	// some message (e.g. not enough money to buy)
	char[] message;
	// current docking mode
	int currentDockMode;
	bool detailedView;
	float[] priceScale;
	// current selection in dock (can be casted to ship, job etc.)
	FU_ITEM itemSelection;
	FU_SHIP shipSelection;
	FU_JOB jobSelection;
	dlinkedlist!(FU_ITEM) sellItems;
	dlinkedlist!(FU_SHIP) sellShips;
	dlinkedlist!(FU_JOB) jobs;
	this(FU_OBJECT host) {
		priceScale = new float[ITEM_CARGO.max+1];
		sellItems = new dlinkedlist!(FU_ITEM);
		sellShips = new dlinkedlist!(FU_SHIP);
		jobs = new dlinkedlist!(FU_JOB);
		switch( host.getType() ) {
		case OBJECT_TYPE.PLANET:
			getTexture(TEXTURES.DOCKEDPLANET, -1, &backgroundTexID);
			break;
		case OBJECT_TYPE.STATION:
			getTexture(TEXTURES.DOCKEDSTATION, -1, &backgroundTexID);
			break;
		}
		lastDock = 0;
		for(int i=0;i<priceScale.length;i++) priceScale[i] = 1.0;
	}
	// default constructible, in order to be easily serializable
	this() {}
	~this() {}
	void signalAttack() {
		underAttackCount = 4;
	}
	bool underAttack() {
		if( underAttackCount > 0 ) underAttackCount--;
		return (underAttackCount != 0);
	}
	void describe(T)(T ar) {
		ar.describe(backgroundTexID);
		ar.describe(priceScale);
		ar.describe(sellItems);
		ar.describe(sellShips);
		ar.describe(jobs);
		ar.describe(lastDock);
		if( ar.reading && lastDock != 0 ) lastDock = 1;
	}
	void resetControl() {
		detailedView = false;
		itemSelection = null;
		jobSelection = null;
		shipSelection = null;
		message = "None.";
	}
	int getBGID() { return backgroundTexID; }
	FU_OBJECT getHost() { return dockHost; }
	FU_SHIP getClient() { return dockClient; }
	void setStatus(char[] newText) { statusText = newText; }
	char[] getStatus() { return statusText; }
	void dockInit(FU_OBJECT host, FU_SHIP client) {
		debug scope(failure) { writefln("dockInit: dockinfo.d"); }	
		// first update dynamic valuable economy
		// based on time, specific to each dockable
		ulong timeDiff = (globalTimer - lastDock) / TIME_CHUNK;
		resetControl();
		currentDockMode = DOCK_MODE.REPAIR;
		dockClient = client;
		dockHost = host;
		statusText = "";
		// make sure we create stuff and set price flux the first time
		if( lastDock == 0 ) timeDiff = MAX_SCALE;
		if( timeDiff > 0 ) {
			if( timeDiff > MAX_SCALE ) timeDiff = MAX_SCALE;
			float maxFlux = cast(float)MAX_SCALE / 100.0;
			for(int i=0;i<priceScale.length;i++) {
				float priceFlux = cast(float)((rand()%(cast(long)timeDiff*2))-cast(long)timeDiff) / 100.0;
				priceScale[i] += priceFlux;
				if( priceScale[i] > 1.0 + maxFlux ) priceScale[i] = 1.0 + maxFlux;
				else if( priceScale[i] < 1.0 - maxFlux ) priceScale[i] = 1.0 - maxFlux;
			}
			// wait long enough, item in stock change
			if( timeDiff >= 3 ) {
				sellItems.clear();
				sellShips.clear();
				jobs.clear();
				int newJobs, newShips, newUtilities, newCargo, newWeapons, i;
				if( host.getType() == OBJECT_TYPE.PLANET ) {
					newShips = rand()%4+3;
					newUtilities = rand()%5+8;
					newCargo = rand()%4+4;
					newWeapons = rand()%5+8;
					newJobs = rand()%4+4;
				} else {
					newShips = 0;
					newUtilities = rand()%3+4;
					newCargo = rand()%2+2;
					newWeapons = rand()%3+4;
					newJobs = rand()%3+2;
				}
				// use thie boolean to check whether items were actually created
				bool newOK;
				// add ships (if any)
				for(i=0;i<newShips;i++) {
					FU_SHIP newShip = new FU_SHIP(rand()%(SHIP_TYPE.max+1),host.getLevel(),false);
					sellShips.add(newShip);
				}
				// add utilities
				for(i=0;i<newUtilities;i++) {
					FU_ITEM newItem = new FU_ITEM(ITEM_TYPE.UTILITY,rand()%25,host.getLevel(),&newOK,host.getFaction());
					if( newOK == true ) sellItems.add(newItem);
					else i--;
				}
				// add weapons
				for(i=0;i<newWeapons;i++) {
					FU_ITEM newItem = new FU_ITEM(ITEM_TYPE.WEAPON,rand()%25+25,host.getLevel(),&newOK,host.getFaction());
					if( newOK == true ) sellItems.add(newItem);
					else i--;
				}
				// add cargo
				for(i=0;i<newCargo;i++) {
					FU_ITEM newItem = new FU_ITEM(ITEM_TYPE.CARGO,rand()%25+50,host.getLevel(),&newOK,host.getFaction());
					if( newOK == true && newItem.getClass() != ITEM_CLASS.CONTRABAND &&
										 newItem.getClass() != ITEM_CLASS.UNIQUE ) sellItems.add(newItem);
					else i--;
				}
				// add jobs
				for(i=0;i<newJobs;i++) {
					FU_JOB newJob = new FU_JOB();
					int objectives = rand()%3 + 1;
					for(int j=0;j<objectives;j++) {
						newJob.generateObjective(rand()%(OBJECTIVE_TYPE.max+1),host.getLevel(),host.getFaction());
					}
					newJob.calculateInfo();
					jobs.add(newJob);
				}
			}
		}
		lastDock = globalTimer;
}
};
