﻿module classes.fu_object;

private import init.init;
private import classes.item;
private import classes.dockinfo;
private import classes.ship;
private import classes.particle;
import arc.phy.point;

// angle maths
float pointAtAngle(double X, double Y, double targetX, double targetY) {
	return 180 - (atan2( targetX-X, targetY-Y ) * 57.295785);
}
double moveToValueX(float ang) { return (sin((180-ang) * 0.0174520069808)); }
double moveToValueY(float ang) { return (cos((180-ang) * 0.0174520069808)); }
float angleDistance(float ang1, float ang2) {
	if( ang1 > 360 ) ang1 -= 360;
	if( ang1 < 0 ) ang1 += 360;
	if( ang2 > 360 ) ang2 -= 360;
	if( ang2 < 0 ) ang2 += 360;
	float retval = fabs( ang1 - ang2 );
	if( retval > 180 ) retval = 360 - retval;
	return retval;
}

// everything is an FU_OBJECT
class FU_OBJECT {
private:
	// standard issue information
	int level;
	int[3] nameIndex;
	int factionIndex;
	int object_type;
	// standard variables
	float rotation_speed;
	float rotation;
	// "health" statistics
	// when armor reaches 0, kill object
	float armor, max_armor;
	// color offset
	short r, g, b, a;
	// shield drawing alpha (not need to be saved)
	short ShieldAlpha;
	// actual sprite for object
	Sprite graphic;
	int TextureClass, TextureValue;
	// item for OBJ_ITEM type
	FU_ITEM item;
	// dockable info for stations/planets
	FU_DOCKINFO dockInfo;
	// link destination, used for telepaths/telegates
	double[2] x_dest, y_dest;
	// last connection angle from telepath
	// (does not need to be saved)
	float[] teleAngle;
	// does this object experience space friction?
	bool friction;
	// was this a quest object that needs to be trimmed?
	ubyte trimMe = 0;
protected:
	// position in space, speeds, rotation
	double x, y, x_speed, y_speed;
	// weight of object
	// total_weight is weight + anything else (e.g. items)
	float weight;
	// value of object
	long value;
	// max object speed
	double max_speed;
	void explode() {
		playSound(SOUND_CLASS.BIG_EXPLOSION,getX(),getY());
		float popSize = graphic.getWidth();
		if( graphic.getHeight() > popSize ) popSize = graphic.getHeight();
		popSize /= 2;
		if( popSize < 30 ) popSize = 30;
		for(int i=0;i<6;i++) {
			FU_PARTICLE pop = FU_PARTICLE.allocate();
			pop.construct(this,200+rand()%56,30+rand()%40,10+rand()%20,popSize);
			float adjXSpeed = (cast(float)(rand()%20)-10.0) / 7.0;
			float adjYSpeed = (cast(float)(rand()%20)-10.0) / 7.0;
			pop.setSpeed(adjXSpeed - x_speed, adjYSpeed - y_speed);
		}
		if( PlayerShip.getCurrentJob() !is null ) PlayerShip.getCurrentJob().handleAction(this,null,JOB_ACTION.KILL);
		if( object_type == OBJECT_TYPE.ROCK ) {
			MainGenerator.spawnRock();
			if( rand()%10<4 ) {
				FU_ITEM newValue = new FU_ITEM(ITEM_TYPE.CARGO, ITEM_CLASS.VALUABLE, (rand()%level)+1,null,factionIndex);
				newValue.getVars().counter = random_range(1,3);
				MainGenerator.spawnItem(x, y, newValue);
			}
		} else if( object_type == OBJECT_TYPE.SHIP ) {
			FU_SHIP deadShip = cast(FU_SHIP)this;
			deadShip.items.reset();
			while(!deadShip.items.last()) {
				float reduceQuality = cast(float)random_range(20,40) / 100.0;
				deadShip.items.data.getVars().quality -= reduceQuality;
				if( deadShip.items.data.getVars().quality > 0 ) {
					deadShip.items.data.forceUninstall();
					FU_OBJECT newObj = MainGenerator.spawnItem(x, y, deadShip.items.data);
					newObj.setXSpeed(cast(float)random_range(-20,20) / 70.0);
					newObj.setYSpeed(cast(float)random_range(-20,20) / 70.0);
				}
			}
		}
	}
	void drawObject() {
		graphic.setPosition(x, y);
		if( graphic.onScreen() == true ) {
			graphic.setFastAngle(rotation);
			graphic.setColor(r, g, b, a);
			graphic.draw();
			if( ShieldAlpha > 0 ) {
				ShieldAlpha -= 8;
				drawCircle(x, y, graphic.getRadius() + 3, 25, 30, 60, 255, ShieldAlpha, false);
			}
			if( PlayerShip.getCurrentJob() !is null && PlayerShip.getCurrentJob().isTarget(this) == true )
				drawCircle(x, y, graphic.getRadius() + 7, 25, 255, 20, 255, 150, false);
		}
	}
public:
	// default constructible only to allow serialization
	this() {}
	this(int parmObjectType, int parmTextureClass, int parmTextureValue = -1) {
		debug scope(failure) { writefln("fu_object constructor: fu_object.d"); }
		TextureClass = parmTextureClass;
		graphic = new Sprite(getTexture(parmTextureClass,parmTextureValue,&TextureValue),COLLISION_TYPE.RADIUS);
		graphic.setCamera(space_view);
		a = 255;
		object_type = parmObjectType;
		if( object_type != OBJECT_TYPE.TRANSPATH ) tweakColor();
		else { r = 255; g = 255; b = 255; }
		x = 0; y = 0;
		x_dest[0] = 0; y_dest[0] = 0;
		x_dest[1] = 0; y_dest[1] = 0;
		x_speed = 0; y_speed = 0; max_speed = 10;
		rotation = 0;
		rotation_speed = 0;
		weight = 1;
		level = 1;
		max_armor = 100;
		armor = max_armor;
		friction = true;
		if( object_type == OBJECT_TYPE.PLANET || object_type == OBJECT_TYPE.STATION ) {
			dockInfo = new FU_DOCKINFO(this);
		}
	}
	~this() {}
	void invalidate() {
		if( object_type == OBJECT_TYPE.SHIP ) (cast(FU_SHIP)this).stopSounds();
		object_type = OBJECT_TYPE.NULL;
		armor = 0;
		x_speed = 0; y_speed = 0;
	}
	char[] getName() {
		return "Default Name";
	}
	void setLevel(int parmLevel) {
		if( parmLevel < 1 ) parmLevel = 1;
		level = parmLevel;
	}
	void setTrim() { trimMe = 1; }
	bool needsTrim() { return (trimMe==1); }
	int getLevel() { return level; }
	float getWeight() { return weight; }
	long getValue() { return value; }
	void addWeight(float parmWeight) { weight += parmWeight; }
	void describe(T)(T ar) {
		ar.describe_staticarray(nameIndex);
		ar.describe(factionIndex);
		ar.describe(object_type);
		ar.describe(trimMe);
		ar.describe(x);
		ar.describe(y);
		ar.describe(x_speed);
		ar.describe(y_speed);
		ar.describe(rotation);
		ar.describe(rotation_speed);
		ar.describe(armor);
		ar.describe(max_armor);
		ar.describe(r);
		ar.describe(g);
		ar.describe(b);
		ar.describe(a);
		ar.describe(TextureClass);
		ar.describe(TextureValue);
		ar.describe_staticarray(x_dest);
		ar.describe_staticarray(y_dest);
		ar.describe(friction);
		ar.describe(level);
		ar.describe(weight);
		ar.describe(value);
		ar.describe(max_speed);
		// should load correct things here depending on type
		if( object_type == OBJECT_TYPE.STATION || object_type == OBJECT_TYPE.PLANET ) 
			ar.describe(dockInfo);
		
		if( ar.reading )
		{			
			graphic = new Sprite(getTexture(TextureClass,TextureValue),COLLISION_TYPE.RADIUS);
			graphic.setCamera(space_view);
			ShieldAlpha = 0;
			item = null;
			teleAngle.length = 0;
		}
	}
	void setPosition(double argX, double argY) {
		x = argX;
		y = argY;
	}
	void accelerate(double parmForce, float parmAngleOffset = 0, bool absolute_angle = false, FU_SHIP host = null) {
		debug scope(failure) { writefln("accelerate: fu_object.d"); }
		float actual_rotation = parmAngleOffset;
		if( absolute_angle == false ) {
			actual_rotation += rotation;
			if( actual_rotation > 360 ) actual_rotation -= 360;
			if( actual_rotation < 0 ) actual_rotation += 360;
		}
		double move_x = (moveToValueX(actual_rotation) / weight) * parmForce;
		double move_y = (moveToValueY(actual_rotation) / weight) * parmForce;
		x_speed -= move_x;
		y_speed -= move_y;
		double my_new_speed = getSpeed();
		double my_max_speed = max_speed;
		if( my_new_speed >= my_max_speed ) {
			// take away acceleration, allow 'space friction' to slow us down
			x_speed *= 0.975;
			y_speed *= 0.975;			
		}
	}
	float getArmor() { return armor; }
	void setArmor(float newArm) { armor = newArm; }
	float getMaxArmor() { return max_armor; }
	void setMaxArmor(float newMax) { max_armor = newMax; }
	void repairHull() { armor = max_armor; }
	float percentArmorRemaining() {
		return armor / max_armor;
	}
	void turn(float parmForce) {
		rotation += parmForce / weight;
		if( rotation > 360 ) rotation -= 360;
		if( rotation < 0 ) rotation += 360;
	}
	// more basic turn-towards code
	float objectTurnTowards(float parmForce, double parmX, double parmY) {
		float angDest = pointAtAngle(getX(), getY(), parmX, parmY);
		float angDist = angleDistance(angDest, rotation);
		if( angDist < 1.5 ) {
			rotation = angDest;
			return angDist;
		}
		float d_angle = angDest - rotation;
		if ( (d_angle < 180 && d_angle >= 0) || (d_angle < -180.) ) turn(parmForce);
		else turn(-parmForce);
		return angDist;
	}
	// returns whether shields absorbed all damage or not
	bool damage(float parmDamage, float armorMultiplier = 1.0, float shieldMultiplier = 1.0, FU_SHIP shieldHolder = null) {
		debug scope(failure) { writefln("damage: fu_object.d"); }
		bool shieldAbsorbed = false;
		if( dockInfo !is null ) dockInfo.signalAttack();
		if( shieldHolder !is null ) {
			float current, max;
			float shields = shieldHolder.modifyUtilityCounter(ITEM_CLASS.SHIELDS, -parmDamage * shieldMultiplier,&current,&max);
			if( shields < 0 ) {
				// shields can NOT absorb all damage		
				shields = (shields / shieldMultiplier) * armorMultiplier;
				armor += shields;
			} else {
				// draw shield circle around object
				shieldAbsorbed = true;
				ShieldAlpha = 255;
			}
		} else {
			armor -= parmDamage * armorMultiplier;
		}
		if( object_type == OBJECT_TYPE.ITEM && item !is null )
			item.getVars().quality = (armor / max_armor);
		if( armor <= 0 ) {
			explode();
			invalidate();
			return shieldAbsorbed;
		}
		if( armor > max_armor ) armor = max_armor;
		return shieldAbsorbed;
	}
	int getType() {
		return object_type;
	}
	void handle_object() {
		debug scope(failure) { writefln("handle_object: fu_object.d"); }
		x -= x_speed;
		y -= y_speed;
		turn(rotation_speed);
		graphic.processCollisionPos();
		drawObject();
		if( arc.io.input.mouseHit(LEFT) == true && object_type != OBJECT_TYPE.PROJECTILE && this !is PlayerShip.getTarget() && selectionMade == false ) {
			float ww = graphic.getWidth()/2;
			if( graphic.getHeight()/2 > ww ) ww = graphic.getHeight()/2;
			
			//FIXME: Doing this transformation once-per-object is extremely inefficient
			Point clicked_point = Point(arc.io.input.mouseX, arc.io.input.mouseY);
			clicked_point = space_view.normalized_to_camera(interface_view.camera_to_normalized(clicked_point));
			
			if( clicked_point.getX() > graphic.getX() - ww && clicked_point.getX() < graphic.getX() + ww &&
					clicked_point.getY() > graphic.getY() - ww && clicked_point.getY() < graphic.getY() + ww ) {
					if( object_type != OBJECT_TYPE.SHIP ||
						this is PlayerShip ||
						canSee(PlayerShip, cast(FU_SHIP)this) == true ) {
						PlayerShip.setTarget(this);
						if( PlayerShip.getAIAction() == AI_ACTION.ATTACKING ) PlayerShip.updateAction(AI_ACTION.WANDERING);
						selectionMade = true;
					}
			}
		}
		if( PlayerShip !is null &&
			PlayerShip.getCurrentJob() !is null &&
			rand()%60 == 0 &&
			distance(PlayerShip.getX(), PlayerShip.getY(), x, y) < 400 ) {
				PlayerShip.getCurrentJob().handleAction(this,null,JOB_ACTION.NEAR);
		}	
	}
	void setDestination(double parmX, double parmY, int index = 0) {
		x_dest[index] = parmX;
		y_dest[index] = parmY;
	}
	double getDestinationX(int index = 0) { return x_dest[index]; }
	double getDestinationY(int index = 0) { return y_dest[index]; }
	void addTelepathAngle(float parmAngle) {
		if( parmAngle > 360 ) parmAngle-=360;
		if( parmAngle < 0 ) parmAngle+=360;		
		teleAngle ~= parmAngle;
	}
	void setRotation(float parmAngle) {
		if( parmAngle > 360 ) parmAngle -= 360;
		if( parmAngle < 0 ) parmAngle += 360;
		rotation = parmAngle;
	}
	float getRotation() { return rotation; }
	bool isGoodTeleAngle(float parmAngle) {
		if( parmAngle > 360 ) parmAngle-=360;
		if( parmAngle < 0 ) parmAngle+=360;
		foreach(float teleAng; teleAngle) {
			if( angleDistance(teleAng, parmAngle) < 60 ) return false;
		}
		return true;
	}
	void tweakColor() {
		r = rand()%COLOR_RANGE+(256-COLOR_RANGE);
		g = rand()%COLOR_RANGE+(256-COLOR_RANGE);
		b = rand()%COLOR_RANGE+(256-COLOR_RANGE);
	}
	void setColor(short parmR, short parmG, short parmB, short parmA) {
		r = parmR;
		g = parmG;
		b = parmB;
		a = parmA;
	}
	void setRotationSpeed(float parmSpeed) { rotation_speed = parmSpeed; }
	float getRotationSpeed() { return rotation_speed; }
	void setXSpeed(double parmXSpeed) { x_speed = parmXSpeed; }
	void setYSpeed(double parmYSpeed) { y_speed = parmYSpeed; }
	Sprite getSprite() { return graphic; }
	void setFriction(bool parmFlag) { friction = parmFlag; }
	bool getFriction() { return friction; }
	void setFaction(int parmNewFaction) { factionIndex = parmNewFaction; }
	int getFaction() { return factionIndex; }
	short getAlpha() { return a; }
	void setAlpha(short newA) { a = newA; }
	float getMaxSpeed() { return max_speed; }
	float getXSpeed() {
		return x_speed;
	}
	float getYSpeed() {
		return y_speed;
	}
	double getSpeed() {
		return sqrt( x_speed * x_speed + y_speed * y_speed );
	}
	double getX() {
		return x;
	}
	double getY() {
		return y;
	}
	FU_ITEM getItem() { return item; }
	void setItem(FU_ITEM parmItem) { item = parmItem; }
	FU_DOCKINFO getDockInfo() { return dockInfo; }
	void setDockInfo(FU_DOCKINFO parmDockInfo) { dockInfo = parmDockInfo; }
	void selfRepair(float amount) {
		if( armor < max_armor ) armor += (amount * level);
		else return;
		if( armor > max_armor ) armor = max_armor;
	}
}
