﻿module classes.item;

private import init.init;
private import classes.ship;

class FU_ITEM {
private:
	// basic information
	int item_type;
	int item_class;
	int item_cargo = 0;
	int level;	// advancement level
	int[3] nameIndex;
	int factionIndex;
	// standard information
	float weight;
	// intrinsic value
	long value;
	// graphic
	Sprite graphic;
	int TextureValue;
	short offset_r, offset_g, offset_b;
	// timer stuffs
	// cruisers = how long before item becomes activated?
	// weapons = how fast can it fire?
	// activate_delay = 0 => firing, shields in use, cruiser in use, cloaking active etc.
	int activate_delay, activate_tmr;
	// does this activate and stay activated, or activate and reset?
	bool stickyActivation;
	// installed boolean == is this item currently installed and accessible?
	bool installed, enabled;
	// item specific variables
	FU_ITEMVARS vars;
	// display variables
	int x, y, mode;
	// create this item!
	bool generateWeapon() {
		// set color offset of projectile
		vars.r = rand()%COLOR_RANGE+(256-COLOR_RANGE);
		vars.g = rand()%COLOR_RANGE+(256-COLOR_RANGE);
		vars.b = rand()%COLOR_RANGE+(256-COLOR_RANGE);
		vars.projTexValue = rand()%GameTextures[TEXTURES.PROJECTILE].length;
		vars.projSndValue = rand()%Sounds[SOUND_CLASS.SHOOT].length;
		if( item_class != ITEM_CLASS.MODIFIER ) {
			// does this damage an item?
			if( rand()%3==0 ) {
				vars.itemTypeEffect = (rand()%ITEM_TYPE.max) + 1;
				switch( vars.itemTypeEffect ) {
				case ITEM_TYPE.UTILITY:
					vars.itemClassEffect = rand()%MAX_UTILS;
					break;
				case ITEM_TYPE.WEAPON:
					vars.itemClassEffect = rand()%MAX_WEAPONS + 25;
					break;
				case ITEM_TYPE.CARGO:
					vars.itemClassEffect = rand()%MAX_CARGO + 50;
					break;
				}
			}
			// does this have wierd modifiers?
			if( rand()%5==0 ) {
				vars.armor_modifier = cast(float)random_range(0,20) / 10.0;
				vars.shield_modifier = 2 - vars.armor_modifier;
			}
		}
		// set other variables
		switch( item_class ) {
		case ITEM_CLASS.MODIFIER:
			vars.acceleration = 0.5;
			vars.duration = 175;
			vars.maximum = 7;
			vars.counter_usage = 1;
			vars.counter = 3;
			vars.regeneration = 0.00125;
			vars.energy_usage = 0;
			vars.total_damage = tweakValue(cast(float)(20 + level * 20),33);
			vars.homing_ability = cast(float)random_range(30, 60) / 100.0;
			activate_delay = 160 + rand()%40;
			vars.shield_modifier = 0.0;
			vars.itemTypeEffect = 0;
			switch( rand()%2 ) {
			case 0:
				vars.modifierEffect = random_range(0, BAD_MODIFIER_COUNT-1);
				vars.armor_modifier = 0.1;
				break;
			case 1:
				vars.modifierEffect = random_range(MODIFIER_SEPERATOR, MODIFIER_SEPERATOR+GOOD_MODIFIER_COUNT-1);
				vars.armor_modifier = 0.0;
				break;
			}
			break;
		case ITEM_CLASS.PULSE:			// simple 'pulse' fire weapon, medium everything
			vars.acceleration = 6;
			vars.duration = 225;
			vars.maximum = 9 + rand()%3;
			vars.energy_usage = tweakValue(cast(float)(2 + level * 2),33);
			vars.total_damage = tweakValue(cast(float)(4 + level * 4),33);
			activate_delay = 12 + rand()%5;
			break;
		case ITEM_CLASS.SNIPER:		// slow rate of fire, high energy and damage
			vars.acceleration = 8;
			vars.duration = 225;
			vars.maximum = 13 + rand()%3;
			vars.energy_usage = tweakValue(cast(float)(15 + level * 15),33);
			vars.total_damage = tweakValue(cast(float)(30 + level * 30),33);
			activate_delay = 70 + rand()%30;
			break;
		case ITEM_CLASS.SPREAD:			// low damage, fast rate of fire, low energy, spread shot
			vars.acceleration = 7;
			vars.duration = 225;
			vars.aimMode = AIM_MODE.SPREAD;
			vars.maximum = 8 + rand()%3;
			vars.energy_usage = tweakValue(cast(float)(2 + level * 2),33);
			vars.total_damage = tweakValue(cast(float)(4 + level * 4),33);
			activate_delay = 9 + rand()%4;
			break;
		case ITEM_CLASS.MISSLE:		// high homing, high damage, low rate of fire, ammo counter
			vars.acceleration = 0.65;
			vars.duration = 225;
			vars.homing_ability = cast(float)random_range(60, 100) / 100.0;
			vars.maximum = 7 + rand()%2;
			vars.energy_usage = 0;
			vars.counter_usage = 1;
			vars.counter = 9;
			vars.regeneration = 0.0025;
			vars.total_damage = tweakValue(cast(float)(20 + 20 * level),33);
			activate_delay = 70 + rand()%15;
			break;
		case ITEM_CLASS.TORPEDO:		// lower homing, very high damage, very low rate of fire, ammo counter
			vars.acceleration = 0.75;
			vars.duration = 500;
			vars.maximum = 9 + rand()%3;
			vars.energy_usage = 0;
			vars.homing_ability = cast(float)random_range(30, 50) / 100.0;
			vars.counter_usage = 1;
			vars.counter = 7;
			vars.regeneration = 0.00125;
			vars.total_damage = tweakValue(cast(float)(50 + 50 * level),33);
			activate_delay = 190 + rand()%30;
			break;
		case ITEM_CLASS.DEATHRAY:		// takes time to charge, strong damage, high energy, fast
			vars.acceleration = 7;
			vars.duration = 225;
			vars.aimMode = AIM_MODE.TURRET;
			vars.maximum = 10 + rand()%2;
			vars.energy_usage = tweakValue(cast(float)(2 + level * 2),33);
			vars.total_damage = tweakValue(cast(float)(4 + level * 4),33);
			activate_delay = 160 + rand()%25;
			break;
		case ITEM_CLASS.TURRET:
			vars.acceleration = 6;
			vars.duration = 225;
			vars.maximum = 9 + rand()%3;
			vars.energy_usage = tweakValue(cast(float)(2 + level * 2),33);
			vars.total_damage = tweakValue(cast(float)(4 + level * 4),33);
			vars.aimMode = AIM_MODE.TURRET;
			activate_delay = 12 + rand()%5;
			break;
		default:
			return false;
		}
		return true;
	}
	bool generateUtility() {
		switch( item_class ) {
		case ITEM_CLASS.ENGINE:		// basic acceleration
			vars.acceleration = tweakValue(cast(float)(level*0.55),33);
			vars.maximum = ENGINE_SPEED;
			vars.energy_usage = 0.1*vars.acceleration;
			break;
		case ITEM_CLASS.STRAFER:		// sideways acceleration
		case ITEM_CLASS.AFTERBURNER:	// extra acceleration, less efficient
			vars.acceleration = tweakValue(cast(float)(level*1.2),33);
			vars.maximum = ENGINE_SPEED*2;
			vars.energy_usage = tweakValue(cast(float)(level*0.5),33);
			break;
		case ITEM_CLASS.CRUISER:		// super acceleration, takes time to activate
			vars.acceleration = tweakValue(cast(float)(level*3),33);
			vars.maximum = ENGINE_SPEED*4.5;
			vars.energy_usage = 0.1*vars.acceleration;
			activate_delay = 180;
			stickyActivation = true;
			break;
		case ITEM_CLASS.GENERATOR:	// makes energy
			vars.maximum = tweakValue(cast(float)(100+((level-1)*30)),33);
			vars.regeneration = tweakValue(cast(float)(0.45*level),33);
			break;
		case ITEM_CLASS.TURNER:		// accelerates turning
			vars.acceleration = tweakValue(cast(float)(32*level),33);
			vars.energy_usage = tweakValue(cast(float)(level*0.1),33);
			break;
		case ITEM_CLASS.SENSOR:		// facillitates radar
			vars.maximum = MAX_SENSOR_RANGE;
			vars.energy_usage = tweakValue(cast(float)(level*0.1),33);
			break;
		case ITEM_CLASS.CLOAKING:		// cloaking device
			vars.energy_usage = tweakValue(cast(float)(0.5*level),33);
			stickyActivation = true;
			activate_delay = 180;
			break;
		case ITEM_CLASS.SHIELDS:		// sheild device
			vars.maximum = tweakValue(cast(float)(100+((level-1)*30)),33);
			vars.regeneration = tweakValue(cast(float)(0.3*level),33);
			break;
		default:
			return false;
		}
		return true;
	}
	bool generateCargo() {
		// need to have array of values of cargo items
		switch( item_class ) {
		case ITEM_CLASS.JUNK:			// space trash
			item_cargo = rand()%JUNK_COUNT;
			vars.counter = rand()%50+20;
			value = cast(long)tweakValue(cast(float)(level * 5),33);
			break;
		case ITEM_CLASS.UNIQUE:		// questing/adventure items
			item_cargo = rand()%UNIQUE_COUNT+25;
			vars.counter = 1;
			value = cast(long)tweakValue(cast(float)(level * 50),33);
			break;
		case ITEM_CLASS.VALUABLE:		// diamonds, worthy tradables/food
			item_cargo = rand()%VALUABLE_COUNT+50;
			vars.counter = rand()%10+5;
			value = cast(long)tweakValue(cast(float)(level * 25),33);
			break;
		case ITEM_CLASS.CONTRABAND:	// illegal stuff cops hate
			item_cargo = rand()%CONTRABAND_COUNT+75;
			vars.counter = rand()%3+1;
			value = cast(long)tweakValue(cast(float)(level * 40),33);
			break;
		default:
			return false;
		}
		return true;
	}
	bool generateItem() {
		debug scope(failure) { writefln("generateItem: item.d"); }
		
		// set color offset of item
		offset_r = rand()%COLOR_RANGE+(256-COLOR_RANGE);
		offset_g = rand()%COLOR_RANGE+(256-COLOR_RANGE);
		offset_b = rand()%COLOR_RANGE+(256-COLOR_RANGE);
		// ok, make the actual thing
		switch( item_type ) {
		case ITEM_TYPE.WEAPON:
			graphic = new Sprite(getTexture(TEXTURES.ITEM,WILDCARD,&TextureValue),COLLISION_TYPE.RADIUS);
			return generateWeapon();
		case ITEM_TYPE.UTILITY:
			graphic = new Sprite(getTexture(TEXTURES.ITEM,WILDCARD,&TextureValue),COLLISION_TYPE.RADIUS);
			return generateUtility();
		case ITEM_TYPE.CARGO:
			graphic = new Sprite(getTexture(TEXTURES.ITEM,WILDCARD,&TextureValue),COLLISION_TYPE.RADIUS);
			return generateCargo();
		default:
			return false;
		}
	}
public:
	this(int parmItemType, int parmItemClass, int parmLevel, bool *failed, int useFaction) {
		// should grab its own image depending on what item it is
		vars = new FU_ITEMVARS();
		if( parmItemType != ITEM_TYPE.CARGO ) weight = tweakValue(cast(float)(parmLevel*2),25);
		else weight = tweakValue(cast(float)(parmLevel*0.2),25);
		value = 0;
		level = parmLevel;
		if( level < 1 ) level = 1;
		item_type = parmItemType;
		item_class = parmItemClass;
		activate_delay = 0;
		activate_tmr = 0;
		stickyActivation = false;
		installed = false; enabled = false;
		bool failedCheck = generateItem();
		if( failed != null ) *failed = failedCheck;
		else if( failed == null && failedCheck == false ) {
			writefln("Item failed creation and no place to put result!");
			assert(0);
		}
		factionIndex = useFaction;
	}
	// default constructible only to allow serialization
	this() {}
	~this() {}
	void describe(T)(T ar) {
		// basic information
		ar.describe(item_type);
		ar.describe(item_class);
		ar.describe(item_cargo);
		ar.describe(level); // advancement level
		ar.describe_staticarray(nameIndex);
		ar.describe(factionIndex);
		// standard information
		ar.describe(weight);
		// intrinsic value
		ar.describe(value);
		// graphic
		ar.describe(TextureValue);
		ar.describe(offset_r);
		ar.describe(offset_g);
		ar.describe(offset_b);
		ar.describe(activate_delay);
		ar.describe(activate_tmr);
		// does this activate and stay activated, or activate and reset?
		ar.describe(stickyActivation);
		// installed boolean == is this item currently installed and accessible?
		ar.describe(installed);
		ar.describe(enabled);
		// item specific variables
		ar.describe(vars);
		
		if( ar.reading )
			graphic = new Sprite(getTexture(TEXTURES.ITEM, TextureValue), COLLISION_TYPE.RADIUS);
	}
	FU_ITEMVARS getVars() { return vars; }
	void setVars(FU_ITEMVARS parmNewVars) { vars = parmNewVars; }
	float getWeight(bool includeCount = true ) {
		float returnWeight = weight;
		if( item_type == ITEM_TYPE.CARGO && includeCount == true ) returnWeight *= vars.counter;
		return returnWeight;
	}
	long getValue(bool includeQuality = true, bool includeCount = true ) {
		try {
			long returnValue = value;
			switch( item_type ) {
			case ITEM_TYPE.WEAPON:
				returnValue += cast(long)(vars.homing_ability * level * 40);
				switch( item_class ) {
				default:
					returnValue += cast(long)(((vars.total_damage - (vars.energy_usage*0.6))*250.0) / activate_delay);
					break;
				case ITEM_CLASS.DEATHRAY:
					returnValue += cast(long)(((vars.total_damage - (vars.energy_usage*0.6))*250.0) / 7.5);
					break;
				}
				returnValue -= cast(long)(weight * 5.0);
				if( vars.aimMode == AIM_MODE.TURRET ) 
					returnValue = cast(long)(cast(float)returnValue * 1.25);
				if( vars.itemTypeEffect != ITEM_TYPE.NULL ) returnValue += cast(long)(level * 30);
				break;
			case ITEM_TYPE.UTILITY:
				returnValue -= cast(long)(weight * 3.0);
				returnValue -= cast(long)(vars.energy_usage * 2);
				returnValue += cast(long)(vars.acceleration * 90.0);
				if( item_class != ITEM_CLASS.SENSOR ) returnValue += cast(long)(vars.maximum * 1.6);
				returnValue += cast(long)(vars.regeneration * 60.0);
				switch( item_class ) {
				default:
					break;
				case ITEM_CLASS.CLOAKING:
					returnValue += (level * 165);
					break;
				case ITEM_CLASS.SENSOR:
					returnValue += (level * 130);
					break;
				case ITEM_CLASS.TURNER:
					returnValue /= 40.0;
					break;
				}
				break;
			case ITEM_TYPE.CARGO:
				returnValue = CargoPrices[item_cargo] * (level + 3);
				if( includeCount == true ) returnValue *= vars.counter;
				break;
			default:
				break;
			}
			if( includeQuality == true ) returnValue = cast(long)(cast(float)returnValue * vars.quality);
			if( returnValue < 1 ) returnValue = 1;
			return returnValue;
		} catch(Exception bad) {
			writefln("ERROR: Unknown getValue() bug!");
			fflush(stdout);
		}
	}
	int getFaction() { return factionIndex; }
	void setFaction(int newFaction) { factionIndex = newFaction; }
	int getType() {	return item_type; }
	int getCargoType() { return item_cargo; }
	int getClass() { return item_class; }
	int getLevel() { return level; }
	void forceInstall() { installed = true; }
	void forceUninstall() {
		installed = false;
		enabled = false;
	}
	void setEnable(bool flag) { 
		if( installed == true ) {
			enabled = flag;
			return;
		}
		enabled = false;
	}
	bool getEnable() { return enabled && installed && (vars.quality  > 0); }
	bool isInstalled() { return installed; }
	// try and activate this item, returns true if successful
	// false otherwise
	bool activateRequest(FU_SHIP host) {
		debug scope(failure) { writefln("activateRequest: item.d"); }
		if( host is null || getEnable() == false ||
			vars.counter_usage > vars.counter ||
			item_type == ITEM_TYPE.WEAPON && host.sapModifier(MODIFIER_TYPE.DISABLE_WEAPONS,cast(float)getLevel()*0.2) == true )
				return false;
		float shipEnergy, maxEnergy;
		if( vars.energy_usage != 0 ) {
			host.modifyUtilityCounter(ITEM_CLASS.GENERATOR,0,&shipEnergy,&maxEnergy);
			if( shipEnergy < vars.energy_usage ) return false;
		}
		activate_tmr += 2;
		if( activate_tmr >= activate_delay ) {
			vars.counter -= vars.counter_usage;
			host.modifyUtilityCounter(ITEM_CLASS.GENERATOR,-vars.energy_usage,&shipEnergy,&maxEnergy);
			if( item_class == ITEM_CLASS.DEATHRAY ) activate_tmr = activate_delay - 6;
			else if( stickyActivation ) activate_tmr = activate_delay + 1;
			else activate_tmr = 0;
			return true;
		}
		return false;
	}
	void disableActivation() {
		activate_tmr = 0;
	}
	void handleItem(FU_SHIP host) {
		debug scope(failure) { writefln("handleItem: item.d"); }
		if( getEnable() == true && host !is null ) {
			if( item_type == ITEM_TYPE.WEAPON ) {
				vars.counter += vars.regeneration;
				if( item_class == ITEM_CLASS.MODIFIER && vars.counter > 4 ) vars.counter = 4;
				else if( vars.counter > 10 ) vars.counter = 10;
			} else if( item_class != ITEM_CLASS.SHIELDS ) {
				float chargeDesired = (vars.regeneration * vars.quality);
				if( item_class == ITEM_CLASS.GENERATOR && host.sapModifier(MODIFIER_TYPE.INCREASE_ENERGY_CHARGE,chargeDesired) == true )
					chargeDesired *= 2;
				if( item_class != ITEM_CLASS.GENERATOR || host.sapModifier(MODIFIER_TYPE.DISABLE_ENERGY_CHARGE,chargeDesired) == false ) {
					vars.counter += chargeDesired;
					if( vars.counter > vars.maximum ) vars.counter = vars.maximum;
				}
			} else if( vars.counter < vars.maximum ) {
				float chargeNeeded = (vars.maximum - vars.counter);
				float maxCharge = vars.regeneration * vars.quality;
				if( chargeNeeded > maxCharge ) chargeNeeded = maxCharge;
				float myEnergy;
				host.modifyUtilityCounter(ITEM_CLASS.GENERATOR,0,&myEnergy,&maxCharge);
				if( myEnergy >= chargeNeeded ) {
					host.modifyUtilityCounter(ITEM_CLASS.GENERATOR,-chargeNeeded,&myEnergy,&maxCharge);
					if( host.sapModifier(MODIFIER_TYPE.INCREASE_SHIELDS_CHARGE, chargeNeeded) == true )
						chargeNeeded *= 2;
					if( host.sapModifier(MODIFIER_TYPE.DISABLE_SHIELD_CHARGE, chargeNeeded) == false )
						vars.counter += chargeNeeded;
				}
				if( vars.counter > vars.maximum ) vars.counter = vars.maximum;
			}
		}
		if( activate_tmr > 0 ) activate_tmr--;
		if( vars.counter < 0 ) vars.counter = 0;
	}
	/// returns whether this is a good modifier or not
	bool isGoodModifier() {
		if( item_class != ITEM_CLASS.MODIFIER ) return false;
		if( vars.modifierEffect < MODIFIER_SEPERATOR ) return false;
		return true;
	}
	void drawDisplay(int parmX, int parmY, int parmMode, bool useGreen = false, long *totalRepairCost = null, float fluxValue = 1.0) {
		int w = DISPLAY_WIDTH[parmMode];
		int h = DISPLAY_HEIGHT[parmMode];
		short r, g, b;
		x = parmX; y = parmY;
		mode = parmMode;
		switch( item_type ) {
		case ITEM_TYPE.WEAPON:
			drawDarkBox(x, y, w, h, 255, 125, 125);
			break;
		case ITEM_TYPE.UTILITY:
			drawDarkBox(x, y, w, h, 125, 125, 255);
			break;
		case ITEM_TYPE.CARGO:
			drawDarkBox(x, y, w, h, 255, 255, 255);
			break;
		}
		if( isInstalled() == true ) {
			if( getEnable() == true ) { r = 100; g = 255; b = 100; }
			else { r = 255; g = 100; b = 100; }
		} else { r = 255; g = 255; b = 255; }
		// fill 'er up!
		long retailValue, sellValue;
		if( mode != ITEMDISPLAY_MODE.COMPACT ) {
			retailValue = getValue(true, false);
			if( item_type != ITEM_TYPE.CARGO ) sellValue = cast(long)(cast(float)retailValue * SELLBACK);
			else {
				sellValue = cast(long)(cast(float)retailValue * fluxValue);
				retailValue = sellValue;
			}
			if( sellValue < 1 ) sellValue = 1;
			if( retailValue < 1 ) retailValue = 1;
		}
		switch( mode ) {
		case ITEMDISPLAY_MODE.COMPACT:
			int useMPMIndex = 0;
			if( item_class == ITEM_CLASS.MODIFIER ) useMPMIndex = 1 + (vars.modifierEffect/MODIFIER_SEPERATOR);
			smallFont.draw(ShortItemNames[item_class] ~ modifierPlusMinus[useMPMIndex], x + 2, y + 2, r, g, b, 255);
			// quality bar
			drawRect(x+2,y+10,60,12,150,150,150,60,true);
			drawRect(x+2,y+10,60.0*vars.quality,6,200,20,200,255,true);
			// activation bar
			if( activate_delay > 0 ) drawRect(x+2,y+16,60.0*cast(float)(cast(float)activate_tmr/cast(float)activate_delay),6,20,150,255,255,true);
			// counter
			if( vars.counter != 0 ) {
				smallFont.draw("C:" ~ std.string.toString(cast(long)vars.counter),x+2,y+23,r,g,b,255);
			} else {
				smallFont.draw("C: N/A",x+2,y+23,r,g,b,255);
			}
			break;
		case ITEMDISPLAY_MODE.NORMAL:
			// draw information text
			char[] iText = "No";
			if( installed == true ) iText = "Yes";
			if( useGreen == true ) { r = 20; g = 255; b = 20; }
			else { r = 255; g = 255; b = 255; }
			standardFont.draw("Item: " ~ ItemNames[item_class], x + 2, y + 1, r, g, b, 255);
			standardFont.draw("Quality: " ~ std.string.toString(cast(int)(vars.quality*100)) ~ "%", x + 2, y + 10, r, g, b, 255);
			if( item_type == ITEM_TYPE.WEAPON || item_type == ITEM_TYPE.UTILITY ) standardFont.draw("Installed: " ~ iText, x + 2, y + 19, r, g, b, 255);
			standardFont.draw("Value: " ~ std.string.toString(retailValue), x + 2, y + 28, r, g, b, 255);
			standardFont.draw("Level: " ~ std.string.toString(level), x + 2, y + 37, r, g, b, 255);
			if( vars.counter != 0 ) {
				standardFont.draw("Count: " ~ std.string.toString(cast(long)vars.counter), x+2, y + 46, r, g, b, 255);
			} else {
				standardFont.draw("Count: N/A", x+2, y+46, r, g, b, 255);
			}
			// draw more information about item
			if( item_type == ITEM_TYPE.CARGO ) standardFont.draw(CargoNames[item_cargo], x+2, y+55, r, g, b, 255);
			else if( vars.quality < 1.0 ) {
				long repairCost = cast(long)(cast(float)getValue(false) * (1-vars.quality));
				standardFont.draw("Repair: " ~ std.string.toString(repairCost), x+2, y+55, r, g, b, 255);
				if( totalRepairCost != null ) *totalRepairCost += repairCost;
			}
			// draw graphic
			graphic.setSize(35,35);
			graphic.setColor(offset_r, offset_g, offset_b, 175);
			graphic.setPosition(x + (w-(graphic.getWidth()/2)-1), y + (h-(graphic.getHeight()/2)-1));
			graphic.draw();
			break;
		case ITEMDISPLAY_MODE.DETAILED:
			// hrm, this is looking a bit ugly, should seperate into two columns?
			char[] iText = "This item is not installed.";
			char[] maxThing = "Speed";
			if( installed == true ) iText = "This item is installed.";
			bigFont.draw("Classification: " ~ ItemNames[item_class], x + 2, y + 2, 255, 255, 255, 255);
			bigFont.draw("Quality: " ~ std.string.toString(cast(int)(vars.quality*100)) ~ "%", x + 2, y + 17, 255, 255, 255, 255);
			bigFont.draw("Level: " ~ std.string.toString(level), x + 320, y + 17, 255, 255, 255, 255);
			if( item_type == ITEM_TYPE.WEAPON || item_type == ITEM_TYPE.UTILITY ) bigFont.draw(iText, x + 2, y + 40, 255, 255, 255, 255);
			else bigFont.draw("This item cannot be installed.", x + 2, y + 40, 255, 255, 255, 255);
			if( item_type == ITEM_TYPE.CARGO ) bigFont.draw("Cargo Contents: " ~ CargoNames[item_cargo], x + 2, y + 55, 255, 255, 255, 255);
			else if( item_type == ITEM_TYPE.WEAPON ) bigFont.draw("This item can be installed as a weapon.", x + 2, y + 55, 255, 255, 255, 255);
			else if( item_type == ITEM_TYPE.UTILITY ) {
				bigFont.draw("This item can be installed as a utility.", x + 2, y + 55, 255, 255, 255, 255);
				switch( item_class ) {
				case ITEM_CLASS.SENSOR:
					maxThing = "Range";
					break;
				case ITEM_CLASS.SHIELDS:
					maxThing = "Shields";
					break;
				case ITEM_CLASS.GENERATOR:
					maxThing = "Energy";
					break;
				default:
					break;
				}
			}
			bigFont.draw("Retail Value: " ~ std.string.toString(retailValue), x + 2, y + 75, 200, 255, 200, 255);
			bigFont.draw("Sell Value: " ~ std.string.toString(sellValue), x + 320, y + 75, 200, 255, 200, 255);
			bigFont.draw("Count/Amount: " ~ std.string.toString(vars.counter), x + 2, y + 90, 200, 255, 200, 255);
			if( item_type != ITEM_TYPE.CARGO ) {
				bigFont.draw("Max " ~ maxThing ~ ": " ~ std.string.toString(vars.maximum), x + 320, y + 90, 200, 255, 200, 255);
				bigFont.draw("Counter Usage: " ~ std.string.toString(vars.counter_usage), x + 2, y + 105, 200, 255, 200, 255);
				bigFont.draw("Energy Usage: " ~ std.string.toString(vars.energy_usage), x + 320, y + 105, 200, 255, 200, 255);
				bigFont.draw("Acceleration: " ~ std.string.toString(vars.acceleration), x + 2, y + 125, 255, 200, 200, 255);
				bigFont.draw("Regeneration: " ~ std.string.toString(vars.regeneration), x + 320, y + 125, 255, 200, 200, 255);
				bigFont.draw("Duration: " ~ std.string.toString(vars.duration), x + 2, y + 140, 255, 200, 200, 255);
				if( item_class == ITEM_CLASS.MODIFIER ) bigFont.draw("Modify Effect: " ~ modifierNames[vars.modifierEffect], x + 320, y + 140, 255, 200, 200, 255);
				bigFont.draw("Activation Delay: " ~ std.string.toString(activate_delay), x + 320, y + 210, 255, 255, 255, 255);
			}
			if( item_type == ITEM_TYPE.WEAPON ) {
				char[] damOReff = "Damage";
				if( item_class == ITEM_CLASS.MODIFIER ) damOReff = "Effect";
				bigFont.draw("Total " ~ damOReff ~ ": " ~ std.string.toString(vars.total_damage), x + 2, y + 160, 200, 200, 255, 255);
				bigFont.draw("Homing Ability: " ~ std.string.toString(cast(int)(vars.homing_ability * 100.0)) ~ "%", x + 320, y + 160, 200, 200, 255, 255);
				bigFont.draw("Shield Modifier: " ~ std.string.toString(vars.shield_modifier) ~ "x", x + 2, y + 175, 200, 200, 255, 255);
				bigFont.draw("Armor Modifier: " ~ std.string.toString(vars.armor_modifier) ~ "x", x + 320, y + 175, 200, 200, 255, 255);
				if( vars.itemTypeEffect == ITEM_TYPE.NULL ) bigFont.draw("This item is not designed to cause damage to any specific item.", x + 2, y + 190, 200, 200, 255, 255);
				else {
					char[] typeText;
					switch( vars.itemTypeEffect ) {
					case ITEM_TYPE.CARGO:
						typeText = "Item damages cargo: ";
						break;
					case ITEM_TYPE.UTILITY:
						typeText = "Item damages utility: ";
						break;
					case ITEM_TYPE.WEAPON:
						typeText = "Item damages weapon: ";
						break;
					}
					if( vars.itemClassEffect != WILDCARD ) bigFont.draw(typeText ~ ItemNames[vars.itemClassEffect], x + 2, y + 190, 200, 200, 255, 255);
					else bigFont.draw(typeText ~ "Random", x + 2, y + 190, 200, 200, 255, 255);
				}
				bigFont.draw("Projectile Image:", x + 320, y + 225, 200, 200, 255, 255);
				Texture projImage = getTexture(TEXTURES.PROJECTILE, vars.projTexValue);
				drawImage(x + 50/2 + 320, y + 50/2 + 240, 50, 50, vars.r, vars.g, vars.b, 255, 0, 0, 0, projImage.getID);
			}
			bigFont.draw("Weight: " ~ std.string.toString(weight), x + 2, y + 210, 255, 255, 255, 255);
			bigFont.draw("Item Image:", x + 2, y + 225, 255, 255, 255, 255);
			graphic.setSize(50,50);
			graphic.setColor(offset_r, offset_g, offset_b, 175);
			graphic.setPosition(x + graphic.getWidth()/2 + 2, y + graphic.getHeight()/2 + 240);
			graphic.draw();
			break;
		}
	}
	bool displayClicked(int mouseButton = LEFT) {
		int w = DISPLAY_WIDTH[mode];
		int h = DISPLAY_HEIGHT[mode];
		if( arc.io.input.mouseHit(mouseButton) == false ) return false;
		if( arc.io.input.mouseX >= x && arc.io.input.mouseY >= y &&
			arc.io.input.mouseX <= x+w && arc.io.input.mouseY <= y+h ) return true;
		return false;
	}
}

enum AIM_MODE {
	STRAIGHT,
	SPREAD,
	TURRET,
};

class FU_ITEMVARS {
public:
	float acceleration;	// engine/turning, weapon acceleration
	float regeneration;	// shield regeneration, generator etc. (when not activated)
	float energy_usage;	// how much 'counter' from generator(s) to take upon activation
	float counter_usage;// how much 'counter' from this to take upon activation
	float maximum;		// maximum counter, shields, speed, sensor distance, energy etc.
	float counter;		// current energy, weapon count
	float quality;		// 1 = 100% new, 0 = 0% broken
	uint duration;		// length of projectile in flight etc.

	// more weapon specific items:
	float total_damage;		
	float shield_modifier;		// total_damage * shield_modifier = shield damage
	float armor_modifier;		// total_damage * armor_modifier = armor damage
	float homing_ability;		// 1 = 100% homing ability, 0 = 0% straight fire
	int itemTypeEffect;			// type of item this weapon effects
	int itemClassEffect;		// class of item this weapon effects
	short r, g, b;				// color to draw weapon projectiles at
	int projTexValue, projSndValue;
	int modifierEffect;			// type of modifier effect
	int aimMode;				// can this auto-aim?

	this() {
		acceleration = 0;
		regeneration = 0;
		energy_usage = 0;
		counter_usage = 0;
		maximum = 0;
		counter = 0;
		quality = 1.0;
		duration = 0;
		total_damage = 0;
		shield_modifier = 1.0;
		armor_modifier = 1.0;
		homing_ability = 0;
		itemTypeEffect = ITEM_TYPE.NULL;
		itemClassEffect = ITEM_TYPE.NULL;
		projTexValue = 0;
		aimMode = AIM_MODE.STRAIGHT;
	}
	void describe(T)(T ar) {
		ar.describe(acceleration);
		ar.describe(regeneration);
		ar.describe(energy_usage);
		ar.describe(counter_usage);
		ar.describe(maximum);
		ar.describe(counter);
		ar.describe(quality);
		ar.describe(duration);
		ar.describe(total_damage);
		ar.describe(shield_modifier);
		ar.describe(armor_modifier);
		ar.describe(homing_ability);
		ar.describe(itemTypeEffect);
		ar.describe(itemClassEffect);
		ar.describe(r);
		ar.describe(g);
		ar.describe(b);
		ar.describe(projTexValue);
		ar.describe(projSndValue);
		ar.describe(modifierEffect);
		ar.describe(aimMode);
	}
}
