﻿module classes.job;

private import init.init;
private import classes.ship;
private import classes.fu_object;
private import classes.item;
static import space.generation;
static import space.hud;

class FU_OBJECTIVE {
private:
	FU_SHIP[] targetShip;
	FU_OBJECT[] targetObject;
	FU_ITEM[] targetItem;
	double item_x, item_y;
	int counter, type;
	bool triggered = false;
public:
	this(int objectiveType) { type = objectiveType;	}
	~this() { }
	// default constructible only to allow serialization
	this() {}
	void describe(T)(T ar) {
		ar.describe(item_x);
		ar.describe(item_y);
		ar.describe(counter);
		ar.describe(type);
		ar.describe(targetShip);
		ar.describe(targetObject);
		ar.describe(targetItem);
	}
	void addTarget(FU_SHIP target) { targetShip ~= target; }
	void addTarget(FU_OBJECT target) { targetObject ~= target; }
	void addTarget(FU_ITEM target) { targetItem ~= target; }
	void addCounter(int addAmnt = 1) { counter += addAmnt; }
	// returns true if the current objective gets completed
	void reduceCounter() {
		if( counter > 0 ) {
			space.hud.addMessage("+ Job Progress",20,255,20);
			counter--;
		}
	}
	bool triggerCheck() {
		bool activatedTrigger = !triggered;
		triggered = true;
		return activatedTrigger;
	}
	void getInfo(double *outX, double *outY, int *level) {
		if( targetShip.length > 0 ) {
			*outX = targetShip[0].getX();
			*outY = targetShip[0].getY();
			*level = targetShip[0].getLevel();
			return;
		}
		if( targetObject.length > 0 ) {
			*outX = targetObject[0].getX();
			*outY = targetObject[0].getY();
			*level = targetObject[0].getLevel();
			return;
		}
		if( targetItem.length > 0 ) {
			*outX = item_x;
			*outY = item_y;
			*level = targetItem[0].getLevel();
			return;
		}
		*outX = 0;
		*outY = 0;
		*level = 1;
		return;
	}
	bool isTarget(FU_SHIP target) {
		if( target is null ) return false;
		foreach(FU_SHIP tCheck; targetShip) {
			if( tCheck is target ) return true;
		}
		return false;
	}
	bool isTarget(FU_OBJECT target) {
		if( target is null ) return false;
		switch( target.getType() ) {
		case OBJECT_TYPE.SHIP:
			return isTarget(cast(FU_SHIP)target);
		case OBJECT_TYPE.ITEM:
			return isTarget(target.getItem());
		default:
			foreach(FU_OBJECT tCheck; targetObject) {
				if( tCheck is target ) return true;
			}
		}
		return false;
	}
	bool isTarget(FU_ITEM target) {
		if( target is null ) return false;
		if( target.getType() != ITEM_TYPE.CARGO ||
			target.getClass() != ITEM_CLASS.UNIQUE ) return false;
		foreach(FU_ITEM tCheck; targetItem) {
			if( target.getLevel() == tCheck.getLevel() &&
				target.getCargoType() == tCheck.getCargoType() )
					return true;
		}
		return false;
	}
	int getType() { return type; }
	void setItemPos(double x, double y) {
		item_x = x;
		item_y = y;
	}
	bool isCompleted() {
		return (counter == 0);
	}
	void activate() {
		foreach(FU_SHIP s; targetShip) GameObjects.add(s);
		foreach(FU_OBJECT o; targetObject) GameObjects.add(o);
		foreach(FU_ITEM i; targetItem) {
			FU_OBJECT newItem = MainGenerator.spawnItem(item_x + random_range(-400,400), item_y + random_range(-400,400), i);
			newItem.setTrim();
		}
	}
	void drawDisplay(int x, int y, int drawMode) {
		switch( drawMode ) {
		case ITEMDISPLAY_MODE.COMPACT:
			break;
		case ITEMDISPLAY_MODE.NORMAL:
			break;
		case ITEMDISPLAY_MODE.DETAILED:
			bigFont.draw(jobNames[type] ~ " Mission",x,y,255,200,200,255);
			if( targetShip.length > 0 ) {
				bigFont.draw("Target: " ~ std.string.toString(targetShip.length) ~ " Ship(s)",x,y+15,255,255,255,255);
				bigFont.draw("Target Level: " ~ std.string.toString(targetShip[0].getLevel()),x,y+30,255,255,255,255);
				bigFont.draw("Relative Position: (X: " ~ std.string.toString(ceil(PlayerShip.getX()-targetShip[0].getX())) ~ ", Y: " ~ std.string.toString(ceil(PlayerShip.getY()-targetShip[0].getY())) ~ ")",x,y+45,255,255,255,255); 
				bigFont.draw("Target Faction: " ~ FactionNames[targetShip[0].getFaction()],x,y+60,255,255,255,255);
			} else if( targetObject.length > 0 ) {
				bigFont.draw("Target: " ~ std.string.toString(targetObject.length) ~ " " ~ ObjectNames[targetObject[0].getType()] ~ "(s)",x,y+15,255,255,255,255);
				bigFont.draw("Target Level: " ~ std.string.toString(targetObject[0].getLevel()),x,y+30,255,255,255,255);
				bigFont.draw("Relative Position: (X: " ~ std.string.toString(ceil(PlayerShip.getX()-targetObject[0].getX())) ~ ", Y: " ~ std.string.toString(ceil(PlayerShip.getY()-targetObject[0].getY())) ~ ")",x,y+45,255,255,255,255); 
				bigFont.draw("Target Faction: " ~ FactionNames[targetObject[0].getFaction()],x,y+60,255,255,255,255);
			} else if( targetItem.length > 0 ) {
				bigFont.draw("Target: " ~ std.string.toString(targetItem.length) ~ " " ~ CargoNames[targetItem[0].getCargoType()] ~ "(s)",x,y+15,255,255,255,255);
				bigFont.draw("Target Level: " ~ std.string.toString(targetItem[0].getLevel()),x,y+30,255,255,255,255);
				bigFont.draw("Relative Position: (X: " ~ std.string.toString(ceil(PlayerShip.getX()-item_x)) ~ ", Y: " ~ std.string.toString(ceil(PlayerShip.getY()-item_y)) ~ ")",x,y+45,255,255,255,255); 
				bigFont.draw("Target Faction: " ~ FactionNames[targetItem[0].getFaction()],x,y+60,255,255,255,255);
			}
			break;
		}
	}
};

class FU_JOB {
private:
	FU_SHIP makeShipObjective(FU_OBJECTIVE addTo, double x, double y, int level ) {
		FU_SHIP newShip = MainGenerator.spawnShip(null,true);
		newShip.setPosition(x + random_range(-400, 400), y + random_range(-400, 400));
		newShip.setForcePatrolFlag(true);
		newShip.setDestination(newShip.getX(), newShip.getY());
		newShip.setDestination(newShip.getX(), newShip.getY(), 1);
		newShip.setAIAction(AI_ACTION.PATROLING);
		newShip.setTrim();
		addTo.addTarget(newShip);
		addTo.addCounter();
		return newShip;
	}
	FU_ITEM makeItemObjective(FU_OBJECTIVE addTo, double x, double y, int level, int setFaction ) {
		FU_ITEM newItem = new FU_ITEM(ITEM_TYPE.CARGO, ITEM_CLASS.UNIQUE, level,null,setFaction);
		addTo.setItemPos(x, y);
		addTo.addTarget(newItem);
		addTo.addCounter();
		return newItem;
	}
	FU_OBJECT makeObjectObjective(FU_OBJECTIVE addTo, double x, double y, int level, int objectType ) {
		FU_OBJECT newObj;
		switch( objectType ) {
		case 0:
			newObj = new FU_OBJECT(OBJECT_TYPE.PLANET, TEXTURES.PLANET);
			newObj.setMaxArmor(25000 + level * 200);			
			break;
		case 1:
			newObj = new FU_OBJECT(OBJECT_TYPE.STATION, TEXTURES.STATION);
			newObj.setMaxArmor(17500 + level * 150);
			break;
		}
		newObj.setPosition(x + random_range(-400, 400), y + random_range(-400, 400));
		newObj.setArmor(newObj.getMaxArmor());
		newObj.setLevel(level);
		newObj.setTrim();
		addTo.addTarget(newObj);
		addTo.addCounter();
		return newObj;
	}
	int correctFaction(int objectiveType) {
		int returnFaction = helpFaction;
		switch( objectiveType ) {
		case OBJECTIVE_TYPE.DESTROY:
			// evil faction
			do {
				returnFaction = random_range(0,4);
			} while(abs(returnFaction-helpFaction)<2)			
			break;
		case OBJECTIVE_TYPE.RETURN:
		// case OBJECTIVE_TYPE.LOCATE:
			// any faction
			returnFaction = random_range(0,4);
			break;
		// case OBJECTIVE_TYPE.DELIVER:
		case OBJECTIVE_TYPE.FOLLOW:
		case OBJECTIVE_TYPE.PROTECT:
			// good faction
			do {
				returnFaction = random_range(0,4);
			} while(abs(returnFaction-helpFaction)>1)
			break;
		}
		return returnFaction;
	}
	double totalDistance;
	float jobLevel;
	int helpFaction;
	bool activated = false;
	int factionImportance;
	// display thingies
	int disp_x, disp_y, disp_mode;
public:
	long reward;
	FU_OBJECTIVE[] jobObjective;
	this() {
		jobObjective.length = 0;
	}
	~this() { }
	void describe(T)(T ar) {
		ar.describe(totalDistance);
		ar.describe(jobLevel);
		ar.describe(totalDistance);
		ar.describe(jobLevel);
		ar.describe(helpFaction);
		ar.describe(activated);
		ar.describe(factionImportance);
		ar.describe(reward);
		ar.describe(jobObjective);
		/* if( ar.reading )
		{
			activated = false;
		} */
	}
	bool isTarget(FU_OBJECT check) {
		foreach(FU_OBJECTIVE jO; jobObjective) {
			if( jO.isTarget(check) == true ) return true;
		}
		return false;
	}
	double getDistance() { return totalDistance; }
	void calculateInfo() {
		double gX, gY;
		int gL;
		jobLevel = 0;
		totalDistance = 0;
		foreach(FU_OBJECTIVE jO; jobObjective) {
			jO.getInfo(&gX, &gY, &gL);
			jobLevel += gL;
			totalDistance += fabs(PlayerShip.getX() - gX);
			totalDistance += fabs(PlayerShip.getY() - gY);
		}
		jobLevel /= jobObjective.length;
	}
	bool activateJob() {
		if( activated == true ) return false;
		activated = true;
		foreach(FU_OBJECTIVE jObj; jobObjective) {
			jObj.activate();
		}
		return true;
	}
	bool isActivated() { return activated; }
	bool isCompleted() {
		foreach(FU_OBJECTIVE jObj; jobObjective) {
			if( jObj.isCompleted() == false ) return false;
		}
		return true;
	}
	int getFaction() { return helpFaction; }
	int getImportance() { return factionImportance; }
	void generateObjective(int objType, int level, int faction) {
		// need to set factions, durations
		FU_OBJECTIVE newObj = new FU_OBJECTIVE(objType);
		double x = PlayerShip.getX() + random_range(-(MAX_SENSOR_RANGE*2), MAX_SENSOR_RANGE*2);
		double y = PlayerShip.getY() + random_range(-(MAX_SENSOR_RANGE*2), MAX_SENSOR_RANGE*2);
		int objLevel = PlayerShip.getLevel() + random_range(-1,1);
		if( objLevel < 1 ) objLevel = 1;
		helpFaction = faction;
		factionImportance += random_range(75,150);
		switch( objType ) {
		//case OBJECTIVE_TYPE.LOCATE:
		case OBJECTIVE_TYPE.DESTROY:
		case OBJECTIVE_TYPE.PROTECT:
			{
				int oType;
				if( objType == OBJECTIVE_TYPE.PROTECT ) {
					newObj.addCounter(random_range(110,130));
					oType = random_range(1,2);
				} else oType = random_range(0,2);
				int objCount;
				if( oType != 2 ) objCount = random_range(1,5);
				else objCount = 1;
				int setFact = correctFaction( objType );
				for(int i=0;i<objCount;i++) {
					switch( oType ) {
					case 0:
						{
							FU_ITEM newItem = makeItemObjective(newObj, x, y, objLevel, setFact); 
							reward += objLevel * 90;
						}
						break;
					case 1:
						{
							FU_SHIP newShip = makeShipObjective(newObj, x, y, objLevel );
							newShip.setFaction(setFact);
							if( objType == OBJECTIVE_TYPE.DESTROY ) reward += objLevel * 200;
							else reward += objLevel * 350;
						}
						break;
					case 2:
						{
							FU_OBJECT newO = makeObjectObjective(newObj, x, y, objLevel, rand()%2 ); 
							newO.setFaction(setFact);
							if( objType == OBJECTIVE_TYPE.DESTROY ) reward += objLevel * 400;
							else reward += objLevel * 250;
						}
						break;
					}
				}
			}
			break;
		case OBJECTIVE_TYPE.FOLLOW:
			{
				FU_SHIP newShip = makeShipObjective(newObj, x, y, objLevel);
				newShip.setFaction(correctFaction(objType));
				newObj.addCounter(random_range(40,55));
				reward += objLevel * 175;
			}
			break;
		// case OBJECTIVE_TYPE.DELIVER:
		case OBJECTIVE_TYPE.RETURN:
			{
				FU_ITEM newItem = makeItemObjective(newObj, x, y, objLevel,correctFaction(objType)); 
				reward += objLevel * 100;
			}
			break;
		}
		jobObjective ~= newObj;
	}
	void drawDisplay(int parmX, int parmY, int parmDrawMode, bool useGreen = false ) {
		drawDarkBox(parmX, parmY, DISPLAY_WIDTH[parmDrawMode], DISPLAY_HEIGHT[parmDrawMode]);
		disp_x = parmX; disp_y = parmY; disp_mode = parmDrawMode;
		short r, g, b;
		if( useGreen == true ) { r = 20; g = 255; b = 20; }
		else { r = 255; g = 255; b = 255; }
		if( jobObjective.length == 0 ) return;
		switch( disp_mode ) {
		case ITEMDISPLAY_MODE.COMPACT:
			break;
		case ITEMDISPLAY_MODE.NORMAL:
			foreach(int i, FU_OBJECTIVE jO; jobObjective) {
				standardFont.draw("(" ~ std.string.toString(i+1) ~ ") " ~ jobNames[jO.getType()] ~ " Mission", disp_x + 2, disp_y + 2 + i * 10, r, g, b, 255);
			}
			standardFont.draw("Level: " ~ std.string.toString(jobLevel), disp_x + 2, disp_y + 34, r, g, b, 255);
			//standardFont.draw("Objectives: " ~ std.string.toString(jobObjective.length), disp_x + 2, disp_y + 42, r, g, b, 255);
			standardFont.draw("Reward: " ~ std.string.toString(reward), disp_x + 2, disp_y + 44, r, g, b, 255);
			//standardFont.draw("Faction: " ~ FactionNames[getFaction()], disp_x + 2, disp_y + 42, r, g, b, 255);
			standardFont.draw("Importance: " ~ std.string.toString(getImportance()), disp_x + 2, disp_y + 54, r, g, b, 255);
			break;
		case ITEMDISPLAY_MODE.DETAILED:
			{
				foreach(int i, FU_OBJECTIVE jO; jobObjective) {
					int drawY = disp_y + 2 + (i * 80);
					jO.drawDisplay(disp_x + 2, drawY, ITEMDISPLAY_MODE.DETAILED);
					if( jO.isCompleted() == true ) bigFont.draw("[ COMPLETE ]",disp_x + 350, drawY, 20, 255, 20, 255);
					else if( i == PlayerShip.WaypointIndex() ) bigFont.draw("[ WAYPOINT ]",disp_x + 350, drawY, 255, 20, 255, 255);
				}
				bigFont.draw("Reward: " ~ std.string.toString(reward), disp_x + 2, disp_y + 240, 200, 255, 200, 255);
				bigFont.draw("Job Faction: " ~ FactionNames[getFaction()], disp_x + 2, disp_y + 255, 200, 255, 200, 255);
				bigFont.draw("Job Importance: " ~ std.string.toString(getImportance()), disp_x + 2, disp_y + 270, 200, 255, 200, 255);
			}
			break;
		}
	}
	void handleAction(FU_OBJECT parmObj, FU_ITEM parmItem, int action) {
		FU_OBJECTIVE actualObj;
		if( parmObj !is null ) {
			foreach(FU_OBJECTIVE jO; jobObjective) {
				switch( parmObj.getType() ) {
				case OBJECT_TYPE.SHIP:
					if( jO.isTarget(cast(FU_SHIP)parmObj) == true ) actualObj = jO;
					break;
				case OBJECT_TYPE.ITEM:
					if( jO.isTarget(parmObj.getItem()) == true ) actualObj = jO;
					break;
				default:
					if( jO.isTarget(parmObj) == true ) actualObj = jO;
					break;
				}
			}
		} else if( parmItem !is null ) {
			foreach(FU_OBJECTIVE jO; jobObjective) {
				if( jO.isTarget(parmItem) == true ) actualObj = jO;
			}
		}
		if( actualObj is null || actualObj.isCompleted() == true ) return;
		switch( actualObj.getType() ) {
		case OBJECTIVE_TYPE.DESTROY:
			if( action == JOB_ACTION.KILL ) actualObj.reduceCounter();
			break;
		case OBJECTIVE_TYPE.PROTECT:
			if( action == JOB_ACTION.KILL ) PlayerShip.cancelJob();
			else if( action == JOB_ACTION.NEAR ) {
				if( rand()%2==0 ) {
					foreach(inout FU_OBJECT data; GameObjects) {
						if( objectExists(data) && data.getType() == OBJECT_TYPE.SHIP ) {
							FU_SHIP tShip = cast(FU_SHIP)data;
							if( abs(tShip.getFaction() - parmObj.getFaction()) > 1 &&
								tShip !is PlayerShip && isTarget(tShip) == false &&
								tShip.getAIAction() != AI_ACTION.ATTACKING &&
								tShip.getAIClass() != AI_CLASS.TOURIST &&
								tShip.getItemCount(ITEM_TYPE.WEAPON, -1, true, true, true) > 0 ) {
									tShip.setTarget(parmObj);
									tShip.setAIAction(AI_ACTION.ATTACKING);
									tShip.setLeader(null);
									space.hud.addMessage("Attacker coming!",255,20,20);
							}
						}
					}
				}
				actualObj.reduceCounter();
			}
			break;
		case OBJECTIVE_TYPE.FOLLOW:
			if( action == JOB_ACTION.KILL ) PlayerShip.cancelJob();
			else if( action == JOB_ACTION.NEAR ) {
				actualObj.reduceCounter();
				if( actualObj.triggerCheck() == true ) {
					FU_SHIP tShip = cast(FU_SHIP)parmObj;
					tShip.setForcePatrolFlag(false);
					tShip.setForceWanderFlag(true);
					tShip.setAIAction(AI_ACTION.WANDERING);
				}
			}
			break;
		/* case OBJECTIVE_TYPE.LOCATE:
			if( action == JOB_ACTION.KILL ) PlayerShip.cancelJob();
			else if( action == JOB_ACTION.NEAR ) actualObj.reduceCounter();
			break; */			
		case OBJECTIVE_TYPE.RETURN:
		// case OBJECTIVE_TYPE.DELIVER:
			if( action == JOB_ACTION.KILL ) PlayerShip.cancelJob();
			else if( action == JOB_ACTION.RETURN ) actualObj.reduceCounter();
			break;
		}
		if( actualObj !is null && PlayerShip.getCurrentJob() !is null ) {
			if( isCompleted() == true ) {
				space.hud.addMessage("Job Completed: Reward Granted!",20,255,20);
				playerCredits += reward;
				updatePlayerFaction(getFaction(), cast(int)(getImportance()/12.5), true); 
				PlayerShip.cancelJob(true);
			} else if( actualObj.isCompleted() == true ) {
				space.hud.addMessage("Objective Completed",20,255,20);
				setWaypointsFor(PlayerShip);
			}
		}
	}
	void setWaypointsFor(FU_SHIP targetShip, bool showMessage = true ) {
		double gX, gY; int gL;
		foreach(int i, FU_OBJECTIVE jO; jobObjective) {
			if( jO.isCompleted() == false ) {
				jO.getInfo(&gX, &gY, &gL);
				targetShip.setWaypoint(gX, gY, i);		
				if( targetShip is PlayerShip && showMessage == true )
					space.hud.addMessage("New Waypoint Set.",255,20,255);
				return;
			}
		}
	}
	bool displayClicked(int mouseButton = LEFT) {
		int w = DISPLAY_WIDTH[disp_mode];
		int h = DISPLAY_HEIGHT[disp_mode];
		if( arc.io.input.mouseHit(mouseButton) == false ) return false;
		if( arc.io.input.mouseX >= disp_x && arc.io.input.mouseY >= disp_y &&
			arc.io.input.mouseX <= disp_x+w && arc.io.input.mouseY <= disp_y+h ) return true;
		return false;
	}
};