﻿module classes.objectholder; 

private import classes.particle; 
private import arc.templates.dlinkedlist;

private import classes.fu_object;
private import classes.item;
private import classes.ship;
private import classes.projectile;
private import space.hud;

private import space.stars;
private import init.init;
private import derelict.sdl.keysym;
private import space.generation;
private import space.storms;

/// FreeUniverse's object hander, all game objects are put in here
struct ObjectHolder
{
  public: 

	/// construct all game objects 
	void init()
	{
		SpaceParticles = new dlinkedlist!(FU_PARTICLE);

		// FU_OBJECT's 
		SpaceRocks = new dlinkedlist!(FU_OBJECT); 
		SpaceShips = new dlinkedlist!(FU_OBJECT); 
		SpaceStations = new dlinkedlist!(FU_OBJECT); 
		Planets = new dlinkedlist!(FU_OBJECT); 
		Items = new dlinkedlist!(FU_OBJECT); 
		Telegates = new dlinkedlist!(FU_OBJECT); 
		Transpaths = new dlinkedlist!(FU_OBJECT); 
		Projectiles = new dlinkedlist!(FU_OBJECT); 
	}

	/// clear all game objects 
	void clear()
	{
		SpaceParticles.clear();
		SpaceRocks.clear(); 
		SpaceShips.clear(); 
		SpaceStations.clear(); 
		Planets.clear(); 
		Items.clear(); 
		Telegates.clear(); 
		Transpaths.clear(); 
		Projectiles.clear(); 
	}


	void restoreAll()
	{
		SpaceRocks.restore();
		SpaceShips.restore(); 
		SpaceStations.restore(); 
		Planets.restore(); 
		Items.restore(); 
		Telegates.restore(); 
		Transpaths.restore(); 
		Projectiles.restore(); 
	}

	void resetAll()
	{
		SpaceRocks.reset();  
		SpaceShips.reset(); 
		SpaceStations.reset(); 
		Planets.reset(); 
		Items.reset(); 
		Telegates.reset(); 
		Transpaths.reset(); 
		Projectiles.reset(); 
	}

	void backupAll()
	{
		SpaceRocks.backup();  
		SpaceShips.backup(); 
		SpaceStations.backup(); 
		Planets.backup(); 
		Items.backup(); 
		Telegates.backup(); 
		Transpaths.backup(); 
		Projectiles.backup(); 
	}


	/// add right object into correct list 
	void add(FU_OBJECT newObj)
	{
		switch(newObj.getType())
		{
			case OBJECT_TYPE.ROCK:
				SpaceRocks.add(newObj); 
			break; 

			case OBJECT_TYPE.SHIP:
				SpaceShips.add(newObj); 
			break; 

			case OBJECT_TYPE.STATION:
				SpaceStations.add(newObj);
			break; 

			case OBJECT_TYPE.PLANET:
				Planets.add(newObj); 
			break; 

			case OBJECT_TYPE.ITEM:
				Items.add(newObj); 
			break; 

			case OBJECT_TYPE.TELEGATE:
			case OBJECT_TYPE.PORTHOLE:
				Telegates.add(newObj); 
			break; 

			case OBJECT_TYPE.TRANSPATH:
				Transpaths.add(newObj); 
			break; 

			case OBJECT_TYPE.PROJECTILE:
				Projectiles.add(newObj); 
			break; 
		}
	}

	// total number of all objects 
	int size()
	{
		return 
			SpaceRocks.size()+ 
			SpaceShips.size()+ 
			SpaceStations.size()+ 
			Planets.size()+ 
			Items.size()+ 
			Telegates.size()+ 
			Transpaths.size()+ 
			Projectiles.size(); 
	}

	/// remove current object 
	void remove()
	{
		switch( iterPos ) {
		case 0:
			return;
		case 1:
			Planets.remove();
			return;
		case 2:
			SpaceStations.remove(); 
			return;
		case 3:
			Transpaths.remove();
			return;
		case 4:
			Telegates.remove();
			return;
		case 5:
			SpaceRocks.remove(); 
			return;
		case 6:
			Items.remove();
			return;
		case 7:
			Projectiles.remove();
			return;
		case 8:
			SpaceShips.remove();
			return;
		}
		/* if (iterSpaceRocks == true)
		{
		}
		else if (iterSpaceShips == true)
		{
		}
		else if (iterSpaceStations == true)
		{
		}
		else if (iterPlanets == true)
		{
		} 
		else if (iterItems == true)
		{
		}
		else if (iterTelegates == true)
		{
		}
		else if (iterTranspaths == true)
		{
		}
		else if (iterProjectiles == true)
		{
		}  */
	}

	// allows for foreach iteration over everything except for FU_PARTICLES 
	// because FU_PARTICLE is not an FU_OBJECT 
	int opApply(int delegate(inout FU_OBJECT) dg)
	{
		int result = 0; 
		int count = 0;
		FU_OBJECT curr; 

		backupAll();
		resetAll(); 

		//iterSetFalse(); 

		// Iterate through planets ////////////////
		//iterPlanets = true; 
		iterPos = 1;
		
		while(!Planets.last)
		{
			curr = Planets.data; 
			result = dg(curr);
			if(result) return result;
		} 

		//iterPlanets = false; 
		iterPos = 2;

		// Iterate through space stations ////////////////
		//iterSpaceStations = true; 
		
		while(!SpaceStations.last)
		{
			curr = SpaceStations.data; 
			result = dg(curr);
			if(result) return result;
		} 

		//iterSpaceStations = false; 
		iterPos = 3;

		// Iterate through transpaths ////////////////
		//iterTranspaths = true; 
		
		while(!Transpaths.last)
		{
			curr = Transpaths.data; 
			result = dg(curr);
			if(result) return result;
		} 

		//iterTranspaths = false; 

		// Iterate through telegates ////////////////
		//iterTelegates = true; 
		iterPos = 4;
		
		while(!Telegates.last)
		{
			curr = Telegates.data; 
			result = dg(curr);
			if(result) return result;
		} 

		//iterTelegates = false; 

		// Iterate through space rocks ////////////////
		//iterSpaceRocks = true; 
		iterPos = 5;
		
		while(!SpaceRocks.last)
		{
			curr = SpaceRocks.data; 
			result = dg(curr);
			if(result) return result;
		} 

		//iterSpaceRocks = false; 

		// Iterate through items ////////////////
		//iterItems = true; 
		iterPos = 6;
		
		while(!Items.last)
		{
			curr = Items.data; 
			result = dg(curr);
			if(result) return result;
		} 

		//iterItems = false; 

		// Iterate through projectiles ////////////////
		//iterProjectiles = true; 
		iterPos = 7;
		
		while(!Projectiles.last)
		{
			curr = Projectiles.data; 
			result = dg(curr);
			if(result) return result;
		} 

		//iterProjectiles = false; 

		// Iterate through space ships ////////////////
		//iterSpaceShips = true;
		iterPos = 8;
		
		while(!SpaceShips.last)
		{ 
			curr = SpaceShips.data; 
			result = dg(curr);
			if(result) return result;
		} 

		//iterSpaceShips = false; 
		iterPos = 0;

		restoreAll(); 

		return result; 
	}

	// set all iterator indicators to false 
	/* void iterSetFalse()
	{
		iterSpaceRocks = false;
		iterSpaceStations = false; 
		iterSpaceStations = false;
		iterTelegates = false;
		iterTranspaths = false; 
		iterProjectiles = false; 
		iterPlanets = false;
		iterItems = false; 
	} */
	
	// serializes the objects currently held
	void describe(T)(T ar)
	{
		ar.describe(SpaceRocks);
		ar.describe(SpaceStations);
		ar.describe(Planets);
		ar.describe(Telegates);
		ar.describe(Transpaths);
	}

	/// special case add particle 
	void addParticle(FU_PARTICLE newObj)
	{
		SpaceParticles.add(newObj);
	}

	/// special case draw particles 
	void drawParticles()
	{
		SpaceParticles.reset();
		while(!SpaceParticles.first()) {
			if( SpaceParticles.data.draw() == false ) {
				FU_PARTICLE.free(SpaceParticles.data);
				SpaceParticles.remove();
			}
		}	
	}

  private:
	// NOTE: for each new linked list you add of type FU_OBJECT, make sure to 
	//		1) Add iterator bool 
	//		2) Apply correct opApply code and iterSetFalse code 
	// 		3) Special Case add Switch 
	//		4) Add init code
	//		5) Add clear code
	//		6) Add to .size code 
	//		7) Add to resetAll, restoreAll, and backupAll
	//		8) Apply correct remove code 
  
	// big linked list of all loaded space objects
	dlinkedlist!(FU_PARTICLE) SpaceParticles;
	dlinkedlist!(FU_OBJECT) SpaceRocks; 
	dlinkedlist!(FU_OBJECT) SpaceShips;
	dlinkedlist!(FU_OBJECT) SpaceStations; 
	dlinkedlist!(FU_OBJECT) Planets;
	dlinkedlist!(FU_OBJECT) Items; 
	dlinkedlist!(FU_OBJECT) Telegates;
	dlinkedlist!(FU_OBJECT) Transpaths; 
	dlinkedlist!(FU_OBJECT) Projectiles; 

	// whether or not we are currently iterating in this list
	// needed by remove, so remove knows which list to remove from 
	int iterPos=0;
	/* bool 
		iterSpaceRocks=false,
		iterSpaceShips=false,
		iterSpaceStations=false,
		iterPlanets=false,
		iterItems=false,
		iterTelegates=false,
		iterTranspaths=false,
		iterProjectiles=false; */
}