﻿module classes.projectile;

private import init.init;
private import classes.fu_object;
private import classes.ship;
private import classes.item;
private import classes.particle;

class FU_PROJECTILE: FU_OBJECT {
private:
	float damage, accel;
	float shieldModifier, armorModifier;
	int itemClassEffect, itemTypeEffect;
	int modifierShot, modifierEffect;
	uint remaining;
	bool smokeTrail = false;
	float homing;
	FU_OBJECT target;
	FU_SHIP owner;
public:
	this(FU_SHIP sourceShip, FU_ITEM sourceWeapon) {
		super(OBJECT_TYPE.PROJECTILE, TEXTURES.PROJECTILE, sourceWeapon.getVars().projTexValue );
		setColor(sourceWeapon.getVars().r, sourceWeapon.getVars().g, sourceWeapon.getVars().b, 255);
		setLevel(sourceWeapon.getLevel());
		damage = sourceWeapon.getVars().total_damage * sourceWeapon.getVars().quality;
		shieldModifier = sourceWeapon.getVars().shield_modifier;
		armorModifier = sourceWeapon.getVars().armor_modifier;
		itemClassEffect = sourceWeapon.getVars().itemClassEffect;
		itemTypeEffect = sourceWeapon.getVars().itemTypeEffect;
		accel = sourceWeapon.getVars().acceleration;
		max_speed = sourceWeapon.getVars().maximum;
		homing = sourceWeapon.getVars().homing_ability;
		remaining = sourceWeapon.getVars().duration;
		setPosition(sourceShip.getX(), sourceShip.getY());
		if( sourceWeapon.getClass() == ITEM_CLASS.MODIFIER ) {
			modifierShot = 1 + (sourceWeapon.getVars().modifierEffect / MODIFIER_SEPERATOR);
			modifierEffect = sourceWeapon.getVars().modifierEffect;
		} else {
			float damIncrease = damage * 0.3;
			if( sourceShip.sapModifier(MODIFIER_TYPE.INCREASE_DAMAGE,damIncrease) == true )
				damage += damIncrease;
			if( sourceWeapon.getClass() == ITEM_CLASS.TORPEDO ||
				sourceWeapon.getClass() == ITEM_CLASS.MISSLE ) smokeTrail = true;
		}
		owner = sourceShip;
		if( sourceShip !is PlayerShip && sourceShip.getAIAction() == AI_ACTION.ATTACKING &&
			sourceWeapon.isGoodModifier() == true )
				target = sourceShip;
		else target = sourceShip.getTarget();
		if( objectExists(target) == false /*|| target is owner && sourceWeapon.isGoodModifier() == false*/ )
			target = null;
		if( sourceWeapon.getVars().aimMode == AIM_MODE.TURRET && objectExists(target) ) {
			setRotation(pointAtAngle(getX(), getY(), target.getX(), target.getY()));
		} else if( sourceWeapon.getVars().aimMode == AIM_MODE.SPREAD ) {
			float newAngle = sourceShip.getRotation();
			newAngle += random_range(-20,20);
			setRotation(newAngle);
		} else setRotation(sourceShip.getRotation());
		if( sourceWeapon.getVars().aimMode != AIM_MODE.TURRET ) {
			setXSpeed(sourceShip.getXSpeed());
			setYSpeed(sourceShip.getYSpeed());
		} else {
			setXSpeed(sourceShip.getXSpeed()/2);
			setYSpeed(sourceShip.getYSpeed()/2);
		}
		accelerate(accel);
	}
	~this() { }
	void handle_object() {
		debug scope(failure) { writefln("handle_object: projectile.d"); }
		if( homing > 0 ) {
			accelerate(accel);
			x_speed *= 0.94;
			y_speed *= 0.94;
		}
		super.handle_object();
		if( smokeTrail == true && rand()%3==0 ) {
			FU_PARTICLE newParticle = FU_PARTICLE.allocate();
			newParticle.construct(this,255,255,255,4,true);	
		}
		if( modifierShot != 0 && rand()%3==0 ) {
			switch( modifierShot ) {
			case 1: // bad modifier
				FU_PARTICLE newParticle = FU_PARTICLE.allocate();
				newParticle.construct(this,255,20,20,8,false);
				break;
			case 2: // good modifier
				FU_PARTICLE newParticle = FU_PARTICLE.allocate();
				newParticle.construct(this,20,255,20,8,false);
				break;
			}
		}
		if( objectExists(target) == true ) {
			// collision detection with target
			if( homing > 0 ) {
				double intX = getIntercept(super.getMaxSpeed() * 0.8,getX(),target.getXSpeed(),target.getX()); 
				double intY = getIntercept(super.getMaxSpeed() * 0.8,getY(),target.getYSpeed(),target.getY()); 
				super.objectTurnTowards(7.0 * homing, intX, intY);
			}
			if( target.getSprite().collide(super.getSprite()) == true && target.getType() != OBJECT_TYPE.PORTHOLE ) {
				// hit!
				if( owner is PlayerShip && rand()%3==0 && target.getType() != OBJECT_TYPE.ITEM &&
										   target.getType() != OBJECT_TYPE.ROCK ) {
					updatePlayerFaction(target.getFaction(),1,(modifierShot==2));
				}
				if( target.getType() == OBJECT_TYPE.SHIP ) {
					FU_SHIP tShip = cast(FU_SHIP)target;
					// apply any effect
					if( modifierShot != 0 ) tShip.setModifier(modifierEffect, damage);
					// disable any transpath docking state of tShip
					tShip.disableTransTravel();
					float dmgAmount = tShip.percentArmorRemaining();
					if( target.damage(damage,armorModifier,shieldModifier,tShip) == false ) {
						dmgAmount = dmgAmount - tShip.percentArmorRemaining();
						dmgAmount /= cast(float)random_range(2,8) / 6.0;
						FU_ITEM itemToDamage;
						int itemCount;
						if( itemTypeEffect != ITEM_TYPE.NULL ) {
							tShip.items.reset();
							while(!tShip.items.last()) {
								if( tShip.items.data.getType() == itemTypeEffect &&
									tShip.items.data.getClass() == itemClassEffect &&
									tShip.items.data.getVars().quality > 0 ) {
										itemToDamage = tShip.items.data;
										itemToDamage.disableActivation();
										break;
								}
							}
						}
						if( itemToDamage is null ) {
							int itemType = (rand()%ITEM_TYPE.max)+1;
							itemCount = tShip.getItemCount(itemType);
							if( itemCount > 0 ) itemToDamage = tShip.getItem(itemType, WILDCARD, rand()%itemCount); 
						}
						if( itemToDamage !is null ) {
							itemToDamage.getVars().quality -= dmgAmount;
							if( itemToDamage.getVars.quality <= 0 ) {
								itemToDamage.getVars.quality = 0;
								itemToDamage.forceUninstall();
							}
						}
					} else playSound(SOUND_CLASS.SHIELD_HIT,getX(),getY());
					if( tShip !is PlayerShip ) {
						if( modifierShot != 2 ) {
							switch( tShip.getAIClass() ) {
							case AI_CLASS.TOURIST:
								if( tShip.getAIAction() != AI_ACTION.DOCKING ) {
									tShip.setTarget(owner);
									tShip.setAIAction(AI_ACTION.RUNNING);
								}
								break;
							case AI_CLASS.NONE:
							case AI_CLASS.MINER:
							case AI_CLASS.TRAVELER:
							case AI_CLASS.EXPLORER:
							case AI_CLASS.POLICE:
							case AI_CLASS.PIRATE:
							case AI_CLASS.KILLER:
								if( tShip.getAIAction != AI_ACTION.ATTACKING ||
									tShip.getTarget() !is null && tShip.getTarget().getType() != OBJECT_TYPE.SHIP ||
									distance(tShip.getTarget().getX(), tShip.getTarget().getY(),tShip.getX(),tShip.getY()) > MAX_TARGETABLE ) {
									tShip.setTarget(owner);
									tShip.setAIAction(AI_ACTION.ATTACKING);
								}
								break;
							}
						} else tShip.allyRequest(owner);
					} else if( tShip.getTarget() is null ) tShip.setTarget(owner);
					// did we save someone?
					if( tShip.getArmor() <= 0 && owner is PlayerShip &&
						tShip.getTarget() !is null &&
						tShip.getAIAction == AI_ACTION.ATTACKING &&
						tShip.getTarget() !is PlayerShip ) {
							long giveMoney = cast(long) tweakValue(tShip.getTarget().getLevel()*60,40);
							space.hud.addMessage("Thanks! +" ~ std.string.toString(giveMoney) ~ " Credits",20,255,20);
							playerCredits += giveMoney;
					} 
				} else {
					target.damage(damage,armorModifier,shieldModifier);
					playSound(SOUND_CLASS.SMALL_EXPLOSION,getX(),getY());
				}
				FU_PARTICLE newParticle = FU_PARTICLE.allocate();
				newParticle.construct(this,255,60,0,rand()%10+7);
				invalidate();
			}
		}
		remaining--;
		if( remaining == 0 ) invalidate();
	}
}
