﻿module classes.ship;

private import init.init;
private import classes.item;
private import classes.fu_object;
private import classes.projectile;
private import classes.particle;
private import classes.job;
private import space.storms;
static import space.hud;

enum AI_CLASS {
	NONE,			// default, no AI class
	TOURIST,		// not agressive, boring, runs from fights, does nothing specific
	TRAVELER,		// stays along transpaths, docks with planets, stations, telegates
	POLICE,			// monitors near planets, stations, searches for contraband, defensive
	EXPLORER,		// rarely docks with anything, flys around, sometimes runs, sometimes fights
	PIRATE,			// same as police, but doesn't patrol
	KILLER,			// picks fights with different factions
	MINER			// destroys rocks and picks up items, finds stations to dock with when full
};

enum AI_ACTION {
	WANDERING,		// flying aimlessly, searching for targets
	DOCKING,		// trying to dock with target
	PATROLING,		// flying towards, or near destination coordinates
	RUNNING,		// fleeing from destination coordinates
	ATTACKING,		// engaged in target
	FOLLOWING,		// following leader
	PICKUP			// picking up item
};

// ship types
// use these for variations on AI, item population
enum SHIP_TYPE {
//	ABANDONED,		// no driver, sparse (if any) items
	STANDARD,		// nothing special, sensors, 1 or 2 weapons, random items
	POLICE,		// many disable-type weapons (effect engines, weapons), store contraband
	PIRATE,		// lots of contraband, aggressive, weapons
	FIGHTER,		// many weapons, light and fast
	CRUISER,		// big ship, heavy and slow, many weapons and shields, dockable?
	SHUTTLE,		// no weapons, basic cargo items (e.g. food), boring
	CARRIER		// cargo ship, not many weapons, lots of cargo stuff
}

class FU_SHIP : FU_OBJECT {
private:
	// item information
	float max_weight;
	int max_utility, max_weapons;
	// ship type
	int ship_type;
	int stormProtection;
	// intrinsic weight, value
	float hull_weight;
	long hull_value;
	// object target does not need to be saved
	// make this null upon loading
	FU_OBJECT target;
	// leader in some formation
	FU_SHIP leader;
	// current job
	FU_JOB currentJob;
	// sound info
	int soundIndex = -1;
	bool thrustUsed = false;
	// AI integers
	int playerActionTimer;
	int attackMode, straferMode;
	int AIClass = AI_CLASS.NONE;
	int AIAction = AI_ACTION.WANDERING;
	bool usingCruiser;
	// display drawing stuff
	int disp_mode, disp_x, disp_y;
	bool ownItem(FU_ITEM itemCheck) {
		items.reset();
		while(!items.last()) {
			if( items.data is itemCheck ) {
				return true;
			}
		}
		return false;
	}
	void setCorrectSize() {
		switch( ship_type ) {
//		case SHIP_TYPE.ABANDONED:
		case SHIP_TYPE.STANDARD:
			getSprite().setSize(24,24);
			break;
		case SHIP_TYPE.POLICE:
		case SHIP_TYPE.PIRATE:
			getSprite().setSize(30,30);
			break;
		case SHIP_TYPE.FIGHTER:
			getSprite().setSize(28, 28);
			break;
		case SHIP_TYPE.CRUISER:
			getSprite().setSize(50, 50);
			break;
		case SHIP_TYPE.SHUTTLE:
			getSprite().setSize(20, 20);
			break;
		case SHIP_TYPE.CARRIER:
			getSprite().setSize(40, 50);
			break;
		}
	}
	// waypoints
	double wpX, wpY;
	int WPIndex = -1;
	// job modifiers
	bool forcePatrol = false, forceWander = false;
	float effectModifier[];
public:
	// save all these to store ship
	dlinkedlist!(FU_ITEM) items;
	this(int parmShipType, int parmLevel, bool populateShip) {
		// should grab its own image on what type of ship it is
		int texClass = TEXTURES.SHIP;
		if( parmLevel < 1 ) parmLevel = 1;
		items = new dlinkedlist!(FU_ITEM);
		ship_type = parmShipType;
		max_utility = 6;
		max_weapons = 0;
		currentJob = null;
		max_weight = (19 * parmLevel);
		super(OBJECT_TYPE.SHIP, texClass);
		setLevel(parmLevel);
		effectModifier.length = MODIFIER_TYPE.max+1;
		for(int i=0;i<effectModifier.length;i++) effectModifier[i]=0;
		switch( ship_type ) {
//		case SHIP_TYPE.ABANDONED:
		case SHIP_TYPE.STANDARD:
			max_weapons += rand()%2+1;
			max_utility += rand()%2+1;
			setMaxArmor(tweakValue(cast(float)(100 + 25 * (parmLevel-1)),10));
			hull_weight = tweakValue(parmLevel * 3.0, 25);
			hull_value = cast(long)tweakValue(cast(float)((parmLevel * 400) - (hull_weight * 3)),25);
			break;
		case SHIP_TYPE.POLICE:
		case SHIP_TYPE.PIRATE:
			max_weight += (parmLevel * 10);
			max_weapons += rand()%2+3;
			max_utility += rand()%2+2;
			setMaxArmor(tweakValue(cast(float)(200 + 35 * (parmLevel-1)),10));
			hull_weight = tweakValue(parmLevel * 6.0, 25);
			hull_value = cast(long)tweakValue(cast(float)((parmLevel * 600) - (hull_weight * 3)),25);
			break;
		case SHIP_TYPE.FIGHTER:
			max_weight += (parmLevel * 6);
			max_weapons += rand()%2+4;
			max_utility += rand()%2+1;
			setMaxArmor(tweakValue(cast(float)(150 + 30 * (parmLevel-1)),25));
			hull_weight = tweakValue(parmLevel * 4.5, 25);
			hull_value = cast(long)tweakValue(cast(float)((parmLevel * 500) - (hull_weight * 3)),25);
			break;
		case SHIP_TYPE.CRUISER:
			max_weight += (parmLevel * 26);
			max_weapons += rand()%3+4;
			max_utility += rand()%2+4;
			setMaxArmor(tweakValue(cast(float)(300 + 45 * (parmLevel-1)),10));
			hull_weight = tweakValue(parmLevel * 12.0, 25);
			hull_value = cast(long)tweakValue(cast(float)((parmLevel * 900) - (hull_weight * 3)),25);
			break;
		case SHIP_TYPE.SHUTTLE:
			max_weight -= (parmLevel * 4);
			setMaxArmor(tweakValue(cast(float)(75 + 20 * (parmLevel-1)),10));
			hull_weight = tweakValue(parmLevel * 1.5, 25);
			hull_value = cast(long)tweakValue(cast(float)((parmLevel * 300) - (hull_weight * 3)),25);
			break;
		case SHIP_TYPE.CARRIER:
			max_weight += (parmLevel * 24);
			max_utility += rand()%2+3;
			max_weapons += rand()%2+2;
			setMaxArmor(tweakValue(cast(float)(250 + 40 * (parmLevel-1)),10));
			hull_weight = tweakValue(9.0 * parmLevel, 25);
			hull_value = cast(long)tweakValue(cast(float)((parmLevel * 750) - (hull_weight * 3)),25);
			break;
		}
		setCorrectSize();
		hull_value += (max_utility + max_weapons) * parmLevel * 25;
		hull_value += getMaxArmor();
		setArmor(getMaxArmor());
		usingCruiser = false;
		playerActionTimer = 0;
		stormProtection = random_range(1,STORM_TYPE.max);
		straferMode = 0;
		attackMode = random_range(0,2);
		if( populateShip ) populate();
		updateWeight();
		updateValue();
	}
	~this() {}
	// default constructible, in order to be easily serializable
	this() {}
	void describe(T)(T ar) {
		super.describe(ar);
		ar.describe(max_weight);
		ar.describe(max_utility);
		ar.describe(max_weapons);
		ar.describe(ship_type);
		ar.describe(hull_weight);
		ar.describe(hull_value);
		ar.describe(stormProtection);
		ar.describe(items);
		ar.describe(AIClass);
		ar.describe(AIAction);
		
		if( ar.reading )
		{
			effectModifier.length = MODIFIER_TYPE.max+1;
			for(int i=0;i<effectModifier.length;i++) effectModifier[i]=0;
			usingCruiser = false;
			straferMode = 0;
			attackMode = random_range(0,2);
			setCorrectSize();			
		}
	}
	float getModifierAmount(int modifierType) {
		if( modifierType < 0 || modifierType > MODIFIER_TYPE.max ) return 0;
		return effectModifier[modifierType];
	}
	/// returns true if the amount of modifier was available before subtraction
	bool sapModifier(int modifierType, float amount) {
		if( modifierType < 0 || modifierType > MODIFIER_TYPE.max ) return false;
		if( amount < 0 ) amount = -amount;
		if( effectModifier[modifierType] < amount ) {
			effectModifier[modifierType] = 0;
			return false;
		}
		effectModifier[modifierType] -= amount;
		return true;
	}
	void setModifier(int modifierType, float amount) {
		if( modifierType < 0 || modifierType > MODIFIER_TYPE.max ) return;
		if( effectModifier[modifierType] < amount ) effectModifier[modifierType] = amount;
	}
	void cancelJob(bool skipMessage = false) {
		if( currentJob !is null && skipMessage == false ) space.hud.addMessage("Job Failed!",255,20,20);
		currentJob = null;
		WPIndex = -1;
	}
	FU_JOB getCurrentJob() { return currentJob; }
	void setJob(FU_JOB newJob) { currentJob = newJob; }
	bool acceptNewJob(FU_JOB newJob) {
		if( currentJob !is null ) return false;
		if( newJob.activateJob() == false ) return false;
		currentJob = newJob;
		space.hud.addMessage("New Job Accepted",20,255,20);
		return true;
	}
	int allyRequest(FU_SHIP host) {
		if( objectExists(host) == false ) return -1;
		if( this is PlayerShip ) return -1;
		if( host.getLeader() is this || getLeader() is host ) return -6;
		if( distance(host.getX(), host.getY(), getX(), getY()) > MAX_TARGETABLE ) return -5;
		if( AIAction != AI_ACTION.WANDERING && AIAction != AI_ACTION.PATROLING || forceWander == true ) return -2;
		if( getFriction() == false || host.getFriction() == false ) return -2;
		if( abs( getFaction() - host.getFaction() ) > 1 ) return -3;
		if( host.getLevel() < (getLevel() - 1) ) return -4;
		setLeader(host);
		setTarget(null);
		setAIAction(AI_ACTION.FOLLOWING);
		return 0;
	}
	void setForceWanderFlag(bool newFlag) { forceWander = newFlag; }
	void setForcePatrolFlag(bool newFlag) { forcePatrol = newFlag; }
	long purchaseCost(float *weightCheck = null) {
		if( weightCheck != null ) *weightCheck = (max_weight - hull_weight) - (PlayerShip.getWeight() - PlayerShip.getHullWeight());
		return getHullValue() - cast(long)(cast(float)PlayerShip.getHullValue() * SELLBACK);
	}
	void drawDisplay(int parmX, int parmY, int parmDrawMode, bool useGreen = false) {
		drawDarkBox(parmX, parmY, DISPLAY_WIDTH[parmDrawMode], DISPLAY_HEIGHT[parmDrawMode]);
		short dr, dg, db;
		float weightCheck, weightAvail = (max_weight - super.weight);
		long shipCost = purchaseCost(&weightCheck);
		if( useGreen == true ) { dg = 255; db = 20; dr = 20; }
		else { dg = 255; db = 255; dr = 255; }
		disp_x = parmX;
		disp_y = parmY;
		disp_mode = parmDrawMode;
		switch( disp_mode ) {
		case ITEMDISPLAY_MODE.COMPACT:
			break;
		case ITEMDISPLAY_MODE.NORMAL:
			drawImage(disp_x + DISPLAY_WIDTH[disp_mode] - 21, disp_y + 21, 40, 40, getSprite().getR(), getSprite().getG(), getSprite().getB(),175, 0, 0, 0, getSprite().getCurrFrame().getID);
			standardFont.draw("Ship: " ~ ShipTypeNames[ship_type], disp_x + 2, disp_y + 1, dr, dg, db, 255);
			standardFont.draw("Cost: " ~ std.string.toString(shipCost), disp_x + 2, disp_y + 11, dr, dg, db, 255);
			standardFont.draw("Level: " ~ std.string.toString(getLevel()), disp_x + 2, disp_y + 21, dr, dg, db, 255);
			standardFont.draw("Max Utilities: " ~ std.string.toString(max_utility), disp_x + 2, disp_y + 31, dr, dg, db, 255);
			standardFont.draw("Max Weapons: " ~ std.string.toString(max_weapons), disp_x + 2, disp_y + 41, dr, dg, db, 255);
			standardFont.draw("Wgt Avail: " ~ std.string.toString(weightAvail), disp_x + 2, disp_y + 51, dr, dg, db, 255);
			standardFont.draw("Max Armor: " ~ std.string.toString(getMaxArmor()), disp_x + 2, disp_y + 61, dr, dg, db, 255);
			break;
		case ITEMDISPLAY_MODE.DETAILED:
			bigFont.draw("Ship Classification: " ~ ShipTypeNames[ship_type], disp_x + 2, disp_y + 2, 255, 255, 255, 255);
			bigFont.draw("Hull Value: " ~ std.string.toString(getValue()), disp_x + 2, disp_y + 17, 255, 255, 255, 255);
			bigFont.draw("Level: " ~ std.string.toString(getLevel()), disp_x + 2, disp_y + 32, 255, 255, 255, 255);
			bigFont.draw("Max Utilities: " ~ std.string.toString(max_utility), disp_x + 2, disp_y + 50, 200, 255, 200, 255);
			bigFont.draw("Max Weapons: " ~ std.string.toString(max_weapons), disp_x + 320, disp_y + 50, 200, 255, 200, 255);
			bigFont.draw("Hull Weight: " ~ std.string.toString(hull_weight), disp_x + 2, disp_y + 65, 255, 200, 200, 255);
			bigFont.draw("Max Weight: " ~ std.string.toString(max_weight), disp_x + 320, disp_y + 65, 255, 200, 200, 255);
			bigFont.draw("Weight Available: " ~ std.string.toString(weightAvail), disp_x + 2, disp_y + 80, 255, 200, 200, 255);
			bigFont.draw("Current Armor: " ~ std.string.toString(getArmor()), disp_x + 2, disp_y + 100, 255, 255, 255, 255);
			bigFont.draw("Max Armor: " ~ std.string.toString(getMaxArmor()), disp_x + 320, disp_y + 100, 255, 255, 255, 255);
			bigFont.draw("Storm Protection: " ~ stormNames[stormProtection], disp_x + 2, disp_y + 120, 255, 255, 255, 255);  
			if( this !is PlayerShip ) {
				bigFont.draw("Purchase Cost: " ~ std.string.toString(shipCost), disp_x + 320, disp_y + 17, 255, 255, 255, 255);
				bigFont.draw("Weight Available after Item Transfer: " ~ std.string.toString(weightCheck), disp_x + 2, disp_y + 140, 255, 200, 200, 255);
				if( weightCheck < 0 ) bigFont.draw("You cannot fit all of your items on this ship.", disp_x + 2, disp_y + 155, 255, 200, 200, 255);
				else bigFont.draw("You can fit all your items on this ship.", disp_x + 2, disp_y + 155, 255, 200, 200, 255);
			} else {
				bigFont.draw("This is your ship.", disp_x + 2, disp_y + 140, 255, 255, 255, 255);
			}
			bigFont.draw("Physical Dimensions: " ~ std.string.toString(cast(int)(getSprite().getWidth())) ~ " x " ~ std.string.toString(cast(int)(getSprite().getHeight())), disp_x + 2, disp_y + 175, 255, 255, 255, 255);
			drawImage(disp_x + DISPLAY_WIDTH[disp_mode]/2, disp_y + DISPLAY_HEIGHT[disp_mode] - 80, 60, 60, getSprite().getR(), getSprite().getG(), getSprite().getB(),175, 0, 0, 0, getSprite().getCurrFrame().getID);
			break;
		}
	}
	bool displayClicked(int mouseButton = LEFT) {
		int w = DISPLAY_WIDTH[disp_mode];
		int h = DISPLAY_HEIGHT[disp_mode];
		if( arc.io.input.mouseHit(mouseButton) == false ) return false;
		if( arc.io.input.mouseX >= disp_x && arc.io.input.mouseY >= disp_y &&
			arc.io.input.mouseX <= disp_x+w && arc.io.input.mouseY <= disp_y+h ) return true;
		return false;
	}
	void setWaypoint(double argx, double argy, int objectiveIndex) {
		wpX = argx;
		wpY = argy;
		WPIndex = objectiveIndex;
	}
	void stopSounds() {
		if( soundIndex > -1 ) stopSound(soundIndex);
		soundIndex = -1;
		thrustUsed = false;
	}
	double getWaypointX() {	return wpX; }
	double getWaypointY() {	return wpY; }
	int WaypointIndex() { return WPIndex; }
	void clearWaypoint() { WPIndex = -1; }
	float getHullWeight() { return hull_weight; }
	long getHullValue() { return hull_value; }
	float getMaxWeight() { return max_weight; }
	void updateWeight() {
		super.weight = hull_weight;
		items.reset();
		while(!items.last) { super.weight += items.data.getWeight(); }
	}
	void updateValue(bool includeArmor = true) {
		super.value = cast(long)(cast(float)hull_value);
		if( includeArmor == true ) super.value = cast(long)(cast(float)super.value * percentArmorRemaining());
		items.reset();
		while(!items.last) { super.value += items.data.getValue(); }
	}
	void setAIClass(int newClass) { AIClass = newClass; }
	void setAIAction(int newAction) {
		if( newAction == AI_ACTION.DOCKING ||
			newAction == AI_ACTION.PICKUP ) playerActionTimer = 0;
		if( newAction == AI_ACTION.ATTACKING && target is PlayerShip )
			space.hud.addMessage("You are targetted for attack!",255,20,20);
		AIAction = newAction;
	}
	int getAIClass() { return AIClass; }
	int getAIAction() { return AIAction; }
	FU_OBJECT getTarget() { return target; }
	FU_SHIP getLeader() { return leader; }
	void setLeader(FU_SHIP newLeader) { leader = newLeader; }
	void setTarget(FU_OBJECT newTarget) { target = newTarget; }
	int getShipType() { return ship_type; }
	// returns -1 if we are just too far away
	// returns -2 if we should stop trying to pick this item up
	// returns 0 on success
	int pickupTarget() {
		debug scope(failure) { writefln("pickupTarget: ship.d"); }
		if( objectExists(target) == false ) return -2;
		if( target.getType() != OBJECT_TYPE.ITEM ) {
			if( this is PlayerShip ) space.hud.addMessage("Cannot pickup this target.",255,255,20);
			return -2;
		}
		if( target.getItem() is null ) return -2;
		if( distance(getX(), getY(), target.getX(), target.getY()) > 40 ) {
			if( this is PlayerShip ) space.hud.addMessage("Not close enough to target.",255,255,20);
			return -1;
		}
		if( addItem(target.getItem(),cast(int)target.getItem().getVars().counter) == false ) {
			if( this is PlayerShip ) space.hud.addMessage("Cannot fit target on your ship.",255,255,20);
			return -2;
		}
		target.invalidate();
		return 0;
	}
	bool dockToTarget() {
		debug scope(failure) { writefln("dockToTarget: ship.d"); }
		if( objectExists(target) == false ) return false;
		if( target.getType() != OBJECT_TYPE.STATION && target.getType() != OBJECT_TYPE.PLANET &&
			target.getType() != OBJECT_TYPE.TRANSPATH && target.getType() != OBJECT_TYPE.TELEGATE &&
			target.getType() != OBJECT_TYPE.PORTHOLE ) {
				if( this is PlayerShip ) space.hud.addMessage("Invalid docking target.",255,255,20);
				return false;
		}
		if( getSprite().collide(target.getSprite()) == false ) {
			if( this is PlayerShip ) space.hud.addMessage("Target not close enough.",255,255,20);
			return false;
		}
		if( getSpeed() > 2 ) {
			if( this is PlayerShip ) space.hud.addMessage("You are traveling too fast.",255,255,20);
			return false;
		}
		if( this is PlayerShip && target.needsTrim() == true ) {
			space.hud.addMessage("Cannot dock with job target.",255,255,20);
			return false;
		}
		// we can dock!
		switch( target.getType() ) {
		case OBJECT_TYPE.PLANET:
		case OBJECT_TYPE.STATION:
			if( abs( getFaction() - target.getFaction() ) > 1 ) {
				if( this is PlayerShip ) space.hud.addMessage("Dock request rejected.",255,20,20);
				return false;
			}
			if( this !is PlayerShip ) {
				invalidate();
				return false;
			} else {
				PlayerShip.setDockInfo(target.getDockInfo());
				PlayerShip.getDockInfo().dockInit(target,this);
				gameState = GAME_STATE.DOCKED;
				MainGenerator.trimSpaceObjects(true);
				MainGenerator.resetStorm();
			}
			break;
		case OBJECT_TYPE.TRANSPATH:
			if( target.getDestinationX(1) != 0 || target.getDestinationY(1) != 0 ) {
				// middle transpath, throw player in general direction
				if( angleDistance(target.getRotation(),getRotation()) > angleDistance(target.getRotation()-180,getRotation()) ) {
					setRotation(target.getRotation()-180);
					setDestination(target.getDestinationX(),target.getDestinationY());
				} else {
					setRotation(target.getRotation());
					setDestination(target.getDestinationX(1),target.getDestinationY(1));
				}
			} else {
				setRotation( pointAtAngle(getX(),getY(),target.getDestinationX(),target.getDestinationY()) );
				setDestination( target.getDestinationX(), target.getDestinationY() );
			}
			setFriction(false);
			setXSpeed(moveToValueX(getRotation())*-12);
			setYSpeed(moveToValueY(getRotation())*-12);
			if( this is PlayerShip ) updateAction(AI_ACTION.DOCKING);
			break;
		case OBJECT_TYPE.PORTHOLE:
		case OBJECT_TYPE.TELEGATE:
			if( this !is PlayerShip ) {
				invalidate();
				return false;
			} else setPosition( target.getDestinationX(), target.getDestinationY() );
			break;
		}
		return true;
	}
	void updateAction(int newAction = -1) {
		if( newAction != -1 ) {
			AIAction = newAction;
			playerActionTimer = 330;
			return;
		}
		if( playerActionTimer > 0 ) {
			playerActionTimer--;
		} else AIAction = AI_ACTION.WANDERING;
	}
	void airBrakes() {
		x_speed *= 0.95;
		y_speed *= 0.95;
	}
	void handle_object() {
		debug scope(failure) { writefln("handle_object: ship.d"); }
		// set my alpha correctly
		short myAlpha = getAlpha();
		if( myAlpha < 255 ) setAlpha(myAlpha+1);
		// engine sound handling
		if( soundIndex == -2 ) soundIndex = playSound(SOUND_CLASS.ENGINE,getX(),getY(),WILDCARD,true,0.6);
		else if( thrustUsed == false ) stopSounds();
		else if( updateSound(soundIndex,getX(),getY(),getSpeed()) == false ) soundIndex = -1;
		thrustUsed = false;
		// handle transpath/friction stuff
		if( getFriction() == true && getSpeed() > ENGINE_SPEED ) {
			x_speed *= 0.985;
			y_speed *= 0.985;
		} else if( getFriction() == false && distance(getX(),getY(),
										     getDestinationX(),getDestinationY()) < 100 ) {
			disableTransTravel();
		}
		// handle FU_OBJECT stuff
		super.handle_object();
		// handle all of my items
		items.reset();
		while(!items.last) {
			// only handle installed items
			if( items.data.isInstalled() == true ) {
				items.data.handleItem(this);
			}
			if( items.data.getVars().quality < 1 && sapModifier(MODIFIER_TYPE.REPAIR_ITEMS,items.data.getLevel()*3) ) {
				items.data.getVars().quality += 0.003;
				if( items.data.getVars().quality > 1 ) items.data.getVars().quality = 1;
			}
		}
		// repair hull?
		if( sapModifier(MODIFIER_TYPE.REPAIR_HULL,getMaxArmor()*0.002) == true )
			damage(-(getMaxArmor()*0.001), 1.0, 0.0, null);
		// make sure my target still exists, and we can see it
		if( objectExists(target) == false || target.getType() == OBJECT_TYPE.SHIP &&
			canSee(this, cast(FU_SHIP)target) == false &&
			!(this is PlayerShip && target is PlayerShip)) setTarget(null);
		// handle AI (do nothing if we are on a transpath
		if( this !is PlayerShip ) {
			if( getFriction() == true ) {
				switch( AIAction ) {
				case AI_ACTION.WANDERING:		// flying aimlessly, searching for targets
					double dist = flyTowards(getDestinationX(),getDestinationY());
					if( dist < 300 && rand()%240==0 ) setDestination(MainGenerator.getCellAreaX(), MainGenerator.getCellAreaY());
					float current, max;
					modifyUtilityCounter(ITEM_CLASS.GENERATOR, 0, &current, &max);
					if( (current / max) > 0.6 ) cloak();
					break;
				case AI_ACTION.DOCKING:		// trying to dock with target
					if( target is null ) setAIAction(defaultAction());
					else if( playerActionTimer < 30 ) {
						// spend a moment slowing down and turning
						// this will make sure we have a clear shot to dock
						playerActionTimer++;
						turnTowards(target.getX(), target.getY());
						airBrakes();
					} else {
						if( getSprite().collide(target.getSprite()) == false ) flyTowards(getTarget().getX(), getTarget().getY());
						else if( target.getType() == OBJECT_TYPE.TRANSPATH && getLeader() !is null &&
							     angleDistance(getRotation(), pointAtAngle(getX(), getY(), getLeader().getX(), getLeader().getY())) > 80 ) {
							// we need to turn to my leader before docking
							airBrakes();
							turnTowards(getLeader().getX(), getLeader().getY());
						} else if( dockToTarget() == true ) {
							// we successfully docked
							// set my new 'home' location
							setDestination(getDestinationX(),getDestinationY(),1);
							setTarget(null);
							setAIAction(defaultAction());
						}
					}
					break;
				case AI_ACTION.PATROLING:		// flying near destination coordinates
					double dist = flyTowards(getDestinationX(), getDestinationY());
					if( dist < 150 || distance(getDestinationX(), getDestinationY(), getDestinationX(1), getDestinationY(1)) > 1250 ) {
						setDestination(getDestinationX(1) + random_range(-800,800), getDestinationY(1) + random_range(-800,800));
					}
					break;
				case AI_ACTION.RUNNING:		// fleeing from destination coordinates
					if( getTarget() !is null && getTarget() !is this ) {
						flyTowards(getDestinationX(), getDestinationY());
						double dist = distance(getDestinationX(), getDestinationY(), getTarget().getX(), getTarget().getY());
						if( dist < 2750 || getDestinationX() > MainGenerator.getCellLowerX()
										|| getDestinationX() < MainGenerator.getCellUpperX()
										|| getDestinationY() > MainGenerator.getCellLowerY()
										|| getDestinationY() < MainGenerator.getCellUpperY()) {
							double newX, newY;
							do {
								newX = MainGenerator.getCellAreaX();
								newY = MainGenerator.getCellAreaY();
							} while( distance(newX,newY,getTarget().getX(),getTarget().getY()) < 2750 );
							setDestination(newX, newY);
						}
						if( distance(getX(), getY(), getTarget().getX(), getTarget().getY()) > 2600 ) {
							setTarget(null);
							setAIAction(defaultAction());
						}
						// use cloaking if enough energy
						float current, max;
						modifyUtilityCounter(ITEM_CLASS.GENERATOR, 0, &current, &max);
						if( max > 0 && (current / max) > 0.4 ) cloak();
					} else {
						setTarget(null);
						setAIAction(defaultAction());
					}
					break;
				case AI_ACTION.ATTACKING:		// engaged in target
					if( getTarget() is null || getTarget() is this ) {
						setTarget(null);
						setAIAction(defaultAction());
					} else if( rand()%5000 == 0 && distance(getX(), getY(), getTarget().getX(), getTarget().getY()) > 2250 ) {
						setTarget(null);
						setAIAction(defaultAction());
					} else {
						if( getItemCount(ITEM_TYPE.WEAPON, WILDCARD, true, true, true) > 0 ) {
							double goodX, goodY;
							double getIX = getIntercept(7.0+getXSpeed(), getX(), getTarget().getXSpeed(), getTarget.getX()); 
							double getIY = getIntercept(7.0+getYSpeed(), getY(), getTarget().getYSpeed(), getTarget.getY()); 
							float PAA = pointAtAngle(getX(), getY(), getIX, getIY);
							if( getTarget().getType() == OBJECT_TYPE.SHIP ) {
								float rotOff = -90.0;
								if( rand()%400==0 ) attackMode = random_range(0,2);
								if( angleDistance( getTarget().getRotation(), PAA - 180 ) < 30 ) rotOff = 0;
								else rotOff += (90 * attackMode);
								goodX = getTarget().getX() - (moveToValueX(getTarget().getRotation()+rotOff) * 375);
								goodY = getTarget().getY() - (moveToValueY(getTarget().getRotation()+rotOff) * 375);
							} else {
								goodX = getTarget().getX();
								goodY = getTarget().getY();
							}
							double dist = distance(getX(), getY(), goodX, goodY);
							if( dist > 325 ) flyTowards(goodX, goodY);
							else {
								turnTowards(getTarget().getX(), getTarget().getY());
								FU_ITEM sEngine = getItem(ITEM_TYPE.UTILITY, ITEM_CLASS.STRAFER, 0, true, true);
								if( sEngine !is null ) {
									if( rand()%240==0 ) straferMode = random_range(0,2);
									if( straferMode != 0 ) {
										float energy, max;
										modifyUtilityCounter(ITEM_CLASS.GENERATOR,0,&energy,&max);
										if( (energy / max) > 0.5 ) {
											if( straferMode == 1 ) thrust(ITEM_CLASS.STRAFER, 1);
											else thrust(ITEM_CLASS.STRAFER, -1);
										}
									}
								}
							}
							if( distance(getX(), getY(), getTarget.getX(), getTarget.getY()) < MAX_TARGETABLE ) {
								if(angleDistance(PAA,getRotation()) < 30 ||
								   getSprite().collide(getTarget().getSprite()) == true ) fire(WILDCARD,false,true);
								else fire(WILDCARD,false,true,true);
							}
						} else if( getTarget().getType() == OBJECT_TYPE.SHIP ) setAIAction(AI_ACTION.RUNNING);
						else {
							setTarget(null);
							setAIAction(defaultAction());
						}
					}
					break;
				case AI_ACTION.FOLLOWING:		// following leader
					if( objectExists(getLeader()) == false || getLeader() is this ||
						rand()%8000 == 0 && getLeader() !is PlayerShip ||
						getLeader().getAIAction() == AI_ACTION.FOLLOWING ) {
						// decide I don't want to follow anymore
						setLeader(null);
						setTarget(null);
						setAIAction(defaultAction());
					} else {
						double leaderDist = distance(getX(), getY(), getLeader().getX(), getLeader().getY());
						if( getLeader().getAlpha() < 200 ) cloak();
						if( leaderDist < 450 ) {
							setTarget(getLeader());
							fire(WILDCARD,true);
						}
						if( getLeader().getAIAction() == AI_ACTION.DOCKING ) {
							setTarget(getLeader().getTarget());
							setAIAction(AI_ACTION.DOCKING);
						} else if( leaderDist > 200 ) flyTowards( getLeader().getX(), getLeader().getY() );
						else turnTowards( getLeader().getX(), getLeader().getY() );
						if( getLeader().getAIAction() == AI_ACTION.ATTACKING ||
							getLeader().getAIAction() == AI_ACTION.RUNNING ) {
								if( getLeader().getTarget() !is null && getLeader().getTarget() !is this ) {
									if( abs( getLeader().getTarget().getFaction() - getFaction() ) > 1 ) {
										setTarget(getLeader().getTarget());
										setAIAction(AI_ACTION.ATTACKING);
									} else if( getLeader().getTarget().getType() != OBJECT_TYPE.ROCK &&
											   getLeader().getTarget().getType() != OBJECT_TYPE.ITEM ) {
										setTarget(getLeader());
										setLeader(null);
										setAIAction(AI_ACTION.ATTACKING);
									}
								}
						}
					}
					break;
				case AI_ACTION.PICKUP:		// picking up items
					if( getTarget() is null ||
						getTarget().getType() != OBJECT_TYPE.ITEM ||
						getTarget().getItem() is null ) {
							setTarget(null);
							setAIAction(defaultAction());
					} else if( playerActionTimer < 40 ) {
						// spend a moment slowing down and turning
						// this will make sure we have a clear shot to item
						playerActionTimer++;
						turnTowards(target.getX(), target.getY());
						airBrakes();
					} else {
						double dist = flyTowards(getTarget().getX(), getTarget().getY());
						int pickupAttempt = pickupTarget();
						if( pickupAttempt == -2 || pickupAttempt == 0 ) {
							setTarget(null);
							setAIAction(defaultAction());
						}
					}
					break;
				}
				// find and decide what to do with targets
				if( forceWander == false && forcePatrol == false &&
					(AIAction == AI_ACTION.WANDERING || AIAction == AI_ACTION.PATROLING) ) {
					if( target is null && rand()%240==0 ) searchForTargets();
					else if( target !is null ) {
						switch( target.getType() ) {
						case OBJECT_TYPE.PLANET:
						case OBJECT_TYPE.STATION:
							if( abs( getFaction() - getTarget().getFaction() ) > 1 ) {
								setAIAction(defaultAction());
								setTarget(null);
							}
						case OBJECT_TYPE.PORTHOLE:
						case OBJECT_TYPE.TRANSPATH:
						case OBJECT_TYPE.TELEGATE:
							if( rand()%2 == 0 ) setAIAction(AI_ACTION.DOCKING);
							else {
								setAIAction(defaultAction());
								setTarget(null);
							}
							break;
						case OBJECT_TYPE.SHIP:
							FU_SHIP tShip = cast(FU_SHIP)getTarget();
							int factionDiff = abs(getTarget().getFaction() - getFaction());
							switch( AIClass ) {
							case AI_CLASS.PIRATE:
								if( tShip.getAIAction() == AI_ACTION.ATTACKING
									&& factionDiff >= 2 || factionDiff >= 3 ) setAIAction(AI_ACTION.ATTACKING);
								else if( factionDiff <= 1 ) {
									setLeader(tShip);
									setTarget(null);
									setAIAction(AI_ACTION.FOLLOWING);
								} else {
									setTarget(null);
									setAIAction(defaultAction());
								}
								break;
							case AI_CLASS.POLICE:
								if( (tShip.getAIAction() == AI_ACTION.ATTACKING || tShip.hasContraband())
									&& factionDiff >= 2 ) {
										setAIAction(AI_ACTION.ATTACKING);
								} else if( factionDiff <= 1 && tShip.getAIClass() != AI_CLASS.KILLER &&
										   tShip.getAIClass() != AI_CLASS.PIRATE ) {
									setLeader(tShip);
									setTarget(null);
									setAIAction(AI_ACTION.FOLLOWING);
								} else {
									setTarget(null);
									setAIAction(defaultAction());
								}
								break;
							case AI_CLASS.NONE:
							case AI_CLASS.TRAVELER:
							case AI_CLASS.TOURIST:
							case AI_CLASS.EXPLORER:
							case AI_CLASS.MINER:
								setTarget(null);
								setAIAction(defaultAction());
								break;
							case AI_CLASS.KILLER:
								if( factionDiff >= 2 ) setAIAction(AI_ACTION.ATTACKING);
								else {
									setLeader(tShip);
									setTarget(null);
									setAIAction(AI_ACTION.FOLLOWING);								
								}
								break;
							}
							break;
						case OBJECT_TYPE.ROCK:
							setAIAction(AI_ACTION.ATTACKING);
							break;
						case OBJECT_TYPE.ITEM:
							if( PlayerShip.getCurrentJob() !is null &&
								PlayerShip.getCurrentJob().isTarget(target) == true ) {
								setTarget(null);
								setAIAction(defaultAction());
							} else setAIAction(AI_ACTION.PICKUP);
							break;
						case OBJECT_TYPE.NULL:
						case OBJECT_TYPE.PROJECTILE:
							setTarget(null);
							setAIAction(defaultAction());
							break;
						}
					}
				}
			}
		} else {
			updateAction();
			if( playerUsingCloak == true ) {
				if( cloak() == false ) {
					space.hud.addMessage("Cloaking device deactivated.",255,20,20);
					playerUsingCloak = false;
				}
			}
		}
		// handle space storm effects
		effectShip(this);
	}
	// return this ship's default action
	int defaultAction() {
		debug scope(failure) { writefln("defaultAction: ship.d"); }
		if( forcePatrol == true ) return AI_ACTION.PATROLING;
		if( forceWander == true ) return AI_ACTION.WANDERING;
		if( getLeader() !is null ) return AI_ACTION.FOLLOWING;
		switch( AIClass ) {
		case AI_CLASS.TRAVELER:
		case AI_CLASS.POLICE:
			return AI_ACTION.PATROLING;
		case AI_CLASS.NONE:
		case AI_CLASS.TOURIST:
		case AI_CLASS.EXPLORER:
		case AI_CLASS.PIRATE:
		case AI_CLASS.KILLER:
		case AI_CLASS.MINER:
			return AI_ACTION.WANDERING;
		}
	}
	// search for targets
	void searchForTargets() {
		debug scope(failure) { writefln("searchForTargets: ship.d"); }
		//SpaceObjects.backup();
		//SpaceObjects.reset();

		if( sapModifier(MODIFIER_TYPE.DISABLE_RADAR, getLevel()*20) == true ) return;
		
		FU_OBJECT[] nearObjects;

		foreach(inout FU_OBJECT data; GameObjects)
		{
			if( AIClass == AI_CLASS.MINER ) 
			{
					if( data.getType() == OBJECT_TYPE.SHIP ) continue;
					if( data.getType() == OBJECT_TYPE.TELEGATE ) continue;				
			} 
			else if( data.getType() == OBJECT_TYPE.ROCK ) continue;
			if( data.getType() == OBJECT_TYPE.PROJECTILE ) continue;
			if( AIClass == AI_CLASS.POLICE ) 
			{
					if( data.getType() == OBJECT_TYPE.TELEGATE ||
						data.getType() == OBJECT_TYPE.TRANSPATH ||
						data.getType() == OBJECT_TYPE.STATION ||
						data.getType() == OBJECT_TYPE.PLANET ) continue;
			}
			if( AIClass == AI_CLASS.TRAVELER && data.getType() != OBJECT_TYPE.STATION &&
				data.getType() != OBJECT_TYPE.PLANET && data.getType() != OBJECT_TYPE.TRANSPATH &&
				data.getType() != OBJECT_TYPE.TELEGATE ) 
			{
					continue;
			}
			if( data.getType() == OBJECT_TYPE.PORTHOLE ) continue;
			if( cast(FU_SHIP)data is this ) continue;
			if( data.getType() == OBJECT_TYPE.SHIP &&
				canSee(this, cast(FU_SHIP)data) == false ) continue;
			if( distance(data.getX(), data.getY(), getX(), getY()) < MAX_TARGETABLE ) 
			{
				nearObjects ~= data;
			}
		} 

		if( nearObjects.length > 0 ) {
			setTarget(nearObjects[rand()%nearObjects.length]);
			setAIAction(defaultAction());
		}
	}
	// general AI turn-towards
	// returns angle distance from target
	float turnTowards(double parmX, double parmY) {
		debug scope(failure) { writefln("turnTowards: ship.d"); }
		float angDest = pointAtAngle(getX(), getY(), parmX, parmY);
		float angDist = angleDistance(angDest, getRotation());
		if( angDist < 1.5 ) {
			setRotation(angDest);
			return angDist;
		}
		float d_angle = angDest - getRotation();
		if ( (d_angle < 180 && d_angle >= 0) || (d_angle < -180.) ) thrust(ITEM_CLASS.TURNER, 1);
		else thrust(ITEM_CLASS.TURNER, -1);
		return angDist;
	}
	// general AI fly-towards
	// returns distance from target
	double flyTowards(double parmX, double parmY) {
		debug scope(failure) { writefln("flyTowards: ship.d"); }
		double targetDistance = distance(parmX, parmY, getX(), getY());
		float angDistance = turnTowards(parmX, parmY);
		if( angDistance > 135 ) {
			// use reverse thrust
			thrust(ITEM_CLASS.ENGINE, -1);
			return targetDistance;
		}
		if( angDistance > 45 ) return targetDistance;
		if( getItemCount(ITEM_TYPE.UTILITY, ITEM_CLASS.CRUISER, true, true) > 0 ) {
			if( targetDistance > 2250 || targetDistance > 375 && usingCruiser == true ) {
				if( thrust(ITEM_CLASS.CRUISER, 1) == false ) {
					thrust(ITEM_CLASS.ENGINE, 1);
				}
				usingCruiser = true;
				return targetDistance;
			}
		}
		usingCruiser = false;
		if( getItemCount(ITEM_TYPE.UTILITY, ITEM_CLASS.AFTERBURNER, true, true) > 0 ) {
			if( targetDistance > 250 ) {
				// only use afterburner if energy is high enough
				float energy, max;
				modifyUtilityCounter(ITEM_CLASS.GENERATOR,0,&energy,&max);
				if( (energy / max) > 0.33 ) {
					thrust(ITEM_CLASS.AFTERBURNER, 1);
					return targetDistance;
				}
			}
		}
		thrust(ITEM_CLASS.ENGINE, 1);
		return targetDistance;
	}
	// returns false if no cloaking item exists
	bool cloak() {
		debug scope(failure) { writefln("cloak: ship.d"); }
		FU_ITEM cloakItem = getItem(ITEM_TYPE.UTILITY, ITEM_CLASS.CLOAKING, 0, true, true);
		if( cloakItem is null ) return false;
		if( cloakItem.activateRequest(this) == false ) return true;
		if( sapModifier(MODIFIER_TYPE.DISABLE_CLOAK, cloakItem.getLevel()*0.15) == true ) return true;
		short myAlpha = getAlpha();
		short alphaLimit;
		if( this is PlayerShip || canSee(PlayerShip, this) == true ) alphaLimit = 100;
		else alphaLimit = 7;
		if( myAlpha > alphaLimit ) {
			setAlpha(myAlpha-3);
			if( getAlpha() < alphaLimit ) setAlpha(alphaLimit);
		}
		return true;
	}
	// fire weapons
	bool fire(int weaponType = -1, bool goodTarget = false, bool fireGoodAndBad = false, bool fireJustTurret = false ) {
		debug scope(failure) { writefln("fire: ship.d"); }
		if( getFriction() == false ) return false;
		items.reset();
		while(!items.last()) {
			if( items.data.getType() == ITEM_TYPE.WEAPON &&
				(items.data.getClass() == weaponType || weaponType == -1 ) &&
				(fireJustTurret == false || items.data.getVars().aimMode == AIM_MODE.TURRET) &&
				(goodTarget == items.data.isGoodModifier() || fireGoodAndBad == true) &&
				items.data.getEnable() == true ) {
				if( items.data.activateRequest(this) ) {
					FU_PROJECTILE newShot = new FU_PROJECTILE(this, items.data);
					GameObjects.add(cast(FU_OBJECT)newShot);
					if( this is PlayerShip && items.data.isGoodModifier() == false ) updateAction(AI_ACTION.ATTACKING);
					playSound(SOUND_CLASS.SHOOT,getX(),getY(),items.data.getVars().projSndValue);
				}
			}
		}
		return true;
	}
	void disableTransTravel() {
		if( getFriction() == false ) {
			x_speed *= 0.65;
			y_speed *= 0.65;
			setFriction(true);
			if( this is PlayerShip ) updateAction(AI_ACTION.WANDERING);
		}		
	}
	bool thrust(int itemClassUsed, float throttle) {			
		debug scope(failure) { writefln("thrust: ship.d"); }
		disableTransTravel();
		int angle_offset = 0;
		bool success = false;
		if( itemClassUsed == ITEM_CLASS.STRAFER ) angle_offset = 90;
		items.reset();
		while(!items.last()) {
			if( items.data.getType() == ITEM_TYPE.UTILITY && items.data.getClass() == itemClassUsed &&
				items.data.getEnable() == true ) {
				if( items.data.activateRequest(this) ) {
					float amount = items.data.getVars().acceleration * throttle * items.data.getVars().quality;
					success = true;
					if( itemClassUsed != ITEM_CLASS.TURNER ) {
						if( sapModifier(MODIFIER_TYPE.DISABLE_ENGINES, amount*0.175) == true ) return success;
						if( soundIndex == -1 ) soundIndex = -2;
						thrustUsed = true;
						if( sapModifier(MODIFIER_TYPE.INCREASE_SPEED, amount*0.075) == true ) amount *= 1.33;
						super.max_speed = items.data.getVars().maximum;
						super.accelerate(amount, angle_offset, false, this);
						if( getAlpha() > 200 && rand()%2==0) {
							FU_PARTICLE newParticle = FU_PARTICLE.allocate();
							switch(itemClassUsed) {
							case ITEM_CLASS.ENGINE:
							case ITEM_CLASS.STRAFER:
								newParticle.construct(this,255,150,100);
								break;
							case ITEM_CLASS.AFTERBURNER:
								newParticle.construct(this,100,150,255);
								break;
							case ITEM_CLASS.CRUISER:
								newParticle.construct(this,255,255,255);
								break;
							}
							newParticle.setSpeed(moveToValueX(getRotation()+angle_offset-180)*2,moveToValueY(getRotation()+angle_offset-180)*2);
						}
					} else {
						if( sapModifier(MODIFIER_TYPE.DISABLE_ENGINES, amount*0.0085) == true ) return success;
						if( sapModifier(MODIFIER_TYPE.INCREASE_SPEED, amount*0.005) == true ) amount *= 1.33;
						super.turn(amount);
					}
				}
			}
		}
		return success;
	}
	int getStormProtection() { return stormProtection; }
	void maxUtils() {
		float cur, max;
		modifyUtilityCounter(ITEM_CLASS.SHIELDS, 825000, &cur, &max);
		modifyUtilityCounter(ITEM_CLASS.GENERATOR, 825000, &cur, &max);
	}
	// can use this for shields, generators etc.
	// returns whatever wasn't absorbed by modifyAmount
	float modifyUtilityCounter(int item_class, float modifyAmount, float *remainingAmount, float *maxAmount) {
		*maxAmount = 0;
		*remainingAmount = 0;
		items.backup();
		items.reset();
		while(!items.last()) {
			if( items.data.getType() == ITEM_TYPE.UTILITY &&
				items.data.getClass() == item_class &&
				items.data.isInstalled() ) {
					items.data.getVars().counter += modifyAmount;
					modifyAmount = 0;
					if( items.data.getVars().counter < 0 ) {
						modifyAmount = items.data.getVars().counter;
						items.data.getVars().counter = 0;
					} else if( items.data.getVars().counter > items.data.getVars().maximum ) {
						items.data.getVars().counter = items.data.getVars().maximum;
					}
					*maxAmount += items.data.getVars().maximum;
					*remainingAmount += items.data.getVars().counter;
			}
		}
		items.restore();
		if( *maxAmount < 1 ) *maxAmount = 1;
		return modifyAmount;
	}
	bool hasContraband() {
		items.reset();
		while(!items.last) {
			if( items.data.getType() == ITEM_TYPE.CARGO &&
				items.data.getCargoType() == ITEM_CLASS.CONTRABAND )
					return true;
		}
		return false;
	}
	int getItemCount(int parmItemType, int parmItemClass = -1, bool installedOnly = false, bool enabledOnly = false, bool ignoreGoodMods = false) {
		int count = 0;
		items.reset();
		while(!items.last) {
			if( parmItemType == items.data.getType() ) {
				if( installedOnly == false || items.data.isInstalled() == true ) {
					if( enabledOnly == false || items.data.getEnable() == true ) {
						if( parmItemClass == -1 || parmItemClass == items.data.getClass() )
							if( ignoreGoodMods == false || items.data.isGoodModifier() == false )count++;
					}
				}
			}
		}
		return count;
	}
	FU_ITEM getItem(int parmItemType, int parmItemClass = -1, int index = 0, bool installedOnly = false, bool enabledOnly = false) {
		items.reset();
		while(!items.last) {
			if( parmItemType == items.data.getType() ) {
				if( installedOnly == false || items.data.isInstalled() == true ) {
					if( enabledOnly == false || items.data.getEnable() == true ) {
						if( parmItemClass == -1 || parmItemClass == items.data.getClass() ) {
							index--;
							if( index < 0 ) return items.data;
						}
					}
				}
			}
		}
		return null;
	}
	void jettison(FU_ITEM byeItem) {
		if( byeItem is null ) return;
		MainGenerator.spawnItem(getX(), getY(), byeItem);
		removeItem(byeItem);
	}
	bool removeItem(FU_ITEM byeItem) {
		if( byeItem is null ) return false;
		items.reset();
		while(!items.last) {
			if( byeItem is items.data ) {
				items.data.forceUninstall();
				items.remove();
				updateValue();
				updateWeight();
				return true;
			}
		}
		return false;
	}
	int installItem(FU_ITEM itemToInstall) {
		if( itemToInstall is null ) return -1;
		if( itemToInstall.getType() != ITEM_TYPE.WEAPON && itemToInstall.getType() != ITEM_TYPE.UTILITY ) return -1;
		if( ownItem(itemToInstall) == false ) return -1;
		if( getLevel() < itemToInstall.getLevel() - 2 ) return -2;
		if( itemToInstall.getType() == ITEM_TYPE.WEAPON ) {
			int weap_count = getItemCount(ITEM_TYPE.WEAPON,WILDCARD,true) + 1;
			if( weap_count > max_weapons ) return -3;
		} else if( itemToInstall.getType() == ITEM_TYPE.UTILITY ) {
			int util_count = getItemCount(ITEM_TYPE.UTILITY,WILDCARD,true) + 1;
			if( util_count > max_utility ) return -4;
		}
		itemToInstall.forceInstall();
		itemToInstall.setEnable(true);
		return 0;
	}
	bool uninstallItem(FU_ITEM itemToUninstall) {
		if( itemToUninstall is null ) return false;
		if( itemToUninstall.getType() != ITEM_TYPE.WEAPON && itemToUninstall.getType() != ITEM_TYPE.UTILITY ) return false;
		if( itemToUninstall.isInstalled() == false ) return false;
		if( ownItem(itemToUninstall) == false ) return false;
		itemToUninstall.forceUninstall();
		itemToUninstall.setEnable(false);
		return true;
	}
	bool allowWeight(FU_ITEM new_item, int itemCount = 1) {
		updateWeight();
		if( this.weight + (new_item.getWeight(false) * itemCount) > max_weight ) return false;
		return true;
	}
	bool addItem(FU_ITEM new_item, int itemCount = 1) {
		if( new_item.getType() != ITEM_TYPE.CARGO ) {
			if( allowWeight(new_item, 1) == false ) return false;
			bool alreadyAdded = false;
			items.reset();
			while(!items.last()) {
				if( items.data is new_item ) alreadyAdded = true;
			}
			if( alreadyAdded == false ) {
				if( items.length() >= MAX_ITEMS ) return false;
				items.add(new_item);
			}
		} else {
			items.reset();
			while(!items.last()) {
				if( items.data.getType() == ITEM_TYPE.CARGO &&
					items.data.getClass() == new_item.getClass() &&
					items.data.getCargoType() == new_item.getCargoType() &&
					items.data.getLevel() == new_item.getLevel()) {
						items.data.getVars().counter += itemCount;
						new_item.getVars().counter -= itemCount;
						updateWeight();
						updateValue();
						return true;
				}
			}
			if( PlayerShip.allowWeight(new_item, itemCount) == true ) {
				if( items.length() >= MAX_ITEMS ) return false;
				FU_ITEM placeItem = ItemCopy(new_item);
				placeItem.getVars().counter = itemCount;
				new_item.getVars().counter -= itemCount;
				items.add(placeItem);
			} else return false;
		}
		updateWeight();
		updateValue();
		return true;
	}
	// populates this ship_type with items
	void populate() {
		debug scope(failure) { writefln("populate: ship.d"); }
		int mainThrusters = 1, i=0;
		// if( ship_type == SHIP_TYPE.ABANDONED ) mainThrusters--;
		if( ship_type == SHIP_TYPE.CRUISER || ship_type == SHIP_TYPE.CARRIER ) mainThrusters++;
		for(;i<mainThrusters;i++) {
			FU_ITEM engine = new FU_ITEM(ITEM_TYPE.UTILITY, ITEM_CLASS.ENGINE, getLevel(),null,getFaction());
			FU_ITEM turner = new FU_ITEM(ITEM_TYPE.UTILITY, ITEM_CLASS.TURNER, getLevel(),null,getFaction());
			addItem(engine);
			addItem(turner);
			if( installItem(engine) != 0 ) removeItem(engine);
			if( installItem(turner) != 0 ) removeItem(turner);
			mainThrusters--;
		}
		int genCount = 1;
		if( ship_type == SHIP_TYPE.CRUISER ) genCount++;
		for(i=0;i<genCount;i++) {
			// everybody needs a generator
			FU_ITEM generator = new FU_ITEM(ITEM_TYPE.UTILITY, ITEM_CLASS.GENERATOR, getLevel(),null,getFaction());
			addItem(generator);
			if( installItem(generator) != 0 ) removeItem(generator);
		}
		// everybody needs a sensor
		FU_ITEM sensor = new FU_ITEM(ITEM_TYPE.UTILITY, ITEM_CLASS.SENSOR, getLevel(),null,getFaction());
		addItem(sensor);
		if( installItem(sensor) != 0 ) removeItem(sensor);
		int shieldCount = 1;
		if( ship_type == SHIP_TYPE.CARRIER || ship_type == SHIP_TYPE.CRUISER ) shieldCount++;
		for(i=0;i<shieldCount;i++) {
			// everyobdy needs shields
			FU_ITEM shields = new FU_ITEM(ITEM_TYPE.UTILITY, ITEM_CLASS.SHIELDS, getLevel(),null,getFaction());
			addItem(shields);
			if( installItem(shields) != 0 ) removeItem(shields);
		}
		// should we have an afterburner?
		if( ship_type == SHIP_TYPE.STANDARD && rand()%3==0 ||
			ship_type == SHIP_TYPE.POLICE ||
			ship_type == SHIP_TYPE.PIRATE ||
			ship_type == SHIP_TYPE.FIGHTER ) {
				FU_ITEM afterburner = new FU_ITEM(ITEM_TYPE.UTILITY, ITEM_CLASS.AFTERBURNER, getLevel(),null,getFaction());
				addItem(afterburner);
				if( installItem(afterburner) != 0 ) removeItem(afterburner);
		}
		// should we have a cruiser?
		if( ship_type == SHIP_TYPE.CRUISER ||
			ship_type == SHIP_TYPE.CARRIER ||
			ship_type == SHIP_TYPE.STANDARD && rand()%3 == 0 ||
			ship_type == SHIP_TYPE.PIRATE ||
			ship_type == SHIP_TYPE.SHUTTLE && rand()%2 == 0 ||
			ship_type == SHIP_TYPE.POLICE ) {
				FU_ITEM cruiser = new FU_ITEM(ITEM_TYPE.UTILITY, ITEM_CLASS.CRUISER, getLevel(),null,getFaction());
				addItem(cruiser);
				if( installItem(cruiser) != 0 ) removeItem(cruiser);
		}
		// should we have strafer engines?
		if( ship_type == SHIP_TYPE.FIGHTER ) {
			FU_ITEM strafer = new FU_ITEM(ITEM_TYPE.UTILITY, ITEM_CLASS.STRAFER, getLevel(),null,getFaction());
			addItem(strafer);
			if( installItem(strafer) != 0 ) removeItem(strafer);
		}
		// should we have a cloaking device?
		if( (ship_type == SHIP_TYPE.FIGHTER || ship_type == SHIP_TYPE.CRUISER || ship_type == SHIP_TYPE.PIRATE) && rand()%2 == 0 ||
			ship_type == SHIP_TYPE.SHUTTLE ) {
				FU_ITEM cloaker = new FU_ITEM(ITEM_TYPE.UTILITY, ITEM_CLASS.CLOAKING, getLevel(),null,getFaction());
				addItem(cloaker);
				if( installItem(cloaker) != 0 ) removeItem(cloaker);
		}
		// outfit with weapons?
		if( max_weapons > 0 ) {
			int weap_count = rand()%(max_weapons);
			for(i=0;i<weap_count;i++) {
				FU_ITEM weapon = new FU_ITEM(ITEM_TYPE.WEAPON, (rand()%MAX_WEAPONS) + 25, getLevel(),null,getFaction());
				addItem(weapon);
				if( installItem(weapon) != 0 ) removeItem(weapon);
			}
		}
		// should we have some cargo?
		if( ship_type == SHIP_TYPE.CARRIER ) {
			FU_ITEM cargo = new FU_ITEM(ITEM_TYPE.CARGO, (rand()%MAX_CARGO) + 50, getLevel(),null,getFaction());
			cargo.getVars().counter = random_range(1,15);
			addItem(cargo);
		}
		// should we have some contraband?
		if( ship_type == SHIP_TYPE.PIRATE ||
			ship_type == SHIP_TYPE.POLICE ) {
			FU_ITEM contraband = new FU_ITEM(ITEM_TYPE.CARGO, ITEM_CLASS.CONTRABAND, getLevel(),null,getFaction());
			contraband.getVars().counter = random_range(1,3);
			addItem(contraband);
		}
	}
}
