﻿module docked.docked;

private import init.init;
private import classes.dockinfo;
private import classes.ship;
private import classes.fu_object;
private import classes.gui;
private import classes.job;
private import classes.item;

static const int DRAW_DETAILED_X = 10;
static const int DRAW_DETAILED_Y = 175;

void dockHandler() {
	// make some shortcut variables
	FU_OBJECT host = PlayerShip.getDockInfo().getHost();
	FU_SHIP client = PlayerShip.getDockInfo().getClient();
	
	// handle drawing
	// draw pretty background, get pre-name
	char[] hostType;
	switch( host.getType() ) {
	case OBJECT_TYPE.PLANET:
		hostType = "Planet ";
		drawImage(400, 300, 800, 600, 255, 255, 255, 140, 0, 0, 0, GameTextures[TEXTURES.DOCKEDPLANET][PlayerShip.getDockInfo().getBGID()].getID);
		break;
	case OBJECT_TYPE.STATION:
		hostType = "Station ";
		drawImage(400, 300, 800, 600, 255, 255, 255, 140, 0, 0, 0, GameTextures[TEXTURES.DOCKEDSTATION][PlayerShip.getDockInfo().getBGID()].getID);
		break;
	}
	
	// always draw upper left docking icon
	drawRect(0,1,150,150,0,0,0,150,true);
	drawImage(75, 75, 149, 149, host.getSprite().getR(),host.getSprite().getG(),host.getSprite().getB(),255, 0, 0, 0, host.getSprite().getCurrFrame().getID);
	drawImage(35, 35, 50, 50, PlayerShip.getSprite().getR(),PlayerShip.getSprite().getG(),PlayerShip.getSprite().getB(),255, 0, 0, 0, PlayerShip.getSprite().getCurrFrame().getID);
	drawRect(0,1,150,150,255,255,255,255,false);
	// draw station name and faction
	largeFont.draw(hostType ~ host.getName(), 155, 4, 255, 255, 255, 255);
	bigFont.draw("Faction: " ~ FactionNames[host.getFaction()], 155, 35, 255, 255, 255, 255);

	// draw container box
	drawRect(2, 155, 800-4, 600 - 157, 255, 255, 255, 255, false);
	drawRect(3, 156, 800-6, 600 - 159, 0, 0, 0, 75, true);

	// draw all the main buttons
	// (highlight the current mode in green)
	TOP_repairButton.draw(PlayerShip.getDockInfo().currentDockMode == DOCK_MODE.REPAIR);
	TOP_shipsButton.draw(PlayerShip.getDockInfo().currentDockMode == DOCK_MODE.SHIPS);
	TOP_equipButton.draw(PlayerShip.getDockInfo().currentDockMode == DOCK_MODE.EQUIPMENT);
	TOP_waresButton.draw(PlayerShip.getDockInfo().currentDockMode == DOCK_MODE.WARES);
	TOP_jobButton.draw(PlayerShip.getDockInfo().currentDockMode == DOCK_MODE.JOBS);
	TOP_myItemsButton.draw(PlayerShip.getDockInfo().currentDockMode == DOCK_MODE.MYITEMS);
	TOP_exitButton.draw();

	// draw the status bar
	bigFont.draw("Credits: " ~ std.string.toString(playerCredits) ~ "\n" ~ PlayerShip.getDockInfo().getStatus() ~ "\nInformation: " ~ PlayerShip.getDockInfo().message, 160, 100, 255, 255, 255, 255);

	// basic exit/undocking option
	if( TOP_exitButton.isClicked() || arc.io.input.keyHit(SDLK_ESCAPE) ) {
		gameState = GAME_STATE.SPACE;
		PlayerShip.maxUtils();
		PlayerShip.setPosition(host.getX()+random_range(-150,150),host.getY()+random_range(-150,150));
		PlayerShip.setRotation(random_range(0,360));
		PlayerShip.setXSpeed(0.0);
		PlayerShip.setYSpeed(0.0);
		saveCurrentGame();
		return;
	}

	// select mode with buttons, clear any selection
	if( TOP_repairButton.isClicked() ) { PlayerShip.getDockInfo().currentDockMode = DOCK_MODE.REPAIR; PlayerShip.getDockInfo().resetControl(); }
	if( TOP_shipsButton.isClicked() ) { PlayerShip.getDockInfo().currentDockMode = DOCK_MODE.SHIPS; PlayerShip.getDockInfo().resetControl(); }
	if( TOP_equipButton.isClicked() ) { PlayerShip.getDockInfo().currentDockMode = DOCK_MODE.EQUIPMENT; PlayerShip.getDockInfo().resetControl(); }
	if( TOP_waresButton.isClicked() ) { PlayerShip.getDockInfo().currentDockMode = DOCK_MODE.WARES; PlayerShip.getDockInfo().resetControl(); }
	if( TOP_jobButton.isClicked() ) { PlayerShip.getDockInfo().currentDockMode = DOCK_MODE.JOBS; PlayerShip.getDockInfo().resetControl(); }
	if( TOP_myItemsButton.isClicked() ) { PlayerShip.getDockInfo().currentDockMode = DOCK_MODE.MYITEMS; PlayerShip.getDockInfo().resetControl(); }

	// right clicking resets stuff
	if( arc.io.input.mouseHit(RIGHT) == true ) PlayerShip.getDockInfo().resetControl();

	// display this mode's buttons
	switch( PlayerShip.getDockInfo().currentDockMode ) {
	case DOCK_MODE.REPAIR:
		repairButton.draw();
		repairAllButton.draw();
		repairHullButton.draw();
		long totalRepairCost = 0;
		drawItems(PlayerShip.items,&totalRepairCost);
		long hullPrice = cast(long)(PlayerShip.getMaxArmor() * (1 - PlayerShip.percentArmorRemaining()));
		if( hullPrice < 0 ) hullPrice = 0;
		PlayerShip.getDockInfo().setStatus("Hull repair cost: " ~ std.string.toString(hullPrice) ~ "\nRepair All Cost: " ~ std.string.toString(totalRepairCost));
		// handle clicking the repair button
		if( repairButton.isClicked() == true ) {
			if( PlayerShip.getDockInfo().itemSelection is null ) PlayerShip.getDockInfo().message = "Please select an item to repair.";
			else {
				long repairCost = PlayerShip.getDockInfo().itemSelection.getValue();
				if( PlayerShip.getDockInfo().itemSelection.getVars().quality == 1.0 ) PlayerShip.getDockInfo().message = "Item needs no repair.";
				else if( repairCost > playerCredits ) PlayerShip.getDockInfo().message = "Not enough credits to repair.";
				else {
					PlayerShip.getDockInfo().itemSelection.getVars().quality = 1.0;
					playerCredits -= repairCost;
					PlayerShip.getDockInfo().message = "Item repaired.";
				}
			}
		}
		if( repairHullButton.isClicked() == true ) {
			if( hullPrice == 0 ) PlayerShip.getDockInfo().message = "No hull repair needed.";
			else if( playerCredits >= hullPrice ) {
				playerCredits -= hullPrice;
				PlayerShip.repairHull();
				PlayerShip.getDockInfo().message = "Hull repaired.";
			} else PlayerShip.getDockInfo().message = "Not enough credits to repair hull.";
		}
		if( repairAllButton.isClicked() == true ) {
			if( playerCredits >= totalRepairCost ) {
				if( totalRepairCost == 0 ) PlayerShip.getDockInfo().message = "No items need repair.";
				else {
					playerCredits -= totalRepairCost;
					PlayerShip.items.reset();
					while(!PlayerShip.items.last()) {
						if( PlayerShip.items.data.getType() != ITEM_TYPE.CARGO )
							PlayerShip.items.data.getVars().quality = 1.0;
					}
					PlayerShip.getDockInfo().message = "All items have been repaired.";
				}
			} else PlayerShip.getDockInfo().message = "Not enough credits to repair all items.";
		}
		break;
	case DOCK_MODE.SHIPS:
		buy1Button.draw();
		detailToggleButton.draw();
		myShipDetailButton.draw();
		PlayerShip.getDockInfo().setStatus("Your ship's trade-in value (not including items) is included.\nYour items must fit on new ship for purchase to complete.");
		if( buy1Button.isClicked() == true ) {
			if( PlayerShip.getDockInfo().shipSelection is null || PlayerShip.getDockInfo().shipSelection is PlayerShip ) {
				PlayerShip.getDockInfo().message = "Please make a ship selection.";
				PlayerShip.getDockInfo().shipSelection = null;
				PlayerShip.getDockInfo().detailedView = false;
			} else {
				FU_SHIP buyShip = PlayerShip.getDockInfo().shipSelection;
				float weightCheck;
				long tradePrice = buyShip.purchaseCost(&weightCheck);
				if( weightCheck < 0 ) PlayerShip.getDockInfo().message = "You cannot fit your items on this ship.";
				else if( playerCredits < tradePrice ) PlayerShip.getDockInfo().message = "You cannot afford this ship.";
				else {
					playerCredits -= tradePrice;
					PlayerShip.items.reset();
					while(!PlayerShip.items.last()) {
						PlayerShip.items.data.forceUninstall();
						buyShip.items.add(PlayerShip.items.data);
						PlayerShip.items.remove();
					}
					PlayerShip.getDockInfo().sellShips.reset();
					while(!PlayerShip.getDockInfo().sellShips.last()) {
						if( PlayerShip.getDockInfo().sellShips.data is buyShip ) PlayerShip.getDockInfo().sellShips.remove();
					}
					PlayerShip.getDockInfo().message = "Ship purchase completed.";

					foreach(inout FU_OBJECT data; GameObjects)
					{
						if( cast(FU_SHIP)data is PlayerShip ) {
							GameObjects.remove();
						}
					}

					buyShip.setPosition(PlayerShip.getX(),PlayerShip.getY());
					buyShip.setDockInfo(PlayerShip.getDockInfo());
					if( PlayerShip.getCurrentJob() !is null ) {
						buyShip.setJob(PlayerShip.getCurrentJob());
						PlayerShip.getCurrentJob().setWaypointsFor(buyShip,false);
					}
					GameObjects.add(buyShip);
					buyShip.setFaction(PlayerShip.getFaction());
					PlayerShip.invalidate();
					PlayerShip = buyShip;
					PlayerShip.updateValue();
					PlayerShip.updateWeight();
				}
			}
		}
		if( detailToggleButton.isClicked() == true ) {
			if( PlayerShip.getDockInfo().shipSelection !is null ) {
				PlayerShip.getDockInfo().detailedView = !PlayerShip.getDockInfo().detailedView;
				if( PlayerShip.getDockInfo().detailedView == false && PlayerShip.getDockInfo().shipSelection is PlayerShip ) PlayerShip.getDockInfo().shipSelection = null;
			} else {
				PlayerShip.getDockInfo().detailedView = false;
				PlayerShip.getDockInfo().message = "Please select a ship.";
			}
		}
		if( myShipDetailButton.isClicked() == true ) {
			PlayerShip.getDockInfo().shipSelection = PlayerShip;
			PlayerShip.getDockInfo().detailedView = true;
		}
		if( PlayerShip.getDockInfo().detailedView == false || PlayerShip.getDockInfo().shipSelection is null ) drawShips(PlayerShip.getDockInfo().sellShips);
		else PlayerShip.getDockInfo().shipSelection.drawDisplay(DRAW_DETAILED_X, DRAW_DETAILED_Y, ITEMDISPLAY_MODE.DETAILED);
		break;
	case DOCK_MODE.EQUIPMENT:
		buy1Button.draw();
		if( buy1Button.isClicked() == true ) buyItem();
		detailToggleButton.draw();
		PlayerShip.getDockInfo().setStatus("");
		if( detailToggleButton.isClicked() == true ) {
			if( PlayerShip.getDockInfo().itemSelection !is null ) PlayerShip.getDockInfo().detailedView = !PlayerShip.getDockInfo().detailedView;
			else {
				PlayerShip.getDockInfo().detailedView = false;
				PlayerShip.getDockInfo().message = "Please select an item.";
			}
		}
		if( PlayerShip.getDockInfo().detailedView == false || PlayerShip.getDockInfo().itemSelection is null ) drawItems(PlayerShip.getDockInfo().sellItems);
		else PlayerShip.getDockInfo().itemSelection.drawDisplay(DRAW_DETAILED_X, DRAW_DETAILED_Y, ITEMDISPLAY_MODE.DETAILED);
		break;
	case DOCK_MODE.WARES:
		buy1Button.draw();
		buy10Button.draw();
		buy100Button.draw();
		detailToggleButton.draw();
		PlayerShip.getDockInfo().setStatus("Values on these items may fluxuate.");
		if( detailToggleButton.isClicked() == true ) {
			if( PlayerShip.getDockInfo().itemSelection !is null ) PlayerShip.getDockInfo().detailedView = !PlayerShip.getDockInfo().detailedView;
			else {
				PlayerShip.getDockInfo().detailedView = false;
				PlayerShip.getDockInfo().message = "Please select an item.";
			}
		}
		if( buy1Button.isClicked() == true ) buyItem();
		if( buy10Button.isClicked() == true ) buyItem(10);
		if( buy100Button.isClicked() == true ) buyItem(100);
		if( PlayerShip.getDockInfo().detailedView == false || PlayerShip.getDockInfo().itemSelection is null ) drawItems(PlayerShip.getDockInfo().sellItems);
		else PlayerShip.getDockInfo().itemSelection.drawDisplay(DRAW_DETAILED_X, DRAW_DETAILED_Y, ITEMDISPLAY_MODE.DETAILED, false, null, PlayerShip.getDockInfo().priceScale[PlayerShip.getDockInfo().itemSelection.getCargoType()]);
		break;
	case DOCK_MODE.JOBS:
		acceptJobButton.draw();
		detailToggleButton.draw();
		PlayerShip.getDockInfo().setStatus("");
		if( acceptJobButton.isClicked() == true ) {
			if( PlayerShip.getCurrentJob() !is null ) {
				PlayerShip.getDockInfo().message = "You already have a job.";
			} else if( PlayerShip.getDockInfo().jobSelection is null ) {
				PlayerShip.getDockInfo().message = "Please select a job.";
			} else if( PlayerShip.acceptNewJob(PlayerShip.getDockInfo().jobSelection) == false ) {
				PlayerShip.getDockInfo().message = "Failed to accept job.";
			} else {
				PlayerShip.getDockInfo().message = "Job accepted!";
				PlayerShip.getDockInfo().jobSelection.setWaypointsFor(PlayerShip);
			}
		}
		if( detailToggleButton.isClicked() == true ) {
			if( PlayerShip.getDockInfo().jobSelection !is null ) PlayerShip.getDockInfo().detailedView = !PlayerShip.getDockInfo().detailedView;
			else {
				PlayerShip.getDockInfo().detailedView = false;
				PlayerShip.getDockInfo().message = "Please select a job.";
			}
		}
		if( PlayerShip.getCurrentJob() !is null ) bigFont.draw("You have already accepted a job.", 5, 160, 255, 255, 255, 255);
		else if( PlayerShip.getDockInfo().detailedView == false || PlayerShip.getDockInfo().jobSelection is null ) drawJobs(PlayerShip.getDockInfo().jobs);
		else PlayerShip.getDockInfo().jobSelection.drawDisplay(DRAW_DETAILED_X, DRAW_DETAILED_Y, ITEMDISPLAY_MODE.DETAILED);
		break;
	case DOCK_MODE.MYITEMS:
		sell1Button.draw();
		sell10Button.draw();
		sell100Button.draw();
		PlayerShip.getDockInfo().setStatus("");
		if( sell1Button.isClicked() == true ) sellItem();
		if( sell10Button.isClicked() == true ) sellItem(10);
		if( sell100Button.isClicked() == true ) sellItem(100);
		detailToggleButton.draw();
		installToggleButton.draw();
		installAllButton.draw();
		if( detailToggleButton.isClicked() == true ) {
			if( PlayerShip.getDockInfo().itemSelection !is null ) PlayerShip.getDockInfo().detailedView = !PlayerShip.getDockInfo().detailedView;
			else {
				PlayerShip.getDockInfo().detailedView = false;
				PlayerShip.getDockInfo().message = "Please select an item.";
			}
		}
		if( installToggleButton.isClicked() == true ) {
			FU_ITEM item = PlayerShip.getDockInfo().itemSelection;
			if( item !is null ) {
				if( item.getType() == ITEM_TYPE.UTILITY || item.getType() == ITEM_TYPE.WEAPON ) {
					if( item.isInstalled() == true ) {
						PlayerShip.uninstallItem(item);
						PlayerShip.getDockInfo().message = "Item uninstalled.";
					} else {
						switch( PlayerShip.installItem(item) ) {
						case 0:
							PlayerShip.getDockInfo().message = "Item installed.";
							break;
						case -2:						
							PlayerShip.getDockInfo().message = "This item's level is too high for your ship.";
							break;
						case -3:						
							PlayerShip.getDockInfo().message = "Maximum weapon count reached.";
							break;
						case -4:						
							PlayerShip.getDockInfo().message = "Maximum utility count reached.";
							break;
						default:
							PlayerShip.getDockInfo().message = "This item cannot be installed.";
							break;
						}
					}		
				} else PlayerShip.getDockInfo().message = "This item cannot be installed.";
			} else PlayerShip.getDockInfo().message = "Please select an item.";
		}
		if( installAllButton.isClicked() == true ) {
			bool instSuccess = true;
			PlayerShip.items.reset();
			while(!PlayerShip.items.last()) {
				if( PlayerShip.items.data.getType() == ITEM_TYPE.UTILITY || PlayerShip.items.data.getType() == ITEM_TYPE.WEAPON ) {
					if( PlayerShip.items.data.isInstalled() == false ) {
						PlayerShip.items.backup();
						if( PlayerShip.installItem(PlayerShip.items.data) != 0 ) instSuccess = false;
						PlayerShip.items.restore();
					}
				}
			}
			if( instSuccess == true ) PlayerShip.getDockInfo().message = "All items installed.";
			else PlayerShip.getDockInfo().message = "Some items not installed.";
		}
		if( PlayerShip.getDockInfo().detailedView == false || PlayerShip.getDockInfo().itemSelection is null ) drawItems(PlayerShip.items);			
		else PlayerShip.getDockInfo().itemSelection.drawDisplay(DRAW_DETAILED_X, DRAW_DETAILED_Y, ITEMDISPLAY_MODE.DETAILED, false, null, PlayerShip.getDockInfo().priceScale[PlayerShip.getDockInfo().itemSelection.getCargoType()]);
		if( PlayerShip.getCurrentJob() !is null &&
			PlayerShip.getDockInfo().itemSelection !is null ) {
				foreach(FU_OBJECTIVE jO; PlayerShip.getCurrentJob().jobObjective) {
					if( jO.isTarget(PlayerShip.getDockInfo().itemSelection) == true ) {
						deliverButton.draw();
						if( deliverButton.isClicked() == true ) {
							PlayerShip.getCurrentJob().handleAction(null,PlayerShip.getDockInfo().itemSelection,JOB_ACTION.RETURN);
							PlayerShip.removeItem(PlayerShip.getDockInfo().itemSelection);
							PlayerShip.getDockInfo().resetControl();
							PlayerShip.getDockInfo().message = "Item returned.";
							PlayerShip.updateValue();
							PlayerShip.updateWeight();
						}
						break;
					}
				}
		}
		break;
	}
}

void sellItem(int count = 1) {
	FU_ITEM item = PlayerShip.getDockInfo().itemSelection;
	if( item is null ) {
		PlayerShip.getDockInfo().message = "Please select an item to sell.";
		return;
	}
	if( count < 1 ) return;
	long itemPrice = item.getValue(true, false);
	if( item.getType() == ITEM_TYPE.CARGO ) {
		if( count > item.getVars().counter ) {
			PlayerShip.getDockInfo().message = "Not enough available for sale.";
			return;
		}
		itemPrice = cast(long)(cast(float)itemPrice * PlayerShip.getDockInfo().priceScale[item.getCargoType()]);
		itemPrice *= count;
	} else if( count > 1 ) {
		PlayerShip.getDockInfo().message = "Not enough available for sale.";
		return;
	} else itemPrice = cast(long)(cast(float)itemPrice * SELLBACK);
	if( itemPrice < 1 ) itemPrice = 1;
	playerCredits += itemPrice;
	if( item.getType() == ITEM_TYPE.CARGO ) {
		item.getVars().counter -= count;
	}
	if( item.getType() == ITEM_TYPE.CARGO && item.getVars().counter == 0 || item.getType() != ITEM_TYPE.CARGO ) {
		PlayerShip.getDockInfo().itemSelection = null;
		PlayerShip.getDockInfo().detailedView = false;
		PlayerShip.removeItem(item);
	}
	PlayerShip.updateValue();
	PlayerShip.updateWeight();
	PlayerShip.getDockInfo().message = "Sale completed.";
}

void buyItem(int count = 1) {
	FU_ITEM item = PlayerShip.getDockInfo().itemSelection;
	if( item is null ) {
		PlayerShip.getDockInfo().message = "Please select an item to buy.";
		return;
	}
	if( count < 1 ) return;
	long itemPrice = item.getValue(true, false);
	if( item.getType() == ITEM_TYPE.CARGO ) {
		if( count > item.getVars().counter ) {
			PlayerShip.getDockInfo().message = "Not enough items available.";
			return;
		}
		itemPrice = cast(long)(cast(float)itemPrice * PlayerShip.getDockInfo().priceScale[item.getCargoType()]);
		itemPrice *= count;
	}
	if( itemPrice > playerCredits ) {
		PlayerShip.getDockInfo().message = "Not enough credits.";
		return;
	}
	if( PlayerShip.addItem(item, count) == false ) {
		PlayerShip.getDockInfo().message = "Weight/amount limit exceeded.";
		return;
	}
	if( itemPrice < 1 ) itemPrice = 1;
	playerCredits -= itemPrice;
	if( item.getType() == ITEM_TYPE.CARGO && item.getVars().counter == 0 || item.getType() != ITEM_TYPE.CARGO ) {
		PlayerShip.getDockInfo().itemSelection = null;
		PlayerShip.getDockInfo().detailedView = false;
		PlayerShip.getDockInfo().sellItems.reset();
		while(!PlayerShip.getDockInfo().sellItems.last()) {
			if( PlayerShip.getDockInfo().sellItems.data is item ) PlayerShip.getDockInfo().sellItems.remove();
		}
	}
	PlayerShip.getDockInfo().message = "Purchase completed.";
}

void drawShips(dlinkedlist!(FU_SHIP) ships) {
	if( ships is null || ships.empty() == true ) {
		bigFont.draw("No ships are available.", 5, 160, 255, 255, 255, 255);
		return;
	}
	ships.reset();
	int drawX = 5, drawY = 160;
	while(!ships.last()) {
		ships.data.drawDisplay(drawX, drawY, ITEMDISPLAY_MODE.NORMAL,PlayerShip.getDockInfo().shipSelection is ships.data);
		if( ships.data.displayClicked() == true ) PlayerShip.getDockInfo().shipSelection = ships.data;
		drawY += DISPLAY_HEIGHT[ITEMDISPLAY_MODE.NORMAL];
		if( drawY > 600 - DISPLAY_HEIGHT[ITEMDISPLAY_MODE.NORMAL] ) {
			// wrap listings
			drawY = 160;
			drawX += DISPLAY_WIDTH[ITEMDISPLAY_MODE.NORMAL];
		}
	}
}

void drawJobs(dlinkedlist!(FU_JOB) jobs) {
	bool anyJobs = false;
	if( jobs !is null ) {
		jobs.reset();
		int drawX = 5, drawY = 160;
		while(!jobs.last()) {
			if( jobs.data.isActivated() == true ) continue;
			anyJobs = true;
			jobs.data.drawDisplay(drawX, drawY, ITEMDISPLAY_MODE.NORMAL, PlayerShip.getDockInfo().jobSelection is jobs.data );
			if( jobs.data.displayClicked() == true ) PlayerShip.getDockInfo().jobSelection = jobs.data;
			drawY += DISPLAY_HEIGHT[ITEMDISPLAY_MODE.NORMAL];
			if( drawY > 600 - DISPLAY_HEIGHT[ITEMDISPLAY_MODE.NORMAL] ) {
				// wrap listings
				drawY = 160;
				drawX += DISPLAY_WIDTH[ITEMDISPLAY_MODE.NORMAL];
			}
		}
	}
	if( anyJobs == false ) {
		bigFont.draw("No current jobs are available.", 5, 160, 255, 255, 255, 255);
		return;
	}
}

void drawItems(dlinkedlist!(FU_ITEM) items, long *totalRepairCost = null) {
	if( items is null || items.empty() == true ) {
		bigFont.draw("No items are available.", 5, 160, 255, 255, 255, 255);
		return;
	}
	items.reset();
	uint drawY = 160, drawX = 5;
	while(!items.last()) {
		switch( PlayerShip.getDockInfo().currentDockMode ) {
		case DOCK_MODE.EQUIPMENT:
		case DOCK_MODE.REPAIR:
			if( items.data.getType() == ITEM_TYPE.CARGO ) continue;
			if( items.data.displayClicked() == true ) PlayerShip.getDockInfo().itemSelection = items.data;
			items.data.drawDisplay(drawX, drawY, ITEMDISPLAY_MODE.NORMAL,items.data is PlayerShip.getDockInfo().itemSelection, totalRepairCost );
			break;
		case DOCK_MODE.WARES:
			if( items.data.getType() != ITEM_TYPE.CARGO ) continue;
		case DOCK_MODE.MYITEMS:
			if( items.data.displayClicked() == true ) PlayerShip.getDockInfo().itemSelection = items.data;
			items.data.drawDisplay(drawX, drawY, ITEMDISPLAY_MODE.NORMAL,items.data is PlayerShip.getDockInfo().itemSelection, null, PlayerShip.getDockInfo().priceScale[items.data.getCargoType()]);
			break;
		}
		drawY += DISPLAY_HEIGHT[ITEMDISPLAY_MODE.NORMAL];
		if( drawY > 600 - DISPLAY_HEIGHT[ITEMDISPLAY_MODE.NORMAL] ) {
			// wrap listings
			drawY = 160;
			drawX += DISPLAY_WIDTH[ITEMDISPLAY_MODE.NORMAL];
		}
	}
}
