﻿module init.init;

public import arc.gfx.sprite;
public import arc.templates.dlinkedlist;
public import arc.phy.time;
public import arc.io.sound;
public import arc.phy.phyutil;
public import arc.phy.point;
public import arc.gfx.font;
public import arc.gfx.frame.frame;
public import arc.io.input;
public import arc.gfx.prim;
public import arc.gfx.texture;
public import arc.io.camera;
public import arc.io.serialization.serializer;
public import derelict.sdl.keysym;

public import std.c.string;
public import std.stream;
public import std.math;
public import std.file;
public import std.random;
public import std.gc;
public import std.stdio;
public import std.mmfile;

private import classes.item;
private import classes.objectholder; 
private import classes.dockinfo;
private import classes.job;
private import space.handler;
private import space.hud;
private import init.player;
private import space.generation;
private import classes.fu_object;
private import classes.ship;
private import classes.particle;
private import menu.main_menu;
private import classes.gui;
private import std.zlib;

// big linked list of all loaded space objects
ObjectHolder GameObjects; 
//dlinkedlist!(FU_OBJECT) SpaceObjects;
//dlinkedlist!(FU_PARTICLE) SpaceParticles;

// cameras
Camera space_view;
Camera interface_view;

// screen size
static const int SCREEN_X = 800, SCREEN_Y = 600;

// object color ranges
static const int COLOR_RANGE = 100;

// wildcard standard constant
static const int MAX_TARGETABLE = 700;
static const int WILDCARD = -1;
static const int MAX_SENSOR_RANGE = 800 * 4;
static const int MAX_SHIPS = 40;
static const int MAX_ITEMS = 24;
static const float SELLBACK = 0.65;
static const float ENGINE_SPEED = 1.75;

// item display width/heights
int[3] DISPLAY_HEIGHT;
int[3] DISPLAY_WIDTH;

// timer object
Time gameTime;

// pointer to the player's ship
FU_SHIP PlayerShip;

// fonts
Font standardFont;
Font smallFont;
Font bigFont;
Font largeFont;

// space generator
FU_SpaceGenerator MainGenerator;

// all names
char[][] FactionNames;
char[][] AIClassNames;
char[][] AIActionNames;
char[][] ObjectNames;
char[][] ShipNames;
char[][] ShipTypeNames;
char[][] ItemNames;
char[][] CargoNames;
char[][] ShortItemNames;
char[][] DockableNames;
char[][] jobNames;
char[][] stormNames;
char[][] modifierNames;
char[][] modifierPlusMinus;

// fill this list as needed
enum GAME_STATE {
	MAIN_MENU,
	NEW_GAME,
	LOAD_GAME,
	SPACE,
	DOCKED,
	QUIT
};

public int gameState;
ulong globalTimer = 1;
// special selection variables
bool selectionMade = false;
int autoSelection = AUTO_SELECT.NULL;
float nearestSelected = 360.0;
int playerNeedsAdd = 0;
bool playerUsingCloak = false;

static const int MODIFIER_SEPERATOR = 30;
static const int BAD_MODIFIER_COUNT = 6;
static const int GOOD_MODIFIER_COUNT = 6;

enum MODIFIER_TYPE {
	// bad things
	DISABLE_SHIELD_CHARGE = 0, // OK
	DISABLE_ENGINES, // OK
	DISABLE_WEAPONS, // OK
	DISABLE_CLOAK, // OK
	DISABLE_RADAR, // OK
	DISABLE_ENERGY_CHARGE, // OK

	// good things
	REPAIR_HULL = MODIFIER_SEPERATOR, // OK
	REPAIR_ITEMS, // OK
	INCREASE_SPEED, // OK
	INCREASE_DAMAGE, // OK
	INCREASE_SHIELDS_CHARGE, // OK
	INCREASE_ENERGY_CHARGE, // OK
}	

enum JOB_ACTION {
	KILL,
	RETURN,
	NEAR
};

enum AUTO_SELECT {
	NULL,		// not autoselecting anything
	ALL,		// just select nearest object
	SHIP,		// select nearest ship
	ENEMY,		// select nearest guy to attack you
	DOCK, 		// select nearest dockable object
	ITEM		// select nearest item
};

// basic item types
enum ITEM_TYPE {
	NULL,    // invalid item (nothing)
	CARGO,   // only goes into cargo space
	WEAPON,  // can use a weapon slot
	UTILITY  // "under the hood" items (e.g. generator, turning device, afterburner)
				  // can use a utility slot
};

enum STORM_TYPE {
	NULL,	// no storms, yay!
	RED,	// fire, armor
	BLUE,	// ion, shields
	WHITE,	// emp, reduces energy
	GREEN,	// disables sensors, targets
	PURPLE,	// space time, teleports / rotates ships
};

// item classes, seperated by 25
enum ITEM_CLASS {
	ENGINE = 0,		// basic acceleration
	AFTERBURNER,	// extra acceleration, less efficient
	CRUISER,		// super acceleration, takes time to activate
	GENERATOR,		// makes energy
	TURNER,		// accelerates turning
	SENSOR,		// facillitates radar
	CLOAKING,		// cloaking device
	SHIELDS,		// sheild device
	STRAFER,		// sideway acceleration
	
	// weapons
	PULSE = 25,			// simple 'pulse' fire weapon, medium everything
	SNIPER,		// slow rate of fire, high energy and damage
	SPREAD,			// low damage, fast rate of fire, low energy, bad accuracy
	MISSLE,		// high homing, high damage, low rate of fire, ammo counter
	TORPEDO,		// lower homing, very high damage, very low rate of fire, ammo counter
	DEATHRAY,		// takes time to charge, strong damage, high energy, fast
	MODIFIER,		// applies effects to other ships
	TURRET,			// pulse-like weapon with turret ability
	
	// common
	JUNK = 50,			// space trash
	VALUABLE,		// diamonds, worthy tradables/food
	CONTRABAND,	// illegal stuff cops hate
	UNIQUE			// questing/adventure items
}

static const int MAX_WEAPONS = 8;
static const int MAX_UTILS = 9;
static const int MAX_CARGO = 4;

enum SOUND_CLASS {
	SHOOT,
	TELEPORT,
	SHIELD_HIT,
	SMALL_EXPLOSION,
	BIG_EXPLOSION,
	SELECT,
	ENGINE,
	TRAN_TRAVEL
};

SoundFile[][SOUND_CLASS.max+1] Sounds;

enum OBJECTIVE_TYPE {
	DESTROY,
//	LOCATE,
	FOLLOW,
	RETURN,
	PROTECT,
//	DELIVER
};

enum ITEMDISPLAY_MODE {
	COMPACT,
	NORMAL,
	DETAILED
};

enum TEXTURES {
	SHIP,
	STATION,
	TELEGATE,
	ROCK,
	PLANET,
	PROJECTILE,
	ITEM,
	TRANSPATH,
	DOCKEDPLANET,
	DOCKEDSTATION,
	PORTHOLE
};

static const int JUNK_COUNT = 4;
static const int UNIQUE_COUNT = 2;
static const int VALUABLE_COUNT = 8;
static const int CONTRABAND_COUNT = 4;

long CargoPrices[80];

// 25 count difference for each category
enum ITEM_CARGO {
	// junk
	PAPER = 0,
	SCRAP_METAL,
	WASTE,
	ROCK,
	
	// unique
	DATA_CUBE = 25,
	ORGANISMS,
	
	// valuables
	FOOD = 50,
	WATER,
	COPPER,
	SILVER,
	GOLD,
	DIAMOND,
	MEDICINE,
	FUEL,
	
	// CONTRABAND
	IDRUG = 75,
	COUNTERFEIT,
	WMD,
	POISON
};

// texture IDs
Texture[][TEXTURES.max+1] GameTextures;

// player's credits
long playerCredits;
char[] playerName = "Anonymous";
// player's faction
int playerFaction = 50;

void updatePlayerFaction(int targetFaction, int amount, bool goodFlag = false) {
	if( goodFlag == true ) targetFaction = 4 - targetFaction;
	for(int i=0;i<amount;i++) {
		switch( targetFaction ) {
		case 0:
			if( playerFaction < 50 ) playerFaction++;
			else amount = i;
			break;
		case 1:
			if( playerFaction < 70 ) playerFaction++;
			else amount = i;
			break;
		case 2:
			if( goodFlag == false || playerFaction <= 50 ) playerFaction++;
			else playerFaction--;
			break;
		case 3:
			if( playerFaction > 30 ) playerFaction--;
			else amount = i;
			break;
		case 4:
			if( playerFaction > 50 ) playerFaction--;
			else amount = i;
			break;
		}
	}
	if( playerFaction > 99 ) playerFaction = 99;
	else if( playerFaction < 0 ) playerFaction = 0;
	PlayerShip.setFaction(playerFaction/20);
}

// docking buttons
FU_BUTTON TOP_repairButton;
FU_BUTTON TOP_shipsButton;
FU_BUTTON TOP_equipButton;
FU_BUTTON TOP_waresButton;
FU_BUTTON TOP_jobButton;
FU_BUTTON TOP_myItemsButton;
FU_BUTTON TOP_exitButton;
// more buttons
FU_BUTTON buy1Button;
FU_BUTTON buy10Button;
FU_BUTTON buy100Button;
FU_BUTTON sell1Button;
FU_BUTTON sell10Button;
FU_BUTTON sell100Button;
FU_BUTTON repairButton;
FU_BUTTON repairAllButton;
FU_BUTTON repairHullButton;
FU_BUTTON detailToggleButton;
FU_BUTTON myShipDetailButton;
FU_BUTTON installToggleButton;
FU_BUTTON installAllButton;
FU_BUTTON acceptJobButton;
FU_BUTTON deliverButton;

enum DOCK_MODE {
	REPAIR,
	SHIPS,
	EQUIPMENT,
	WARES,
	JOBS,
	MYITEMS
};

enum OBJECT_TYPE {
	NULL,		// to be removed from list
	ROCK,		// death may cause items to pop
	SHIP,		// all ship types
	STATION,	// dockable
	PLANET,		// dockable with insane armor/weight rating
	TELEGATE,	// transportation
	TRANSPATH,	// fast travel pathway
	PROJECTILE,	// weapon-fired object
	ITEM,		// floating item
	PORTHOLE	// telegate-like, randomly placed, random teleportation
};

float tweakValue(float inValue, int percent) {
	if( percent == 0 ) return inValue;
	int randValue = rand()%percent;
	randValue -= percent/2;
	float multValue = 1.0 + (cast(float)randValue / 100.0);
	return inValue * multValue;
}

// needs work
double getIntercept(double myspeed, double mypos, double targetspeed, double targetpos) {
	return targetpos - ((mypos - targetpos) / (myspeed - targetspeed));
}

private extern (C) Object _d_newclass (ClassInfo info);

FU_ITEM ItemCopy(FU_ITEM source) {
	if (source is null) return null;
	void *copy = cast(void*)_d_newclass (FU_ITEM.classinfo);
	void *copyVars = cast(void*)_d_newclass (FU_ITEMVARS.classinfo);
	size_t size = FU_ITEM.classinfo.init.length;
	size_t sizeVars = FU_ITEMVARS.classinfo.init.length;
	copy [8 .. size] = (cast (void *) source) [8 .. size];
	copyVars [8 .. sizeVars] = (cast (void *) source.getVars()) [8 .. sizeVars];
	(cast(FU_ITEM)copy).setVars(cast(FU_ITEMVARS)copyVars);
	return cast(FU_ITEM) copy;
}

void InitButtons() {
	int startX = 155, startY = 65;
	TOP_repairButton = new FU_BUTTON("Repair");
	TOP_repairButton.setPosition(startX,startY);
	startX += TOP_repairButton.getWidth()+3;
	TOP_shipsButton = new FU_BUTTON("Ships");
	TOP_shipsButton.setPosition(startX,startY);
	startX += TOP_shipsButton.getWidth()+3;
	TOP_equipButton = new FU_BUTTON("Equipment");
	TOP_equipButton.setPosition(startX, startY);
	startX += TOP_equipButton.getWidth()+3;
	TOP_waresButton = new FU_BUTTON("Wares");
	TOP_waresButton.setPosition(startX, startY);
	startX += TOP_waresButton.getWidth()+3;
	TOP_jobButton = new FU_BUTTON("Jobs");
	TOP_jobButton.setPosition(startX, startY);
	startX += TOP_jobButton.getWidth()+3;
	TOP_myItemsButton = new FU_BUTTON("My Items");
	TOP_myItemsButton.chopWidth(7);
	TOP_myItemsButton.setPosition(startX, startY);
	startX += TOP_myItemsButton.getWidth()+3;
	TOP_exitButton = new FU_BUTTON("Exit");
	TOP_exitButton.chopWidth(5);
	TOP_exitButton.setPosition(startX, startY);
	// more buttons
	startY = 157; startX = 796;
	buy1Button = new FU_BUTTON("Buy One");
	repairButton = new FU_BUTTON("Repair");
	sell1Button = new FU_BUTTON("Sell One");
	acceptJobButton = new FU_BUTTON("Accept");
	acceptJobButton.setPosition(startX - acceptJobButton.getWidth(), startY);
	repairButton.setPosition(startX - repairButton.getWidth(), startY);
	buy1Button.setPosition(startX - buy1Button.getWidth(), startY);
	sell1Button.setPosition(startX - sell1Button.getWidth(), startY);
	startY += buy1Button.getHeight() + 4;
	buy10Button = new FU_BUTTON("Buy Ten");
	repairAllButton = new FU_BUTTON("Repair All");
	repairAllButton.chopWidth(15);
	sell10Button = new FU_BUTTON("Sell Ten");
	repairAllButton.setPosition(startX - repairAllButton.getWidth(), startY);
	buy10Button.setPosition(startX - buy10Button.getWidth(), startY);
	sell10Button.setPosition(startX - sell10Button.getWidth(), startY);
	startY += buy10Button.getHeight() + 4;
	buy100Button = new FU_BUTTON("Buy 100");
	sell100Button = new FU_BUTTON("Sell 100");
	repairHullButton = new FU_BUTTON("Repair Hull");
	repairHullButton.chopWidth(20);
	repairHullButton.setPosition(startX - repairHullButton.getWidth(), startY);
	buy100Button.setPosition(startX - buy100Button.getWidth(), startY);
	sell100Button.setPosition(startX - sell100Button.getWidth(), startY);
	startY += buy100Button.getHeight() + 4;
	detailToggleButton = new FU_BUTTON("Toggle Detail");
	detailToggleButton.chopWidth(23);
	detailToggleButton.setPosition(startX - detailToggleButton.getWidth(), startY);
	startY += detailToggleButton.getHeight() + 4;
	installToggleButton = new FU_BUTTON("(Un)install");
	myShipDetailButton = new FU_BUTTON("My Ship's\nDetail");
	myShipDetailButton.chopWidth(100);
	installToggleButton.chopWidth(23);
	installToggleButton.setPosition(startX - installToggleButton.getWidth(), startY);
	myShipDetailButton.setPosition(startX - myShipDetailButton.getWidth(), startY);
	startY += installToggleButton.getHeight() + 4;
	installAllButton = new FU_BUTTON("Install All");
	installAllButton.chopWidth(20);
	installAllButton.setPosition(startX - installAllButton.getWidth(), startY);
	startY += installAllButton.getHeight() + 4;
	deliverButton = new FU_BUTTON("Deliver");
	deliverButton.chopWidth(5);
	deliverButton.setPosition(startX - deliverButton.getWidth(), startY);
}

Texture getTexture(int parmTextureClass, int parmTextureValue = -1, int *valueUsed = null) {
	debug scope(failure) { writefln("getTexture: init.d"); }
	if( parmTextureValue < 0 ) parmTextureValue = rand()%GameTextures[parmTextureClass].length;
	if( valueUsed != null ) *valueUsed = parmTextureValue;
	return GameTextures[parmTextureClass][parmTextureValue];
}

void quitCurrentGame() {
	MainGenerator.saveCurrentCell();
	GameObjects.clear();
	MainGenerator.resetGenerator();
	playerCredits = 0;
	playerName = "Anonymous";
	playerUsingCloak = false;
	PlayerShip = null;
	gameState = GAME_STATE.MAIN_MENU;
	selectionMade = false;
	autoSelection = AUTO_SELECT.NULL;
	nearestSelected = 360.0;
	playerNeedsAdd = 0;
	std.gc.fullCollect();
}

static const int SOUND_CHANNELS = 12;

bool fill_SNDs() {
	debug scope(failure) { writefln("fill_SNDs: init.d"); }
	char[][] dirListing;
	dirListing = listdir("snd/engine/","*.wav");
	foreach(char[] WAVfile; dirListing) Sounds[SOUND_CLASS.ENGINE] ~= new SoundFile(WAVfile);
	if( Sounds[SOUND_CLASS.ENGINE].length == 0 ) return false;
	dirListing = listdir("snd/large_explosion/","*.wav");
	foreach(char[] WAVfile; dirListing) Sounds[SOUND_CLASS.BIG_EXPLOSION] ~= new SoundFile(WAVfile);
	if( Sounds[SOUND_CLASS.BIG_EXPLOSION].length == 0 ) return false;	
	dirListing = listdir("snd/selection/","*.wav");
	foreach(char[] WAVfile; dirListing) Sounds[SOUND_CLASS.SELECT] ~= new SoundFile(WAVfile);
	if( Sounds[SOUND_CLASS.SELECT].length == 0 ) return false;
	dirListing = listdir("snd/shield/","*.wav");
	foreach(char[] WAVfile; dirListing) Sounds[SOUND_CLASS.SHIELD_HIT] ~= new SoundFile(WAVfile);
	if( Sounds[SOUND_CLASS.SHIELD_HIT].length == 0 ) return false;	
	dirListing = listdir("snd/shoot/","*.wav");
	foreach(char[] WAVfile; dirListing) Sounds[SOUND_CLASS.SHOOT] ~= new SoundFile(WAVfile);
	if( Sounds[SOUND_CLASS.SHOOT].length == 0 ) return false;
	dirListing = listdir("snd/small_explosion/","*.wav");
	foreach(char[] WAVfile; dirListing) Sounds[SOUND_CLASS.SMALL_EXPLOSION] ~= new SoundFile(WAVfile);
	if( Sounds[SOUND_CLASS.SMALL_EXPLOSION].length == 0 ) return false;
	dirListing = listdir("snd/trantravel/","*.wav");
	foreach(char[] WAVfile; dirListing) Sounds[SOUND_CLASS.TRAN_TRAVEL] ~= new SoundFile(WAVfile);
	if( Sounds[SOUND_CLASS.TRAN_TRAVEL].length == 0 ) return false;	
	for(int i=0;i<(SOUND_CHANNELS+1);i++)
		playChannel[i] = new Sound(Sounds[SOUND_CLASS.SELECT][0]);
	return true;
}

Sound playChannel[SOUND_CHANNELS+1];
float playChannel_Volume[SOUND_CHANNELS];

void playSingleSound(int soundClass, int soundIndex = WILDCARD, float volume = 1.0) {
	if( soundIndex == WILDCARD ) soundIndex = random_range(0,Sounds[soundClass].length-1);
	playChannel[SOUND_CHANNELS].stop();
	playChannel[SOUND_CHANNELS].setSound(Sounds[soundClass][soundIndex]);
	playChannel[SOUND_CHANNELS].setVolume(volume);
	playChannel[SOUND_CHANNELS].play();
}

int playSound(int soundClass, double sourceX = 0.0, double sourceY = 0.0, int soundIndex = WILDCARD, bool useLoop = false, float volume = 1.0) {
	debug scope(failure) { writefln("playSound: init.d"); }
	if( soundIndex == WILDCARD ) soundIndex = random_range(0,Sounds[soundClass].length-1);
	double dist, pan;
	if( sourceX == 0.0 && sourceY == 0.0 ) {
		dist = 0.0;
		pan = 0.0;
	} else {
		dist = distance(PlayerShip.getX(), PlayerShip.getY(), sourceX, sourceY);
		pan = (sourceX - PlayerShip.getX());
	}
	if( dist > 1500.0 || Sounds[soundClass][soundIndex] is null ) return -1;
	for(int i=0;i<SOUND_CHANNELS;i++) {
		if( playChannel[i].getPaused() == true ) {
			playChannel[i].stop();
			playChannel_Volume[i] = volume;
			playChannel[i].setSound(Sounds[soundClass][soundIndex]);
			playChannel[i].setPosition(pan / 1500.0, 0.0);
			playChannel[i].setVolume((1.0 - (dist/1500.0))*volume);
			playChannel[i].setLoop(useLoop);
			playChannel[i].play();
			return i;
		}
	}
	return -1;
}

bool stopSound(int soundIndex) {
	if( soundIndex < 0 || soundIndex >= SOUND_CHANNELS ) return false;
	if( playChannel[soundIndex].getPaused() == true ) return false;
	playChannel[soundIndex].stop();
	return true;
}

bool updateSound(int soundIndex, double sourceX = 0.0, double sourceY = 0.0, double speed = 0.0) {
	if( soundIndex < 0 || soundIndex >= SOUND_CHANNELS ) return false;
	if( playChannel[soundIndex].getPaused() == true ) return false;
	double dist = distance(PlayerShip.getX(), PlayerShip.getY(), sourceX, sourceY);
	if( dist > 1500.0 ) {
		stopSound(soundIndex);
		return false;
	} else {
		double pan = (sourceX - PlayerShip.getX());
		playChannel[soundIndex].setPitch(1.0 + speed);
		playChannel[soundIndex].setPosition(pan / 1500.0, 0.0);
		playChannel[soundIndex].setVolume((1.0 - (dist/1500.0))*playChannel_Volume[soundIndex]);
		return true;
	}
}

// read /gfx directory structure and fill texture IDs
bool fill_TIDs() {
	debug scope(failure) { writefln("fill_TIDs: init.d"); }
	char[][] dirListing = listdir("gfx/ships/","*.png");
	foreach(char[] PNGfile; dirListing) GameTextures[TEXTURES.SHIP] ~= arc.gfx.texture.texture(PNGfile);
	if( GameTextures[TEXTURES.SHIP].length == 0 ) return false;
	dirListing = listdir("gfx/stations/","*.png");
	foreach(char[] PNGfile; dirListing) GameTextures[TEXTURES.STATION] ~= arc.gfx.texture.texture(PNGfile);
	if( GameTextures[TEXTURES.STATION].length == 0 ) return false;
	dirListing = listdir("gfx/telegates/","*.png");
	foreach(char[] PNGfile; dirListing) GameTextures[TEXTURES.TELEGATE] ~= arc.gfx.texture.texture(PNGfile);
	if( GameTextures[TEXTURES.TELEGATE].length == 0 ) return false;
	dirListing = listdir("gfx/transpaths/","*.png");
	foreach(char[] PNGfile; dirListing) GameTextures[TEXTURES.TRANSPATH] ~= arc.gfx.texture.texture(PNGfile);
	if( GameTextures[TEXTURES.TRANSPATH].length == 0 ) return false;
	dirListing = listdir("gfx/rocks/","*.png");
	foreach(char[] PNGfile; dirListing) GameTextures[TEXTURES.ROCK] ~= arc.gfx.texture.texture(PNGfile);
	if( GameTextures[TEXTURES.ROCK].length == 0 ) return false;
	dirListing = listdir("gfx/planets/","*.png");
	foreach(char[] PNGfile; dirListing) GameTextures[TEXTURES.PLANET] ~= arc.gfx.texture.texture(PNGfile);
	if( GameTextures[TEXTURES.PLANET].length == 0 ) return false;
	dirListing = listdir("gfx/projectiles/","*.png");
	foreach(char[] PNGfile; dirListing) GameTextures[TEXTURES.PROJECTILE] ~= arc.gfx.texture.texture(PNGfile);
	if( GameTextures[TEXTURES.PROJECTILE].length == 0 ) return false;
	dirListing = listdir("gfx/items/","*.png");
	foreach(char[] PNGfile; dirListing) GameTextures[TEXTURES.ITEM] ~= arc.gfx.texture.texture(PNGfile);
	if( GameTextures[TEXTURES.ITEM].length == 0 ) return false;
	dirListing = listdir("gfx/dockedstation/","*.png");
	foreach(char[] PNGfile; dirListing) GameTextures[TEXTURES.DOCKEDSTATION] ~= arc.gfx.texture.texture(PNGfile);
	if( GameTextures[TEXTURES.DOCKEDSTATION].length == 0 ) return false;
	dirListing = listdir("gfx/dockedplanet/","*.png");
	foreach(char[] PNGfile; dirListing) GameTextures[TEXTURES.DOCKEDPLANET] ~= arc.gfx.texture.texture(PNGfile);
	if( GameTextures[TEXTURES.DOCKEDPLANET].length == 0 ) return false;
	dirListing = listdir("gfx/porthole/","*.png");
	foreach(char[] PNGfile; dirListing) GameTextures[TEXTURES.PORTHOLE] ~= arc.gfx.texture.texture(PNGfile);
	if( GameTextures[TEXTURES.PORTHOLE].length == 0 ) return false;
	return true;
}

void compressFile(char[] cellFN, bool compressBool = true) {
	void[] outBuf;		
	MmFile inCell = new MmFile(cellFN);
	if( compressBool == true ) outBuf = compress(inCell.opSlice());
	else outBuf = uncompress(inCell.opSlice());
	delete inCell;
	BufferedFile outCell = new BufferedFile(cellFN,FileMode.OutNew);
	outCell.writeExact(outBuf.ptr,outBuf.length);
	outCell.close();
}

void saveCurrentGame() {
	if( exists("saves") == false ) mkdir("saves");
	char[] fileName = "saves/" ~ playerName ~ ".ship";
	
	Serializer s = new Serializer(fileName, FileMode.Out);
	serializePlayer(s);
	delete s;
	
	// compressFile(fileName,true);
}

void initPlayer(char[] parmPlayerInfoFilename) {
	if( PlayerShip !is null ) return;
	if( exists(parmPlayerInfoFilename) == 1 ) 
	{
		Serializer s = new Serializer(parmPlayerInfoFilename, FileMode.In);
		serializePlayer(s);
		delete s;
		playerNeedsAdd = 1;
	}
	else initNewPlayer();
}

// show a darkended box
void drawDarkBox(int x, int y, int w, int h, short r = 255, short g = 255, short b = 255) {
	drawRect(x, y, w, h, r, g, b, 255, false);
	drawRect(x+1, y+1, w-2, h-2, 0, 0, 0, 175, true);	
}

bool objectExists(FU_OBJECT checkObj) {
	if( checkObj is null ) return false;
	if( checkObj.getArmor() <= 0 ) return false;
	if( checkObj.getType() == OBJECT_TYPE.NULL ) return false;
	return true;
}

bool canSee(FU_SHIP host, FU_SHIP target) {
	if( target.getAlpha() > 200 ) return true;
	FU_ITEM mySensor = host.getItem(ITEM_TYPE.UTILITY, ITEM_CLASS.SENSOR, 0, true, true);
	if( mySensor is null ) return false;
	FU_ITEM yourCloaker = target.getItem(ITEM_TYPE.UTILITY, ITEM_CLASS.CLOAKING, 0, true, true);
	if( yourCloaker is null ) return true;
	if( mySensor.getLevel() > yourCloaker.getLevel() ) return true;
	return false;
}

// show a general loading screen
void showWaitScreen(char[] loadingMessage) {
	// font calculations
	float width = loadingMessage.length * 8;
	float height = 8;
	// draw it!
	arc.io.window.clear();
	bigFont.draw(loadingMessage, SCREEN_X/2-width/2,SCREEN_Y/2-height/2,255,255,255,255);
	arc.io.window.swap();
}

void InitNames() {
	debug scope(failure) { writefln("InitNames: init.d"); }
	
	modifierPlusMinus.length = 3;
	modifierPlusMinus[0] = "";
	modifierPlusMinus[1] = "(-)";
	modifierPlusMinus[2] = "(+)";
	
	modifierNames.length = MODIFIER_TYPE.max+1;
	modifierNames[MODIFIER_TYPE.DISABLE_CLOAK] 		     = "Cloak Disrupter";
	modifierNames[MODIFIER_TYPE.DISABLE_ENERGY_CHARGE]   = "Energy Disrupter";
	modifierNames[MODIFIER_TYPE.DISABLE_ENGINES] 	     = "Engine Disrupter";
	modifierNames[MODIFIER_TYPE.DISABLE_RADAR] 		     = "Radar Disrupter";
	modifierNames[MODIFIER_TYPE.DISABLE_SHIELD_CHARGE]   = "Shield Disrupter";
	modifierNames[MODIFIER_TYPE.DISABLE_WEAPONS]         = "Weapon Disrupter"; // too long?
	modifierNames[MODIFIER_TYPE.DISABLE_SHIELD_CHARGE]   = "Shield Disrupter";
	modifierNames[MODIFIER_TYPE.INCREASE_DAMAGE]         = "Damage Enhancer";
	modifierNames[MODIFIER_TYPE.INCREASE_ENERGY_CHARGE]  = "Energy Energizer";
	modifierNames[MODIFIER_TYPE.INCREASE_SHIELDS_CHARGE] = "Shield Energizer";
	modifierNames[MODIFIER_TYPE.INCREASE_SPEED]          = "Agility Enhancer";
	modifierNames[MODIFIER_TYPE.REPAIR_HULL]             = "Hull Repairer";
	modifierNames[MODIFIER_TYPE.REPAIR_ITEMS]            = "Item Repairer";
	
	stormNames.length = STORM_TYPE.max+1;
	stormNames[STORM_TYPE.NULL] = "No Storm";
	stormNames[STORM_TYPE.RED] = "Heat Storm (Red)";
	stormNames[STORM_TYPE.BLUE] = "Ion Storm (Blue)";
	stormNames[STORM_TYPE.WHITE] = "Energy Storm (White)";
	stormNames[STORM_TYPE.GREEN] = "EMP Storm (Green)";
	stormNames[STORM_TYPE.PURPLE] = "SpaceTime Storm (Purple)";
	
	jobNames.length = OBJECTIVE_TYPE.max+1;
	// jobNames[OBJECTIVE_TYPE.DELIVER] = "Deliver";
	jobNames[OBJECTIVE_TYPE.DESTROY] = "Destroy";
	jobNames[OBJECTIVE_TYPE.FOLLOW] = "Follow";
	// jobNames[OBJECTIVE_TYPE.LOCATE] = "Locate";
	jobNames[OBJECTIVE_TYPE.PROTECT] = "Protect";
	jobNames[OBJECTIVE_TYPE.RETURN] = "Return";
	
	FactionNames.length = 5;
	FactionNames[0] = "Pure Alliance";
	FactionNames[1] = "Relaxed Allies";
	FactionNames[2] = "Neutrals";
	FactionNames[3] = "Relaxed Rogues";
	FactionNames[4] = "Pure Rogues";
	
	ObjectNames.length = OBJECT_TYPE.max+1;
	ObjectNames[OBJECT_TYPE.NULL] = "Null Object";
	ObjectNames[OBJECT_TYPE.ROCK] = "Space Rock";
	ObjectNames[OBJECT_TYPE.SHIP] = "Ship";
	ObjectNames[OBJECT_TYPE.STATION] = "Space Station";
	ObjectNames[OBJECT_TYPE.PLANET] = "Planet";
	ObjectNames[OBJECT_TYPE.TELEGATE] = "Telegate";
	ObjectNames[OBJECT_TYPE.TRANSPATH] = "Transpath";
	ObjectNames[OBJECT_TYPE.PROJECTILE] = "Projectile";
	ObjectNames[OBJECT_TYPE.ITEM] = "Floating Item";
	ObjectNames[OBJECT_TYPE.PORTHOLE] = "Unstable Porthole";

	AIClassNames.length = AI_CLASS.max+1;
	AIClassNames[AI_CLASS.NONE] = "None";
	AIClassNames[AI_CLASS.TOURIST] = "Tourist";
	AIClassNames[AI_CLASS.TRAVELER] = "Traveler";
	AIClassNames[AI_CLASS.POLICE] = "Police";
	AIClassNames[AI_CLASS.EXPLORER] = "Explorer";
	AIClassNames[AI_CLASS.PIRATE] = "Pirate";
	AIClassNames[AI_CLASS.KILLER] = "Killer";
	AIClassNames[AI_CLASS.MINER] = "Miner";
	
	AIActionNames.length = AI_ACTION.max+1;
	AIActionNames[AI_ACTION.WANDERING] = "Wandering";
	AIActionNames[AI_ACTION.DOCKING] = "Docking";
	AIActionNames[AI_ACTION.PATROLING] = "Patroling";
	AIActionNames[AI_ACTION.RUNNING] = "Fleeing";
	AIActionNames[AI_ACTION.ATTACKING] = "Attacking";
	AIActionNames[AI_ACTION.FOLLOWING] = "Following";
	AIActionNames[AI_ACTION.PICKUP] = "Gathering";

	ShipTypeNames.length = SHIP_TYPE.max + 1;
	// ShipTypeNames[SHIP_TYPE.ABANDONED] = "Interceptor";
	ShipTypeNames[SHIP_TYPE.STANDARD] = "Interceptor";
	ShipTypeNames[SHIP_TYPE.POLICE] = "Corvette";
	ShipTypeNames[SHIP_TYPE.PIRATE] = "Corvette";
	ShipTypeNames[SHIP_TYPE.FIGHTER] = "Fighter";
	ShipTypeNames[SHIP_TYPE.CRUISER] = "Dreadnaught";
	ShipTypeNames[SHIP_TYPE.SHUTTLE] = "Shuttle";
	ShipTypeNames[SHIP_TYPE.CARRIER] = "Carrier";

	ShortItemNames.length = ITEM_CLASS.max+1;
	ShortItemNames[ITEM_CLASS.ENGINE] = "ENGINE";
	ShortItemNames[ITEM_CLASS.AFTERBURNER] = "AFTBURN";
	ShortItemNames[ITEM_CLASS.CRUISER] = "CRUISER";
	ShortItemNames[ITEM_CLASS.GENERATOR] = "GENRTOR";
	ShortItemNames[ITEM_CLASS.TURNER] = "TURNER";
	ShortItemNames[ITEM_CLASS.SENSOR] = "SENSOR";
	ShortItemNames[ITEM_CLASS.CLOAKING] = "CLOAK";
	ShortItemNames[ITEM_CLASS.SHIELDS] = "SHIELDS";
	ShortItemNames[ITEM_CLASS.STRAFER] = "STRAFER";
	ShortItemNames[ITEM_CLASS.PULSE] = "W_PULSE";
	ShortItemNames[ITEM_CLASS.SNIPER] = "W_SNIPR";
	ShortItemNames[ITEM_CLASS.SPREAD] = "W_SPRED";
	ShortItemNames[ITEM_CLASS.MISSLE] = "W_MISSL";
	ShortItemNames[ITEM_CLASS.TORPEDO] = "W_TORP";
	ShortItemNames[ITEM_CLASS.DEATHRAY] = "W_DRAY";
	ShortItemNames[ITEM_CLASS.MODIFIER] = "W_MODR";
	ShortItemNames[ITEM_CLASS.TURRET] = "W_TURRT";
	ShortItemNames[ITEM_CLASS.JUNK] = "JUNK";
	ShortItemNames[ITEM_CLASS.VALUABLE] = "VALUABL";
	ShortItemNames[ITEM_CLASS.CONTRABAND] = "CONTRB";
	ShortItemNames[ITEM_CLASS.UNIQUE] = "UNIQUE";

	ItemNames.length = ITEM_CLASS.max+1;
	ItemNames[ITEM_CLASS.ENGINE] = "Engine";
	ItemNames[ITEM_CLASS.AFTERBURNER] = "Afterburner";
	ItemNames[ITEM_CLASS.CRUISER] = "Cruiser";
	ItemNames[ITEM_CLASS.GENERATOR] = "Generator";
	ItemNames[ITEM_CLASS.TURNER] = "Turning Engine";
	ItemNames[ITEM_CLASS.SENSOR] = "Sensor";
	ItemNames[ITEM_CLASS.CLOAKING] = "Cloaking Device";
	ItemNames[ITEM_CLASS.SHIELDS] = "Shield Generator";
	ItemNames[ITEM_CLASS.STRAFER] = "Strafer Engine";
	ItemNames[ITEM_CLASS.PULSE] = "Plasma Pulse";
	ItemNames[ITEM_CLASS.SNIPER] = "Sniping Cannon";
	ItemNames[ITEM_CLASS.SPREAD] = "Plasma Spread";
	ItemNames[ITEM_CLASS.MISSLE] = "Missle";
	ItemNames[ITEM_CLASS.TORPEDO] = "Torpedo";
	ItemNames[ITEM_CLASS.DEATHRAY] = "Death Ray";
	ItemNames[ITEM_CLASS.MODIFIER] = "Modifier";
	ItemNames[ITEM_CLASS.TURRET] = "Pulse Turret";
	ItemNames[ITEM_CLASS.JUNK] = "Space Junk";
	ItemNames[ITEM_CLASS.VALUABLE] = "Valuable";
	ItemNames[ITEM_CLASS.CONTRABAND] = "Contraband";
	ItemNames[ITEM_CLASS.UNIQUE] = "Unique";

	CargoNames.length = ITEM_CARGO.max+1;
	CargoNames[ITEM_CARGO.PAPER] = "Paper";
	CargoNames[ITEM_CARGO.SCRAP_METAL] = "Scrap Metal";
	CargoNames[ITEM_CARGO.WASTE] = "Space Waste";
	CargoNames[ITEM_CARGO.ROCK] = "Common Rocks";
	CargoNames[ITEM_CARGO.FOOD] = "Food";
	CargoNames[ITEM_CARGO.WATER] = "Water";
	CargoNames[ITEM_CARGO.COPPER] = "Copper";
	CargoNames[ITEM_CARGO.SILVER] = "Silver";
	CargoNames[ITEM_CARGO.GOLD] = "Gold";
	CargoNames[ITEM_CARGO.DIAMOND] = "Diamond";
	CargoNames[ITEM_CARGO.MEDICINE] = "Medicine";
	CargoNames[ITEM_CARGO.FUEL] = "Fuel";
	CargoNames[ITEM_CARGO.IDRUG] = "Illegal Drugs";
	CargoNames[ITEM_CARGO.COUNTERFEIT] = "Counterfeits";
	CargoNames[ITEM_CARGO.WMD] = "Nuclear Weapons";
	CargoNames[ITEM_CARGO.POISON] = "Poisons";
	CargoNames[ITEM_CARGO.DATA_CUBE] = "Data Cube";
	CargoNames[ITEM_CARGO.ORGANISMS] = "Organisms";

	CargoPrices[ITEM_CARGO.PAPER] = 3;
	CargoPrices[ITEM_CARGO.SCRAP_METAL] = 4;
	CargoPrices[ITEM_CARGO.WASTE] = 1;
	CargoPrices[ITEM_CARGO.ROCK] = 2;
	CargoPrices[ITEM_CARGO.FOOD] = 5;
	CargoPrices[ITEM_CARGO.WATER] = 6;
	CargoPrices[ITEM_CARGO.COPPER] = 10;
	CargoPrices[ITEM_CARGO.SILVER] = 13;
	CargoPrices[ITEM_CARGO.GOLD] = 16;
	CargoPrices[ITEM_CARGO.DIAMOND] = 20;
	CargoPrices[ITEM_CARGO.MEDICINE] = 9;
	CargoPrices[ITEM_CARGO.FUEL] = 8;
	CargoPrices[ITEM_CARGO.IDRUG] = 15;
	CargoPrices[ITEM_CARGO.COUNTERFEIT] = 22;
	CargoPrices[ITEM_CARGO.WMD] = 30;
	CargoPrices[ITEM_CARGO.POISON] = 18;
	CargoPrices[ITEM_CARGO.DATA_CUBE] = 14;
	CargoPrices[ITEM_CARGO.ORGANISMS] = 12;
}

bool InitFreeUniverse() {
	debug scope(failure) { writefln("InitFreeUniverse: init.d"); }
	
	char[] useFont = "font/coprgtb.ttf";

	// fullscreen ?
	arc.io.window.open("FreeUniverse", SCREEN_X, SCREEN_Y, true);
	arc.io.log.open(); 
	bigFont = new Font(useFont,12);
	showWaitScreen("Starting FreeUniverse...");
	standardFont = new Font(useFont,8);
	smallFont = new Font(useFont,7);
	largeFont = new Font(useFont,24);
	
	space_view = new Camera;
	space_view.setProjection(-SCREEN_X/2, SCREEN_X/2, SCREEN_Y/2, -SCREEN_Y/2);
	
	interface_view = new Camera;
	interface_view.setProjection(0, SCREEN_X, SCREEN_Y, 0);
	
	arc.io.sound.open();
	arc.io.input.open();
	InitNames();
	InitButtons();
	gameTime = new Time();

	GameObjects.init(); 
	//SpaceObjects = new dlinkedlist!(FU_OBJECT);
	//SpaceParticles = new dlinkedlist!(FU_PARTICLE);
	MainGenerator = new FU_SpaceGenerator();
	space.stars.initStars();
	menu.main_menu.initMainMenu();
	gameState = GAME_STATE.MAIN_MENU;
	
	DISPLAY_HEIGHT[0] = 33;
	DISPLAY_WIDTH[0] = 65;
	DISPLAY_HEIGHT[1] = 72;
	DISPLAY_WIDTH[1] = 158;
	DISPLAY_HEIGHT[2] = 300;
	DISPLAY_WIDTH[2] = 630;
	
	if( fill_TIDs() == false ) {
		writefln("Error loading all image files!");
		return false;
	}
	if( fill_SNDs() == false ) {
		writefln("Error loading all sound files!");
		return false;
	}
	return true;
}

void CloseFreeUniverse() {
	if( MainGenerator !is null ) MainGenerator.saveCurrentCell();
	arc.io.sound.close(); 
	arc.io.log.close(); 
	arc.io.window.close();
}
