﻿module space.generation;

private import classes.fu_object;
private import classes.ship;
private import classes.item;
private import classes.job;

private import init.init;

private import std.math;
private import std.mmfile;
private import std.file;
private import std.stream;
private import std.string;
private import std.random;

static const int CELL_SIZE = 800 * 25;
// FILL_SIZE is radius from center of cell to put things
static const int FILL_SIZE = (CELL_SIZE/2) - MAX_SENSOR_RANGE;

class FU_SpaceGenerator {
private:
	// what cell of the universe are we in now?
	int cellX, cellY;
	// level of cell
	int level;
	// what is the dominating faction of this region?
	int factionIndex;
	// density constants (higher number == less dense)
	// ships are spawned off of stations, planets and transpaths
	int planetCount;
	int stationCount;
	int rockCount;
	int PHCount;
	int stormType, stormTrend;
	short stormAmount;
	double centerPointX = 0, centerPointY = 0;
	// returns cell# given a coordinate
	int getCell(double parm) {
		return cast(int)(floor(parm/CELL_SIZE));
	}
	char[] cellFileName(int X, int Y) {
		if( exists("universe") == false ) mkdir("universe");
		char[] fn = "universe/cell";
		fn ~= std.string.toString(X);
		fn ~= std.string.toString(Y);
		fn ~= ".dat";
		return fn;
	}
	void serializeCurrentCell(Serializer s)
	{
		s.describe(stormType);
		s.describe(factionIndex);
		s.describe(GameObjects);
	}
	void loadCellFile(char[] cellFN) {
		compressFile(cellFN, false);
		
		Serializer s = new Serializer(cellFN, FileMode.In);
		serializeCurrentCell(s);
		delete s;
		
		compressFile(cellFN, true);
	}
	void createNewCell() {
		debug scope(failure) { writefln("createNewCell: generation.d"); }
		FU_OBJECT tpObj;
		int i=0;
		planetCount = random_range(0,4);
		stationCount = random_range(1,5);
		rockCount = random_range(100,300);
		PHCount = random_range(0,2);
		if( rand()%2==0 ) stormType = random_range(1,STORM_TYPE.max);
		else stormType = STORM_TYPE.NULL;
		// first cell is always neutral
		if( cellX != 0 || cellY != 0 ) factionIndex = rand()%FactionNames[].length;
		else {
			if( planetCount == 0 ) planetCount = 1;
			factionIndex = 2;
		}
		// make planets
		for(;i<planetCount;i++) {
			FU_OBJECT newPlanet = new FU_OBJECT(OBJECT_TYPE.PLANET, TEXTURES.PLANET );
			newPlanet.setLevel(level);
			newPlanet.setPosition(getCellAreaX(), getCellAreaY());
			newPlanet.setFaction(factionIndex);
			newPlanet.setMaxArmor(25000 + level * 500);
			newPlanet.setArmor(newPlanet.getMaxArmor());
			GameObjects.add(newPlanet);
		}
		// make stations (must be at least one per cell)
		for(i=0;i<stationCount;i++) {
			FU_OBJECT newStation = new FU_OBJECT(OBJECT_TYPE.STATION, TEXTURES.STATION );
			newStation.setLevel(level);
			newStation.setPosition(getCellAreaX(), getCellAreaY());
			newStation.setFaction(factionIndex);
			newStation.setMaxArmor(17500 + level * 350);
			newStation.setArmor(newStation.getMaxArmor());
			tpObj = newStation;
			GameObjects.add(newStation);
		}
		// make space rocks
		for(i=0;i<rockCount;i++) {
			FU_OBJECT newRock = new FU_OBJECT(OBJECT_TYPE.ROCK, TEXTURES.ROCK );
			newRock.setLevel(level);
			newRock.setPosition(getCellAreaX(), getCellAreaY());
			// make x_speed be rotation speed
			newRock.setRotationSpeed( random_range(-5,5) );
			newRock.setMaxArmor(80 + level * 80);
			newRock.setArmor(newRock.getMaxArmor());
			newRock.addWeight(10);
			newRock.setFaction(factionIndex);
			GameObjects.add(newRock);
		}
		// make 4 telegates in middle edges
		for(i=0;i<4;i++) {
			FU_OBJECT newGate = new FU_OBJECT(OBJECT_TYPE.TELEGATE, TEXTURES.TELEGATE );
			switch( i ) {
			case 0:
				newGate.setPosition(centerPointX - FILL_SIZE, centerPointY);
				newGate.setDestination(centerPointX - CELL_SIZE + FILL_SIZE, centerPointY);
				break;
			case 1:
				newGate.setPosition(centerPointX + FILL_SIZE, centerPointY);
				newGate.setDestination(centerPointX + CELL_SIZE - FILL_SIZE, centerPointY);
				break;
			case 2:
				newGate.setPosition(centerPointX, centerPointY - FILL_SIZE);
				newGate.setDestination(centerPointX, centerPointY - CELL_SIZE + FILL_SIZE);
				break;
			case 3:
				newGate.setPosition(centerPointX, centerPointY + FILL_SIZE);
				newGate.setDestination(centerPointX, centerPointY + CELL_SIZE - FILL_SIZE);
			}
			newGate.setRotationSpeed(-5.0);
			newGate.setLevel(level);
			newGate.setFaction(factionIndex);
			newGate.setMaxArmor(12500 + level * 125);
			newGate.setArmor(newGate.getMaxArmor());
			GameObjects.add(newGate);
		}
		// do we make a porthole?
		for(i=0;i<PHCount;i++) {
			FU_OBJECT newPH = new FU_OBJECT(OBJECT_TYPE.PORTHOLE, TEXTURES.PORTHOLE );
			newPH.setLevel(level);
			newPH.setPosition(getCellAreaX(), getCellAreaY());
			newPH.setRotationSpeed( random_range(-5,5) );
			newPH.setFaction(factionIndex);
			newPH.setDestination(random_range(centerPointX - (CELL_SIZE*4),centerPointX + (CELL_SIZE*4)),
								 random_range(centerPointY - (CELL_SIZE*4),centerPointY + (CELL_SIZE*4)));
			GameObjects.add(newPH);
		}
		// make telepaths
		foreach (inout FU_OBJECT data; GameObjects)
		{
			if( data.getType() == OBJECT_TYPE.STATION ||
				data.getType() == OBJECT_TYPE.PLANET ||
				data.getType() == OBJECT_TYPE.TELEGATE ) 
			{
					// this is something that could be connected
					if( rand()%100 < 65 ) 
					{
						// are they far enough apart?
						if( fabs(data.getX()-tpObj.getX()) +
							fabs(data.getY()-tpObj.getY()) > 800 * 4 ) 
						{
								float angle_path = pointAtAngle(data.getX(),data.getY(),
																tpObj.getX(), tpObj.getY());
								// are these angles far enough apart?
								if( data.isGoodTeleAngle(angle_path) &&
									tpObj.isGoodTeleAngle(angle_path-180) ) 
								{
										// okay, we passed all tests! make the damn thing already
										makeTelePath(data, tpObj, angle_path);
										tpObj = data;
								}
						}
					}
			}
		}
	}
	// make a telepath between objects
	void makeTelePath(FU_OBJECT source, FU_OBJECT destination, float angle_path) {
		// add the angles as teleAngles
		source.addTelepathAngle(angle_path);
		destination.addTelepathAngle(angle_path-180);
		// make first transpath
		int texValue = rand()%GameTextures[TEXTURES.TRANSPATH].length;
		FU_OBJECT teleStart = new FU_OBJECT(OBJECT_TYPE.TRANSPATH, TEXTURES.TRANSPATH, texValue );
		FU_OBJECT teleEnd = new FU_OBJECT(OBJECT_TYPE.TRANSPATH, TEXTURES.TRANSPATH, texValue );
		double placeDist = 300;
		double X_move = moveToValueX(angle_path);
		double Y_move = moveToValueY(angle_path);
		double placeX = source.getX() + X_move * placeDist;
		double placeY = source.getY() + Y_move * placeDist;
		teleStart.setPosition(placeX, placeY);
		teleStart.setRotation(angle_path);
		teleStart.setFaction(factionIndex);
		teleStart.setLevel(level);
		placeX = destination.getX() - X_move * placeDist;
		placeY = destination.getY() - Y_move * placeDist;
		teleEnd.setPosition(placeX, placeY);
		teleEnd.setRotation(angle_path);
		teleStart.setDestination(teleEnd.getX(), teleEnd.getY());
		teleStart.setDestination(0, 0, 1);
		teleEnd.setDestination(teleStart.getX(), teleStart.getY());
		teleEnd.setDestination(0, 0, 1);
		teleEnd.setFaction(factionIndex);
		teleEnd.setLevel(level);
		teleStart.setMaxArmor(10000 + level * 100);
		teleStart.setArmor(teleStart.getMaxArmor());
		teleEnd.setMaxArmor(10000 + level * 100);
		teleEnd.setArmor(teleEnd.getMaxArmor());
		GameObjects.add(teleStart);
		GameObjects.add(teleEnd);
		// add intermediate nodes
		int counter = 0;
		while( placeDist < distance(teleEnd.getX(), teleEnd.getY(), teleStart.getX(), teleStart.getY()) ) {
			placeDist += 25;
			counter++;
			if( counter == 48 ) {
				counter = 0;
				FU_OBJECT teleMiddle = new FU_OBJECT(OBJECT_TYPE.TRANSPATH, TEXTURES.TRANSPATH, texValue );
				placeX = destination.getX() - X_move * placeDist;
				placeY = destination.getY() - Y_move * placeDist;				
				teleMiddle.setPosition(placeX, placeY);
				teleMiddle.setRotation(angle_path);
				teleMiddle.setDestination(teleStart.getX(), teleStart.getY());
				teleMiddle.setDestination(teleEnd.getX(), teleEnd.getY(), 1);
				teleMiddle.setFaction(factionIndex);
				teleMiddle.setLevel(level);
				teleMiddle.setMaxArmor(10000 + level * 100);
				teleMiddle.setArmor(teleMiddle.getMaxArmor());
				GameObjects.add(teleMiddle);
			}
		}
	}
	FU_OBJECT[] carryOver;
public:
	this() { resetGenerator(); }
	void resetGenerator() {
		cellX = 0x8FFFFFFF;
		cellY = 0x8FFFFFFF;
	}
	void trimSpaceObjects(bool dockMode = false) {
		debug scope(failure) { writefln("trimSpaceObjects: generation.d"); }
		bool questObject;
		carryOver.length = 0;

		foreach(inout FU_OBJECT data; GameObjects)
		{
			// make note if this is a questing object
			questObject = false;
			if( data.getType() == OBJECT_TYPE.SHIP ) (cast(FU_SHIP)data).stopSounds();
			if( PlayerShip.getCurrentJob() !is null ) 
			{
				foreach(FU_OBJECTIVE jO; PlayerShip.getCurrentJob().jobObjective) 
				{
					if( jO.isTarget(data) == true ) questObject = true;
				}
			}
			if( data.needsTrim() == false || PlayerShip.getCurrentJob() !is null ) {
				if( dockMode == true ) 
				{
					// do not remove certain things on docking
					if( questObject == true || cast(FU_SHIP)data is PlayerShip ||
						data.getType() != OBJECT_TYPE.ITEM &&
						data.getType() != OBJECT_TYPE.SHIP &&
						data.getType() != OBJECT_TYPE.PROJECTILE ) continue;
				} 
				else if( questObject == true || distance(data.getX(), data.getY(), PlayerShip.getX(), PlayerShip.getY()) < MAX_SENSOR_RANGE ) 
				{
					// save certain things on cell change
					carryOver ~= data;
				}
			}
			GameObjects.remove();
		}

		PlayerShip.setTarget(null);
		std.gc.fullCollect();
	}
	// make sure this 'cell' has stuff in it
	// remember to only generate things in a smaller subsection in the center of the cell
	// so a 'gap' will exist between actual objects (so things on radar won't disappear on change)
	// also generate ships outside of radar view, along transpaths, or on stations / planets
	void generateArea(double X, double Y) {
		debug scope(failure) { writefln("generateArea: generation.d"); }
		// first check if we are in the correct cell
		int desiredCellX = getCell(X);
		int desiredCellY = getCell(Y);
		if( cellX != desiredCellX || cellY != desiredCellY ) {
			// bring up our loading screen
			showWaitScreen("Entering New Sector...");
			// we need to load or generate a new area
			// save our current cell to file if more than 1 object
			saveCurrentCell();
			// empty all space objects except you, and ships near you
			trimSpaceObjects();
			resetStorm();
			// set new location variables
			cellX = desiredCellX;
			cellY = desiredCellY;
			level = std.math.abs( cellX ) + std.math.abs( cellY );
			if( level < 1 ) level = 1;
			centerPointX = cellX * CELL_SIZE + CELL_SIZE/2;
			centerPointY = cellY * CELL_SIZE + CELL_SIZE/2;
			// check if a file exists for this area first
			// if it exists, load file
			// if not, create new cell from scratch
			char[] CFN = cellFileName(desiredCellX, desiredCellY);
			if( exists(CFN) ) loadCellFile(CFN);
			else createNewCell();
			// restore carryOver objects
			foreach(FU_OBJECT addMe; carryOver) GameObjects.add(addMe);
		}
	}
	// save everything (but ships and projectiles) from current cell
	void saveCurrentCell() {
		debug scope(failure) { writefln("saveCurrentCell: generation.d"); }
		// ignore saving if no real objects to save
		if( GameObjects.size < 2 ) return;
		// don't save default cell
		if( cellX == 0x8FFFFFFF && cellY == 0x8FFFFFFF ) return;
		char[] cellFN = cellFileName(cellX,cellY);
		
		Serializer s = new Serializer(cellFN, FileMode.Out);
		serializeCurrentCell(s);
		delete(s);
		
		compressFile(cellFN, true);
	}
	int getCellX() { return cellX; }
	int getCellY() { return cellY; }
	double getCellAreaX() {
		return centerPointX + random_range(-FILL_SIZE, FILL_SIZE);
	}
	double getCellAreaY() {
		return centerPointY + random_range(-FILL_SIZE, FILL_SIZE);
	}
	FU_SHIP spawnShip(FU_OBJECT source = null, bool skipSOAdd = false ) {
		double newX, newY;
		AI_CLASS newAIClass;
		int newType;
		if( source !is null &&
			source.getDockInfo() !is null &&
			source.getDockInfo().underAttack() == true ) {
				newAIClass = AI_CLASS.POLICE;
				newType = SHIP_TYPE.CRUISER;
		} else {
			newAIClass = cast(AI_CLASS) ((rand()%AI_CLASS.max)+1);
			do {
				newType = rand()%(SHIP_TYPE.max+1);
			} while( newType == SHIP_TYPE.SHUTTLE
					 && (newAIClass == AI_CLASS.POLICE || newAIClass == AI_CLASS.MINER || newAIClass == AI_CLASS.KILLER || newAIClass == AI_CLASS.PIRATE ));
			if( newType == SHIP_TYPE.POLICE && newAIClass == AI_CLASS.PIRATE ) newAIClass = AI_CLASS.POLICE;
			if( newType == SHIP_TYPE.PIRATE && newAIClass == AI_CLASS.POLICE ) newAIClass = AI_CLASS.PIRATE;
		}
		FU_SHIP newShip;
		if( source is null ) {
			newX = getCellAreaX();
			newY = getCellAreaY();
			newShip = new FU_SHIP(newType,level,true);
			newShip.setFaction(factionIndex);
		} else {
			newX = source.getX() + random_range(-100,100);
			newY = source.getY() + random_range(-100,100);
			if( source.getType() == OBJECT_TYPE.TELEGATE ) {
				newShip = new FU_SHIP(newType,level+random_range(-1,1),true);
				newShip.setFaction(rand()%FactionNames.length);
			} else {
				newShip = new FU_SHIP(newType,level,true);
				newShip.setFaction(source.getFaction());
			}
		}
		newShip.setRotation(rand()%360);
		newShip.setAIClass(newAIClass);
		newShip.setPosition(newX, newY);
		newShip.setDestination(newX, newY);
		// set 'home' location
		newShip.setDestination(newX, newY, 1);
		newShip.setAIAction(newShip.defaultAction());
		newShip.maxUtils();
		if( skipSOAdd == false ) GameObjects.add(newShip);
		return newShip;
	}
	double getCellUpperX() { return centerPointX - FILL_SIZE; }
	double getCellUpperY() { return centerPointY - FILL_SIZE; }
	double getCellLowerX() { return centerPointX + FILL_SIZE; }
	double getCellLowerY() { return centerPointY + FILL_SIZE; }
	int getFaction() { return factionIndex; }
	// add a rock somewhere randomly away from PlayerShip
	FU_OBJECT spawnRock() {
		double newX, newY;
		do {
			newX = getCellAreaX();
			newY = getCellAreaY();
		} while( PlayerShip !is null && distance(newX, newY, PlayerShip.getX(), PlayerShip.getY() ) < 1500 )
		FU_OBJECT newRock = new FU_OBJECT(OBJECT_TYPE.ROCK, TEXTURES.ROCK );
		newRock.setPosition(newX, newY);
		newRock.setFaction(factionIndex);
		// make x_speed be rotation speed
		newRock.setRotationSpeed( random_range(-5,5) );
		newRock.addWeight(10);
		newRock.setMaxArmor(100 + (level-1) * 10);
		newRock.setArmor(newRock.getMaxArmor());
		GameObjects.add(newRock);
		return newRock;
	}
	FU_OBJECT spawnItem(double parmX, double parmY, FU_ITEM newItem) {
		FU_OBJECT newObj = new FU_OBJECT(OBJECT_TYPE.ITEM, TEXTURES.ITEM);
		newObj.setPosition(parmX, parmY);
		newObj.setRotation(random_range(0,360));
		newObj.getSprite().setSize(17,17);
		newObj.setItem(newItem);
		newObj.setArmor(20*newItem.getVars().quality*newItem.getLevel());
		newObj.setMaxArmor(20*newItem.getLevel());
		newObj.setFaction(newItem.getFaction());
		GameObjects.add(newObj);
		return newObj;
	}
	void getBaseLocation(double *returnX, double *returnY) {

		foreach(inout FU_OBJECT data; GameObjects)
		{
			if( data.getType() == OBJECT_TYPE.STATION ||
				data.getType() == OBJECT_TYPE.PLANET ) {
					*returnX = data.getX();
					*returnY = data.getY();
					GameObjects.restoreAll();
					return;
			}
		}
		*returnX = getCellAreaX();
		*returnY = getCellAreaY();
		return;
	}
	int getStormType() {
		return stormType;
	}
	short getStormAmount() {
		return stormAmount;
	}
	void updateStorm() {
		if( rand()%18 != 0 ) return;
		if( rand()%15 == 0 ) stormTrend = random_range(-4,2);
		stormAmount += random_range(-4 + stormTrend,4 + stormTrend);
		if( stormAmount < 4 ) stormAmount = 4;
		else if( stormAmount > 200 ) stormAmount = 200;
	}
	void resetStorm() {
		stormAmount = 0;
		stormTrend = -4;
	}
	int getLevel() { return level; }
}
