﻿module space.handler;

private import classes.fu_object;
private import classes.item;
private import classes.ship;
private import classes.projectile;
private import space.hud;

private import space.stars;
private import init.init;
private import derelict.sdl.keysym;
private import space.generation;
private import space.storms;

private import std.stream;
private import std.random;

void tryAutoSelect(FU_OBJECT tryObj) {
	debug scope(failure) { writefln("tryAutoSelect: handler.d"); }
	
	if( objectExists(tryObj) == false ) return;
	if( tryObj is PlayerShip ) return;
	if( distance( tryObj.getX(), tryObj.getY(), PlayerShip.getX(), PlayerShip.getY() ) > MAX_TARGETABLE ) return;
	if( tryObj.getType() == OBJECT_TYPE.SHIP && canSee(PlayerShip, cast(FU_SHIP)tryObj) == false ) return;
	float angDis = angleDistance( pointAtAngle( PlayerShip.getX(), PlayerShip.getY(), tryObj.getX(), tryObj.getY() ), 
					PlayerShip.getRotation() );
	if( angDis < nearestSelected && PlayerShip.getTarget() !is tryObj ) {
		nearestSelected = angDis;
		PlayerShip.setTarget(tryObj);
		if( PlayerShip.getAIAction() == AI_ACTION.ATTACKING ) PlayerShip.updateAction(AI_ACTION.WANDERING);
	}
}

void HandleInput() {
	debug scope(failure) { writefln("HandleInput: handler.d"); }
	
	if( arc.io.input.keyHit(SDLK_ESCAPE) ) {
		gameState = GAME_STATE.MAIN_MENU;
		return;
	}	
	if( PlayerShip.getArmor() > 0 ) {
		if( arc.io.input.keyDown(SDLK_DOWN) ) PlayerShip.thrust(ITEM_CLASS.ENGINE,-1);
		else if( arc.io.input.keyDown(SDLK_UP) ) PlayerShip.thrust(ITEM_CLASS.ENGINE,1);
		else if( arc.io.input.keyDown('b') ) PlayerShip.airBrakes();
		
		if( arc.io.input.keyDown(SDLK_RIGHT) ) PlayerShip.thrust(ITEM_CLASS.TURNER,1);
		else if( arc.io.input.keyDown(SDLK_LEFT) ) PlayerShip.thrust(ITEM_CLASS.TURNER,-1);
		
		if( arc.io.input.keyDown(SDLK_SPACE) ) PlayerShip.thrust(ITEM_CLASS.AFTERBURNER,1);
		else if( arc.io.input.keyDown('a') ) PlayerShip.thrust(ITEM_CLASS.STRAFER,1);
		else if( arc.io.input.keyDown('s') ) PlayerShip.thrust(ITEM_CLASS.STRAFER,-1);
		else if( arc.io.input.keyDown('c') ) PlayerShip.thrust(ITEM_CLASS.CRUISER,1);
		
		if( arc.io.input.keyDown('f') ) PlayerShip.fire(WILDCARD,false);
		if( arc.io.input.keyDown('m') ) PlayerShip.fire(WILDCARD,true);
		if( arc.io.input.keyHit('d') ) PlayerShip.dockToTarget();
		if( arc.io.input.keyHit('p') ) PlayerShip.pickupTarget();

		if( arc.io.input.keyHit('l') ) {
			playerUsingCloak = !playerUsingCloak;
			if( playerUsingCloak == true ) space.hud.addMessage("Cloaking device activated.",20,255,20);
			else space.hud.addMessage("Cloaking device deactivated.",20,255,20);
		}

		if( arc.io.input.keyHit('g') == true ) {
			if( objectExists(PlayerShip.getTarget()) == true && PlayerShip.getTarget().getType() == OBJECT_TYPE.SHIP ) {
				switch( (cast(FU_SHIP)PlayerShip.getTarget()).allyRequest(PlayerShip) ) {
				case -1:
					space.hud.addMessage("Group request failed.",255,20,20);
					break;
				case -2:
					space.hud.addMessage("Target is too busy.",255,255,20);
					break;
				case -3:
					space.hud.addMessage("Target faction not compatible.",255,255,20);
					break;
				case -4:
					space.hud.addMessage("Target level too high.",255,255,20);
					break;
				case -5:
					space.hud.addMessage("Target too far away.",255,255,20);
					break;
				case -6:
					space.hud.addMessage("Target already grouped.",20,255,20);
					break;
				case 0:
					space.hud.addMessage("Group request accepted.",20,255,20);
					break;
				}
			} else space.hud.addMessage("Invalid grouping target.",255,255,20);
		}
		
		autoSelection = AUTO_SELECT.NULL;
		if( arc.io.input.keyHit('1') ) { nearestSelected = 361.0; autoSelection = AUTO_SELECT.ALL; }
		else if( arc.io.input.keyHit('2') ) { nearestSelected = 361.0; autoSelection = AUTO_SELECT.SHIP; }
		else if( arc.io.input.keyHit('3') ) { nearestSelected = 361.0; autoSelection = AUTO_SELECT.ENEMY; }
		else if( arc.io.input.keyHit('4') ) { nearestSelected = 361.0; autoSelection = AUTO_SELECT.DOCK; }
		else if( arc.io.input.keyHit('5') ) { nearestSelected = 361.0; autoSelection = AUTO_SELECT.ITEM; }
		else if( arc.io.input.keyHit(SDLK_RETURN) ) PlayerShip.setTarget(PlayerShip);
	}
}

int shipCount, last_shipCount;
void HandleSpaceObjects() {
	debug scope(failure) { writefln("HandleSpaceObjects: handler.d"); }
	
	FU_ITEM playerSensor;
	if( PlayerShip.getArmor() > 0 ) {
		// keep the space view centered on the player ship
		space_view.setPosition(Point(PlayerShip.getX(), PlayerShip.getY()));
		// adjust the zoom of the space view according to ship speed
		// smoothen out the 'wobbles' if travelling at max speed
		if(PlayerShip.getSpeed() >= 0.9 * PlayerShip.getMaxSpeed())
			space_view.setZoom(1. + (0.9 * PlayerShip.getMaxSpeed() / 5.));
		else
			space_view.setZoom(1. + (PlayerShip.getSpeed() / 5.));
		
		MainGenerator.generateArea(PlayerShip.getX(), PlayerShip.getY());
		// find a sensor
		playerSensor = PlayerShip.getItem(ITEM_TYPE.UTILITY, ITEM_CLASS.SENSOR, 0, true, true);
		if( playerSensor !is null && (playerSensor.activateRequest(PlayerShip) == false ||
			PlayerShip.sapModifier(MODIFIER_TYPE.DISABLE_RADAR,playerSensor.getLevel()*0.2) == true))
			playerSensor = null;
	}
	
	space.storms.drawStorm();
	space.stars.updateStars();
	
	// space view
	space_view.process();
	space_view.open();
	
	if( playerNeedsAdd != 0 ) {
		GameObjects.add(PlayerShip);
		if( playerNeedsAdd == 2 ) {
			double newX, newY;
			MainGenerator.getBaseLocation(&newX, &newY);
			PlayerShip.setPosition(newX + 200.0, newY + 200.0);
		}
		playerNeedsAdd = 0;
	}
	
	last_shipCount = shipCount;
	shipCount = 0;
	bool particlesDrawn = false;

	foreach(inout FU_OBJECT data; GameObjects)
	{
		switch(data.getType())
		{
		case OBJECT_TYPE.NULL:
			GameObjects.backupAll();
			GameObjects.remove();
			GameObjects.restoreAll();
			break;
		case OBJECT_TYPE.ROCK:
			(cast(FU_OBJECT)data).handle_object();
			if( autoSelection == AUTO_SELECT.ALL ) tryAutoSelect(data);
			break;
		case OBJECT_TYPE.SHIP:
			if( particlesDrawn == false ) {
				GameObjects.drawParticles(); 
				particlesDrawn = true;
			}
			shipCount++;
			(cast(FU_SHIP)data).handle_object();
			drawSensorObject(playerSensor, data, 20, 120, 255);
			switch( autoSelection ) {
			case AUTO_SELECT.NULL:
			case AUTO_SELECT.DOCK:
			case AUTO_SELECT.ITEM:
				break;
			case AUTO_SELECT.ALL:
			case AUTO_SELECT.SHIP:
				tryAutoSelect(data);
				break;
			case AUTO_SELECT.ENEMY:
				if( (cast(FU_SHIP)data).getTarget() is PlayerShip &&
					(cast(FU_SHIP)data).getAIAction() == AI_ACTION.ATTACKING ) {
						tryAutoSelect(data);
				}
				break;
			}
			break;
		case OBJECT_TYPE.STATION:
			(cast(FU_OBJECT)data).selfRepair(0.25);
			(cast(FU_OBJECT)data).handle_object();
			drawSensorObject(playerSensor, data, 20, 255, 255);
			if( last_shipCount < MAX_SHIPS && rand()%825==0 ) MainGenerator.spawnShip(data);
			if( autoSelection == AUTO_SELECT.ALL || autoSelection == AUTO_SELECT.DOCK )
				tryAutoSelect(data);
			// auto-repair
			break;
		case OBJECT_TYPE.PLANET:
			(cast(FU_OBJECT)data).selfRepair(0.5);
			(cast(FU_OBJECT)data).handle_object();
			drawSensorObject(playerSensor, data, 255, 20, 20);
			if( last_shipCount < MAX_SHIPS && rand()%600==0 ) MainGenerator.spawnShip(data);
			if( autoSelection == AUTO_SELECT.ALL || autoSelection == AUTO_SELECT.DOCK )
				tryAutoSelect(data);
			break;
		case OBJECT_TYPE.TELEGATE:
			(cast(FU_OBJECT)data).handle_object();
			drawSensorObject(playerSensor, data, 255, 20, 255);
			if( last_shipCount < MAX_SHIPS && rand()%825==0 ) MainGenerator.spawnShip(data);
			if( autoSelection == AUTO_SELECT.ALL || autoSelection == AUTO_SELECT.DOCK )
				tryAutoSelect(data);
			break;
		case OBJECT_TYPE.PORTHOLE:
			(cast(FU_OBJECT)data).handle_object();
			if( autoSelection == AUTO_SELECT.ALL || autoSelection == AUTO_SELECT.DOCK )
				tryAutoSelect(data);
			break;
		case OBJECT_TYPE.TRANSPATH:
			(cast(FU_OBJECT)data).handle_object();
			drawSensorObject(playerSensor, data, 255, 255, 20);
			if( autoSelection == AUTO_SELECT.ALL || autoSelection == AUTO_SELECT.DOCK )
				tryAutoSelect(data);
			break;
		case OBJECT_TYPE.PROJECTILE:
			(cast(FU_PROJECTILE)data).handle_object();
			break;
		case OBJECT_TYPE.ITEM:
			(cast(FU_OBJECT)data).handle_object();
			drawSensorObject(playerSensor, data, 20, 255, 20);
			if( autoSelection == AUTO_SELECT.ALL || autoSelection == AUTO_SELECT.ITEM )
				tryAutoSelect(data);
			break;
		}
	}

	space_view.close();
	
	interface_view.process();
	interface_view.open();
	space.hud.drawSpaceHUD(playerSensor);
	interface_view.close();
	
}
